//*****************************************************************************
//*
//*
//*		OsmCtrl.h
//*
//*
//*****************************************************************************
//
//	Copyright Anton Zechner (C) 2014
//
#ifndef		__OSMCTRL_H__
#define 	__OSMCTRL_H__

#include	<AfxWin.h>
#include	<AfxMt.h>
#include	<AZ_Lib.h>
#include	<AZ_Image.h>
#include	"BitmapWnd.h"
#include	"SemaphoreEx.h"
#include	"CoordFunctions.h"

#ifndef		OSM_JOBS
#define 	OSM_JOBS			8						// Maximale Anzahl an Theads fr den Download
#endif
#ifndef		OSM_LAYER
#define 	OSM_LAYER			8						// Maximale Anzahl an berblendbaren Layern
#endif
#ifndef		OSM_PATHS
#define 	OSM_PATHS			8						// Maximale Anzahl an darsellbaren Pfaden
#endif
#ifndef		OSM_ALPHA_ADD
#define 	OSM_ALPHA_ADD		8						// Anzahl an Filterfarben fr Layer
#endif 	
#define 	OSM_SRV_NAME		64						// Maximale Lnge eines Server-Namnes
#define 	OSM_SRV_PATH		96						// Maximale Lnge eines Server-Pfades
#define 	OSM_ZOOM_BASE		30						// Maximale Bre in Bits mit Kachel
#define		OSM_AREA_BITS		8						// Gre einer Kachel in Bit

#define 	OSM_MOVE_L			0x0001					// Verschieben mit der linken    Maustaste
#define 	OSM_MOVE_R			0x0002					// Verschieben mit der rechten   Maustaste
#define 	OSM_MOVE_M			0x0004					// Verschieben mit der mittleren Maustaste
#define 	OSM_MOUSE_ZOOM		0x0008					// Zoomen mit dem Mausrad
#define 	OSM_MOUSE_SCROLL	0x0010					// Scrollen mit dem Mausrad
#define 	OSM_SCROLL_NM		0x0020					// Notifymessage fr's Scrollen
#define 	OSM_CHANGE_NM		0x0040					// Notify Message fr XOR-nderungen
#define 	OSM_MENU			0x0080					// Ein Men im OSM-Fenster anzeigen
#define 	OSM_POINT_CHANGE	0x0100					// Knnen Punkte verndert werden
#define 	OSM_POINT_DELETE	0x0200					// Knnen Punkte gelscht werden

#define 	OSM_NM_SCROLL		0x3000					// Notify Message fr Scrollen
#define 	OSM_NM_RECTDONE		0x3001					// Notify Message fr Rechteckauswahl
#define 	OSM_NM_LINEDONE		0x3002					// Notify Message fr Linie
#define 	OSM_NM_DRAWDONE		0x3003					// Notify Message fr Pfad zeichnen
#define 	OSM_NM_DRAWSTART	0x3004					// Notify Message fr neuen Pfad
#define 	OSM_NM_POINTDONE	0x3005					// Notify Message fr Punkt vershoben
#define 	OSM_NM_CHANGEMAP	0x3006					// Notify Message fr Kartenwechsel
#define 	OSM_NM_CHANGEXOR	0x3007					// Notify Message fr ndern der XOR Eingabe
#define 	OSM_NM_PLACEDONE	0x3008					// Notify Message fr belibiger Punkt vershoben
#define 	OSM_NM_DELDONE		0x3009					// Notify Message fr wenn etwas gelscht wurde

#define 	OSM_ACTION_NONE		0						// Keine Aktion
#define 	OSM_ACTION_MOVE		1						// Die Karte verschieben
#define 	OSM_ACTION_RECT		2						// Rechteckauswahl
#define 	OSM_ACTION_LINE		3						// Linie zeichnen
#define 	OSM_ACTION_DRAW		4						// Einen Pfad zeichnen
#define 	OSM_ACTION_POINT	5						// Einen Punkt verschieben
#define 	OSM_ACTION_PLACE	6						// Punkteingabe mit verschieben


typedef struct
	{
    NMHDR			sHdr;								// Notify-Header
	int 			iMode;								// Cancel oder Modus
	int 			iIndex;								// Pfad Index
	int				iPos1X;								// Pixel-Positionen
	int				iPos1Y;								//		*
	int				iPos2X;								//		*
	int				iPos2Y;								//		*
	double			dPos1X;								// Globale Koordinaten
	double			dPos1Y;								//		*
	double			dPos2X;								//		*
	double			dPos2Y;								//		*
	}OSM_NOTIFY;


typedef	struct
	{
	double			dPosX;								// Globale Koordinate
	double			dPosY;								//		*
	int				iPosX;								// OSM-Koordinate
	int				iPosY;								//		*
	int				iMode;								// Bit0=anzeigen Bit1=Text Bit2..7=Bild
	UINT			uColor;								// Farbe frs Textfeld
	}OSM_WAYPOINT;



//*****************************************************************************
//*
//*		Klasse COsmCtrl
//*
//*****************************************************************************
class COsmCtrl : public CBitmapWnd
{
public:

	typedef			void (_cdecl *DrawCB )(void *pParam,CDC *pDc,PAINTSTRUCT *pPaint);
	typedef			int  (_cdecl *MouseCB)(void *pParam,UINT uType,UINT nFlags,CPoint sPoint);

	COsmCtrl();
   ~COsmCtrl();


	BitmapImage	   *GetImage();

	BOOL			Create  (                DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID=ID_BITMAPWND_DEF);
	BOOL			Create  (                DWORD dwStyle,unsigned	   uId  ,CDialog *pDlg   ,UINT nID=ID_BITMAPWND_DEF);
	BOOL			Create  (                              const RECT &sRect,CWnd *pParentWnd,UINT nID=ID_BITMAPWND_DEF);
	BOOL			Create  (                              unsigned	   uId  ,CDialog *pDlg   ,UINT nID=ID_BITMAPWND_DEF);
	BOOL			CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID=ID_BITMAPWND_DEF);
	BOOL			CreateEx(DWORD dwExStyle,DWORD dwStyle,unsigned	   uId  ,CDialog *pDlg   ,UINT nID=ID_BITMAPWND_DEF);
	BOOL			CreateEx(DWORD dwExStyle,              const RECT &sRect,CWnd *pParentWnd,UINT nID=ID_BITMAPWND_DEF);
	BOOL			CreateEx(DWORD dwExStyle,              unsigned	   uId  ,CDialog *pDlg   ,UINT nID=ID_BITMAPWND_DEF);

	int				StartLine   (int iPosX,int iPosY,int iOffCmd=WM_LBUTTONUP);
	int				StartSelect (int iPosX,int iPosY,int iOffCmd=WM_LBUTTONUP);
	int				StartPlace  (int iPosX,int iPosY,int iOffCmd=WM_LBUTTONUP,int iPath=0);
	int				StartMove   (unsigned uPath,int iIndex,int iOffCmd=WM_LBUTTONUP);
	int				StartDraw   (int      iPath,int iIndex,LPPOINT pPixel,int iOffCmd=WM_LBUTTONUP);
	int				SetDual		(BOOL iMode=1);
	int				GetDual		();
	int 			GetZoom     ();
	int 			SetZoom     (int iZoom);
	void			SetRangeZoom(int iMin,int iMax);
	void			SetRangeX   (int iPosX1,int iPosX2);
	void			SetRangeY   (int iPosY1,int iPosY2);
	void			GetScrollPos(CPoint &sScroll);
	void			GetScrollPos(int &iScrollX,int &iScrollY);
	void			GetScrollPoint(int &iScrollX,int &iScrollY);
	void			GetScrollCoord(double &dScrollX,double &dScrollY);
	void			SetDrawCallback (DrawCB  pProc,void *pParam=0);
	void			SetPostCallback (DrawCB  pProc,void *pParam=0);
	void			SetMouseCallback(MouseCB pProc,void *pParam=0);
	BOOL			SetScrollCoord(double dPosX,double dPosY,BOOL bRedraw=TRUE);
	BOOL			SetScrollPath (unsigned uPath,BOOL bRedraw=TRUE);
	BOOL			SetScrollRect (double dPosX1,double dPosY1,double dPosX2,double dPosY2,BOOL bRedraw=TRUE,int iSizeX=0,int iSizeY=0);
	BOOL			SetScrollPos  (CPoint sScroll     ,BOOL bRedraw=TRUE);
	BOOL			SetScrollPos  (int iPosX,int iPosY,BOOL bRedraw=TRUE);
	BOOL			SetScrollEx   (CPoint sScroll     ,CPoint sPixel          ,BOOL bRedraw=TRUE);
	BOOL			SetScrollEx   (int iPosX,int iPosY,int iPixelX,int iPixelY,BOOL bRedraw=TRUE);
	BOOL			SetServer     (UINT uLayer,LPCTSTR pServerName,LPCTSTR pPath=_T(""),int iPort=0,BOOL bFixed=FALSE,BYTE ucInterface=0,LPCTSTR pInfo=0,int iTimeout=0);
	BOOL			SetServerAlpha(UINT uLayer,int iOffset,int iScale);
	BOOL			SetServer     (const COsmCtrl *pMain);

	UINT			SetWheelMode(UINT uMode);
	UINT			SetAction(unsigned uButton,UINT uAction);
	UINT			GetAction(unsigned uButton);
	UINT			GetAction();
	static	void	SetMenuColor(COLORREF uColor);
	UINT			GetXorRect(int    &iPosX1,int    &iPosY1,int    &iPosX2,int    &iPosY2);
	UINT			GetXorRect(double &dPosX1,double &dPosY1,double &dPosX2,double &dPosY2);
	int				GetMapFrame(int    iPosX,int iPosY,int iZoom,BitmapImage &sImage,int *pProgress=0,int *pBreak=0,int iSizeX=0,int iSizeY=0);
	BOOL			DoCopy(int iMode=0,BitmapImage *pImage=0);
	BOOL			ShowMenu(BOOL bOn);
	LPCTSTR			GetDownloadPath();
	BOOL			CancelAction();
	BOOL			ClearCache();
	BOOL			HasMenu();
	BOOL			Reinit();
	void 			Reload();

	void			PixelToPos  (int    iPixelX,int    iPixelY,LPPOINT pPoint                 );
	void			PixelToPos  (int    iPixelX,int    iPixelY,int    &iPosX  ,int    &iPosY  );
	void			PixelToCoord(int    iPixelX,int    iPixelY,double &dCoordX,double &dCoordY);
	void			PosToPixel  (int    iPosX  ,int    iPosY  ,LPPOINT pPixel                 );
	void			PosToPixel  (int    iPosX  ,int    iPosY  ,int    &iPixelX,int    &iPixelY);
	void			PosToCoord  (int    iPosX  ,int    iPosY  ,double &dCoordX,double &dCoordY);
	void			CoordToPixel(double dCoordX,double dCoordY,LPPOINT pPixel                 );
	void			CoordToPixel(double dCoordX,double dCoordY,int    &iPixelX,int    &iPixelY);
	void			CoordToPos  (double dCoordX,double dCoordY,LPPOINT pPoint                 );
	void			CoordToPos  (double dCoordX,double dCoordY,int    &iPosX  ,int    &iPosY  );
	void			GetViewRect (int   &iPosX1 ,int   &iPosY1 ,int    &iPosX2 ,int    &iPosY2 );
	int 			DistanceToPixel(double dCoordX,double dCoordY,double dDistance,int iMode=3);
	int 			DistanceToPixel(double dCoordX,double dCoordY,double dDeltaX,double dDeltaY);

	void			SetRect		(int    iPosX1,int iPosY1,int    iPosX2,int iPosY2,COLORREF uColor=0);
	void			SetLine		(int    iPosX1,int iPosY1,int    iPosX2,int iPosY2,COLORREF uColor=0);
	void			SetRectOff	();
	void			SetLineOff	();

	LPCTSTR			GetWayPoint (int iIndex,int    &iPosX,int    &iPosY);
	LPCTSTR			GetWayPoint (int iIndex,double &dPosX,double &dPosY);
	BOOL			SetWayPoint (int iIndex,LPCTSTR pName=0,double dPosX=-1000,double dPosY=-1000,int iMode=-1,COLORREF uColor=0xFFFFFFFF      ,BOOL bRedraw=TRUE);
	int				AddWayPoint (           LPCTSTR pName  ,double dPosX      ,double dPosY      ,int iMode= 3,COLORREF uColor=RGB(255,255,255),BOOL bRedraw=TRUE);
	BOOL			DelWayPoint (int iPos,BOOL bRedraw=TRUE);
	void 			DelWayPoints(BOOL bRedraw=TRUE);
	BOOL			SetWayPointImg(UINT uIndex,HBITMAP hBitmap,BYTE bOffX,BYTE bOffY,BOOL bRedraw=TRUE);

	int				AddPathPoint  (unsigned uPath,int     iPosX,int     iPosY,int iIndex=0x7FFFFFFF,BOOL bRedraw=TRUE);
	int				AddPathPointEx(unsigned uPath,double  dPosX,double  dPosY,int iIndex=0x7FFFFFFF,BOOL bRedraw=TRUE);
	int				GetPathPoint  (unsigned uPath,int    &iPosX,int    &iPosY,int iIndex);
	int				GetPathPointEx(unsigned uPath,double &dPosX,double &dPosY,int iIndex);
	BOOL			DelPathPoint  (unsigned uPath,int    iIndex,BOOL bRedraw=TRUE);
	BOOL			DelPath       (unsigned uPath,              BOOL bRedraw=TRUE);
	BOOL			GetPathPen    (unsigned uPath,COLORREF &uColor,unsigned &uWidth);
	BOOL			SetPathPen    (unsigned uPath,COLORREF  uColor,unsigned  uWidth=3,BOOL bRedraw=TRUE);
	BOOL			SetPathMode   (unsigned uPath,unsigned  uMode,BOOL bRedraw=TRUE);
	BOOL			SetPathChanged(unsigned uPath,BOOL bChanged=TRUE);
	BOOL 			HasPathChanged(unsigned uPath);
	int 			GetPathPoints (unsigned uPath);
	BOOL  			SetPathVisible(unsigned uPath,BOOL bShow=TRUE);
	BOOL  			GetPathVisible(unsigned uPath);
	double			GetPathLength (unsigned uPath);

//*****************************************************************************
//*
//*		Variablen
//*
//*****************************************************************************
protected:

	typedef struct
	{
	char 			cPath  [OSM_SRV_PATH];						// Ist der aktuelle Server-Pfad
	char 			cServer[OSM_SRV_NAME];						// Ist der aktuelle Server-Name
	char 			cSuffix[256];								// Ist das Suffix fr den Server
	unsigned 		uTimeout;									// Ist das Timeout fr den Server
	unsigned char	ucFixed;									// Ist dieser im fixen Cache
	unsigned char	ucEnabled;									// Ist dieser Eintrag frei gegeben
	unsigned char	ucInterface;								// Ist das Benutzte Interface
	BYTE volatile	ucTurnId;									// Ist eine vortlaufende ID (0=aus)
	unsigned short 	usTcpPort;									// Ist das aktuelle	Server-Port
	unsigned char	ucAlphaAdd;									// Ist die Anzahl der Filterfarben
	int				iAlphaOff;									// Ist der Offset fr den Alpha-Kanal
	int				iAlphaMul;									// Ist der Multiplikator fr den Alpha-Kanal
	COLORRGB		aAlphaAdd[OSM_ALPHA_ADD];					// Sind die Filterfarben fr Layer
	}ServerEntry;

	typedef struct
	{
	HANDLE		    hThread;									// Thread Handle
	void		   *pThis;										// Zeiger auf die eigene Klasse
	void		   *pStop;										// Ist der Zeiger auf die Shutdown-Klasse
	CSemaphoreEx	sNext;										// Semaphore fr nchsten Job
	UINT volatile	uTurn;										// Ist eine vordlaufende Job-Nummer
	UINT volatile	uIdxX;										// X-Position der Kachel
	UINT volatile	uIdxY;										// Y-Position der Kachel
	BYTE volatile	uZoom;										// Zoom der Kachel
	BYTE volatile	uReady;										// Ist der Thread bereit
	BYTE volatile	uTurnId;									// Ist die ID der Server-Einstellungen
	BYTE volatile	uExtern;									// Ist es ein externer Job
	}JobEntry;

	void		   *m_pOsmMouseParam;							// Maus-Callback Parameter
	MouseCB			m_pOsmMouseCB;								// Maus-Callback
	void		   *m_pOsmDrawParam;							// Draw-Callback Parameter
	DrawCB			m_pOsmDrawCB;								// Draw-Callback
	void		   *m_pOsmPostParam;							// Draw-Callback Parameter
	DrawCB			m_pOsmPostCB;								// Draw-Callback

	int				m_iTempPosX;								// Position der Mitte im virtuellen Fenster mit Zoom OSM_ZOOM_BASE
	int				m_iTempPosY;								//		*
	static int		m_iTempSizeX;								// Gre des virtuellen OSM-Fensters in Pixel
	static int		m_iTempSizeY;								//		*
	static int		m_iTempStepX;								// Gre des virtuellen OSM-Fensters in Kacheln
	static int		m_iTempStepY;								//		*
	LPDWORD			m_pTempStates;								// Zustnde der Kacheln (Bit0..7=Draw 8..15=Jop 16..31=Error)

	int				m_iOsmArea;									// Aktuelle Gre einer Kachel in Pixel
	int				m_iOsmAreaBit;								// Aktuelle Gre einer Kachel in 1<<N Pixel
	int				m_iOsmAreaMask;								// Aktuelle Gre einer Kachel in Pixel-1
	
	int				m_iOsmZoom;									// Ist der aktuelle Zoom-Faktor
	int				m_iOsmZoomMin;								// Ist der minimale Zoom-Faktor
	int				m_iOsmZoomMax;								// Ist der maximale Zoom-Faktor
	int				m_iOsmZoomSize;								// Ist die Weltgre in Kacheln
	int				m_iOsmZoomWitdh;							// Ist die Weltgre in Pixel
	int				m_iOsmZoomMask;								// Ist die Make fr den Zoom
	int				m_iOsmScrollX;								// OSM-Position im der Mitte mit Zoom OSM_ZOOM_BASE
	int				m_iOsmScrollY;								//		*
	int				m_iOsmRangeX1;								// Ist der X-Scroll-Bereich
	int				m_iOsmRangeX2;								//		*
	int				m_iOsmRangeY1;								// Ist der Y-Scroll-Bereich
	int				m_iOsmRangeY2;								//		*

	int				m_iMoveStartX;								// Startkoordinate beim Verschieben
	int				m_iMoveStartY;								//		*
	int				m_iXorMode;									// Aktueller XOR-Modus (0=keiner 1=Rechteck 2=Linie)
	int				m_iXorOn;									// Ist die XOR-Line sichtbar
	int				m_iXorPos1X;								// OSM-XOR-Positionen
	int				m_iXorPos1Y;								//		*
	int				m_iXorPos2X;								//		*
	int				m_iXorPos2Y;								//		*
	int				m_iXorOld1X;								// Vorherige OSM-XOR-Pixelpositionen
	int				m_iXorOld1Y;								//		*
	int				m_iXorOld2X;								//		*
	int				m_iXorOld2Y;								//		*
	CPen			m_sXorPen;									// Zeichenstift fr XOR-Linien

	int				m_iRectColor;								// Ist die Rechteckfarbe (-1=aus)
	int				m_iRectPos1X;								// Rechteck-Position
	int				m_iRectPos1Y;								//		*
	int				m_iRectPos2X;								//		*
	int				m_iRectPos2Y;								//		*

	int				m_iLineColor;								// Ist die Linienfarbe (-1=aus)
	int				m_iLinePos1X;								// Rechteck-Position
	int				m_iLinePos1Y;								//		*
	int				m_iLinePos2X;								//		*
	int				m_iLinePos2Y;								//		*

	int				m_aMouseAction[3];							// Aktion fr die Maustasten
	POINT			m_aActionPt;								// Start Pixel bei Aktion

	JobEntry		m_aJops  [OSM_JOBS];						// Daten fr Thread-Jobs
	ServerEntry		m_aLayers[OSM_LAYER];						// Server-Daten fr die Layer

	CDataField		m_aPaths    [OSM_PATHS];					// Koordinaten der Pfade
	CPen			m_aPathPens	[OSM_PATHS];					// Stifte fr Pfade
	BYTE			m_aPathChg 	[OSM_PATHS];					// Wurde der Pfad verndert
	BYTE			m_aPathShow [OSM_PATHS];					// Sind Pfade sichtbar
	BYTE			m_aPathWidth[OSM_PATHS];					// Breite fr Pfade
	COLORREF		m_aPathColor[OSM_PATHS];					// Farben fr Pfade
	POINT			m_aPathPoint;								// Alte Position eines Punktes
	CSemaphoreEx	m_sPathSem;									// Semaphore fr Pfade
	unsigned		m_uPathValid;								// Welche Pfade sind aktiv
	unsigned		m_uPathMode;								// Welche Pfade sind nderbar
	int				m_iPathSel;									// Welche Pfad  ist gewhlt (-1= keiner)
	int				m_iPathSelPt;								// Welche Punkt ist gewhlt (-1= keiner)
	int				m_iPathMove;								// Auf welchen Pfad  ist die Maus (-1= keiner)
	int				m_iPathMovePt;								// Auf welchen Punkt ist die Maus (-1= keiner)

	CSemaphoreEx	m_sOsmSem;									// Semaphore fr das Fenster
	CSemaphoreEx	m_sOsmExit;									// Semaphore frs Beenden
	CSemaphoreEx	m_sOsmCall;									// Semaphore frs externe Jops
	int	 volatile	m_iJobThreads;								// Threads fr den Download
	int	 volatile	m_iRunning;									// Ist das Fenster aktiv
	BYTE volatile	m_uTurnId;									// Ist die ID der Server-Einstellungen
	BOOL			m_bServerChanged;							// Wurden Server-Einstellungen verndert
	unsigned		m_uJopTurn;									// Vortlaufende Job-Nummer
	unsigned		m_uLayerCount;								// Anzahl der aktiven Layer
	unsigned		m_uLayerMask;								// Welche Layer werden benutzt
	unsigned		m_uNextMode;								// Modus fr das Suchen der nchsten Jobs
	unsigned 		m_uMouseMode;								// Modus fr Mausaktion (0=nichts 1=verschieben 2=Auswahl 3=Linie)
	unsigned		m_uMouseOff;								// Abschalet Message fr Maus-Funktionen
	unsigned 		m_uWndStyle;								// Sind die Fenster-Style-Bits
	unsigned		m_uTickTurn;								// Ticker fr den Timer
	DWORD			m_uWindowThread;							// Thread ID des Fenster Threads
	UINT			m_uWindowDual;								// Die Kacheln im Fenster doppelt so gro zeichnen
	BOOL			m_bWindowOk;								// Ist das Fenster sichtbar
	static LONG		m_lWindowCnt;								// Ist die Anzahl der offenen Fenster


	TCHAR 			m_cDownload[MAX_PATH];						// Ist der Pfad der extern heruntergelden wird
	static UINT		m_uChacheSize;								// Gre des Caches in MB
	static time_t	m_iChacheTime;								// Ist die letzte Prfung des Chaches
	static char 	m_cCachePath[MAX_PATH];						// Ist der Cache-Pfad
	static int		m_iCacheLen;								// Ist die Lnge vom Cache-Pfad
	static LONG		m_lCacheRun;								// Luft der Chache-Thread
	static HBITMAP	m_hMenuBitmap;								// Bitmap fr's Men
	static HDC		m_hMenuDc;									// DC fr's Men
	static UINT		m_uMenuColor;								// Farbe fr's Men
	int				m_iMenuHighLight;							// Welches Men ist aktiv
	static HBITMAP	m_hWayPoint;								// Bitmap fr einen Wegpunkt
	static HDC		m_hWayPointDc;								// DC fr einen Wegpunkt
	CSemaphoreEx	m_sWayPoinSem;								// Semaphore fr Wegpunkte
	CStringChain	m_sWayPoints;								// Liste mit Wegpunkten
	BYTE			m_aWayPointsX [64];							// Offset der Wegpunktbilder
	BYTE			m_aWayPointsY [64];							//		*
	BYTE			m_aWayPointsSX[64];							// Gre  der Wegpunktbilder
	BYTE			m_aWayPointsSY[64];							//		*
	HDC				m_aWayPointDc [64];							// DC's   der Wegpunktbilder
	int				m_iWayPointViews;							// Wie oft ist ein Wegpunkt sichtbar

	//{{AFX_MSG(CBitmapWnd)
	afx_msg LRESULT OnScrollDone(WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnAreaUpdate(WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnInvalidate(WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnReinitWnd (WPARAM wParam,LPARAM lParam);
	afx_msg LRESULT OnNextJob   (WPARAM wParam,LPARAM lParam);
	afx_msg void OnKeyDown(UINT nChar,UINT nRepCnt,UINT nFlags);
	afx_msg void OnSize(UINT nType,int iSizeX,int iSizeY);
	afx_msg void OnTimer(UINT_PTR nIdEvent);
	afx_msg void OnDestroy();
	//}}AFX_MSG

	void					StepToIndex(int iStepX ,int iStepY ,int &iIndexX,int &iIndexY);
	int 					IndexToStep(int iIndexX,int iIndexY,int &iStepX ,int &iStepY );
	void					InvalidateWayPoint(int iIndex,int iRemove=0);
	void					PathInvalidate(unsigned uPath,int iStart,int iCount,LPPOINT pLast=0);
	void					DrawEmpty();
	void					DrawXor(CDC *pDc=NULL);
	void					DrawArea(int iPosX,int iPosY,BitmapImage &sImage,int uLayer=0,int iIdxX=0,int iIdxY=0);
	void					StopThreads();
	void					StartThreads();
	int 					StartNextJop(unsigned uMode);
	int						GetSubTopoArea(char *pCachePath,char *pCacheBase,void *pHttp,unsigned uZoomDiff,unsigned char ucInterface,const char *pSrvPath,unsigned uZoom,unsigned uIdxX,unsigned uIdxY,const char *pSuffix,unsigned uTimeout);
	BOOL					HttpConnect(void *pHttp,const char *pSrvName,unsigned uPort);
	static  int				GetHttpPath(char *pName,unsigned char ucInterface,const char *pSrvPath,unsigned uZoom,unsigned uIdxX,unsigned uIdxY,const char *pSuffix);
	static  void			GrayToAlpha   (BitmapImage &sImage,unsigned  uColors=BGR(255,255,255));
	static  void			DoTranspatent (BitmapImage &sImage,unsigned  uColors,int iDiff=0);
	static  void			DoTranspatent (BitmapImage &sImage,unsigned *pColors,int iCount);
	static  void			MultiplyAlpha (BitmapImage &sImage,int iMul,int iOff=0);
	static  void			HighlightFonts(BitmapImage &sImage);
	static	DWORD WINAPI 	CacheThread   (LPVOID    pParam);
	static	DWORD WINAPI 	DownloadThread(LPVOID    pParam);
			DWORD		  	DownloadThread(JobEntry *pJob);
	static	void _cdecl     DrawCallback (void *pParam,CDC *pDc,PAINTSTRUCT *pPaint);
	static	void _cdecl     MouseCallback(void *pParam,UINT uType,UINT nFlags,CPoint sPoint);
	static	void			SetCacheSize(unsigned uSize);
	static	void			MenuInit(HWND hWnd=0);
	static	void			MenuExit();


	DECLARE_MESSAGE_MAP()


};

//*****************************************************************************
//*
//*		Inline Funktionen
//*
//*****************************************************************************
inline	BOOL	COsmCtrl::CreateEx(DWORD dwExStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID){return CreateEx(dwExStyle,WS_CHILD|WS_VISIBLE,sRect,pParentWnd,nID);}
inline	BOOL	COsmCtrl::CreateEx(DWORD dwExStyle,unsigned	   uId  ,CDialog *pDlg   ,UINT nID){return CreateEx(dwExStyle,WS_CHILD|WS_VISIBLE,uId  ,pDlg      ,nID);}
inline	BOOL	COsmCtrl::Create  (DWORD dwStyle  ,const RECT &sRect,CWnd *pParentWnd,UINT nID){return CreateEx(0        ,dwStyle            ,sRect,pParentWnd,nID);}
inline	BOOL	COsmCtrl::Create  (DWORD dwStyle  ,unsigned	   uId  ,CDialog *pDlg   ,UINT nID){return CreateEx(0        ,dwStyle            ,uId  ,pDlg      ,nID);}
inline	BOOL	COsmCtrl::Create  (                const RECT &sRect,CWnd *pParentWnd,UINT nID){return CreateEx(0        ,WS_CHILD|WS_VISIBLE,sRect,pParentWnd,nID);}
inline	BOOL	COsmCtrl::Create  (                unsigned	   uId  ,CDialog *pDlg   ,UINT nID){return CreateEx(0        ,WS_CHILD|WS_VISIBLE,uId  ,pDlg      ,nID);}
inline	void	COsmCtrl::PosToPixel  (int    iPosX  ,int    iPosY  ,LPPOINT pPixel){PosToPixel  (iPosX  ,iPosY  ,*(int*)&pPixel->x,*(int*)&pPixel->y);}
inline	void	COsmCtrl::PixelToPos  (int    iPixelX,int    iPixelY,LPPOINT pPoint){PixelToPos  (iPixelX,iPixelY,*(int*)&pPoint->x,*(int*)&pPoint->y);}
inline	void	COsmCtrl::CoordToPixel(double dCoordX,double dCoordY,LPPOINT pPixel){CoordToPixel(dCoordX,dCoordY,*(int*)&pPixel->x,*(int*)&pPixel->y);}
inline	void	COsmCtrl::CoordToPos  (double dCoordX,double dCoordY,LPPOINT pPoint){CoordToPos  (dCoordX,dCoordY,*(int*)&pPoint->x,*(int*)&pPoint->y);}
inline	BOOL	COsmCtrl::SetScrollEx (CPoint sScroll,CPoint sPixel,BOOL bRedraw){return SetScrollEx(sScroll.x,sScroll.y,sPixel.x,sPixel.y,bRedraw);}
inline	BOOL	COsmCtrl::SetScrollPos(CPoint sScroll,BOOL bRedraw){return SetScrollPos(sScroll.x,sScroll.y,bRedraw);}
inline	BOOL	COsmCtrl::HasMenu(){return (m_uWndStyle&OSM_MENU)? TRUE:FALSE;}
inline	void 	COsmCtrl::GetScrollCoord(double &dScrollX,double &dScrollY){PosToCoord(m_iOsmScrollX,m_iOsmScrollY,dScrollX,dScrollY);}
inline	void	COsmCtrl::GetScrollPoint(int &iScrollX,int &iScrollY){CBitmapWnd::GetScrollPos(iScrollX,iScrollY);}
inline	void	COsmCtrl::GetScrollPos(int &iScrollX,int &iScrollY){iScrollX=m_iOsmScrollX;iScrollY=m_iOsmScrollY;}
inline	void	COsmCtrl::GetScrollPos(CPoint &sScroll){sScroll.x=m_iOsmScrollX;sScroll.y=m_iOsmScrollY;}
inline	int		COsmCtrl::GetDual(){return m_uWindowDual;}
inline	int 	COsmCtrl::GetZoom(){return m_iOsmZoom;}
inline	UINT	COsmCtrl::GetAction(){return m_uMouseMode;}
inline	int		COsmCtrl::AddPathPointEx(unsigned uPath,double  dPosX,double  dPosY,int iIndex,BOOL bRedraw){int iPosX,iPosY; CoordToPos(dPosX,dPosY,iPosX,iPosY);return AddPathPoint(uPath,iPosX,iPosY,iIndex,bRedraw);}
inline	int		COsmCtrl::GetPathPointEx(unsigned uPath,double &dPosX,double &dPosY,int iIndex             ){int iPosX,iPosY; if(GetPathPoint(uPath,iPosX,iPosY,iIndex)<0)return -1;PosToCoord(iPosX,iPosY,dPosX,dPosY);return iIndex;}
inline	BOOL	COsmCtrl::SetScrollCoord(double dPosX,double dPosY,BOOL bRedraw){int iPosX,iPosY; CoordToPos(dPosX,dPosY,iPosX,iPosY);return SetScrollPos(iPosX,iPosY,bRedraw);}
inline	BOOL 	COsmCtrl::SetPathChanged(unsigned uPath,BOOL bChanged){if(uPath>=OSM_PATHS)return FALSE;BOOL bOld=m_aPathChg[uPath];m_aPathChg[uPath]=bChanged;return bOld;}
inline	BOOL 	COsmCtrl::HasPathChanged(unsigned uPath){if(uPath>=OSM_PATHS)return 0;return m_aPathChg [uPath];}
inline	BOOL  	COsmCtrl::GetPathVisible(unsigned uPath){if(uPath>=OSM_PATHS)return 0;return m_aPathShow[uPath];}
inline	int 	COsmCtrl::GetPathPoints (unsigned uPath){if(uPath>=OSM_PATHS)return 0;return m_aPaths   [uPath].Count();}
inline	LPCTSTR	COsmCtrl::GetDownloadPath(){return m_cDownload;}


#endif
