//*****************************************************************************
//*
//*
//*		OsmCtrl.cpp
//*
//*
//*****************************************************************************
//
//	COsmCtrl ist ein Fenster mit dem Open-Street-Map-Karten angezeigt werden.
//	Copyright Anton Zechner (C) 2014
//
#include	<math.h>
#include	"OsmCtrl.h"
#include	"HttpClient.h"
#include	"AfxUtility.h"
#include	"AfxMacros.h"

#ifdef		_DEBUG
#define new  DEBUG_NEW
#undef		 THIS_FILE
static char	 THIS_FILE[] = __FILE__;
#endif

#define 	 WM_SCROLL_DONE		0x5458
#define 	 WM_AREA_UPDATE		0x5459
#define 	 WM_INVALIDATE		0x545A
#define 	 WM_REINIT_WND		0x545B
#define 	 WM_NEXT_JOB		0x545C
#define 	 ID_TICKER			0x545D
#define		 GRAY_COLOR			0x77665511
#define		 TRANS_COLOR		0x77665522
#define		 ALPHA_COLOR		0x77665533
#define 	 COL_BG				BGR(255,255,255)

#ifndef		 IDC_HAND
#define		 IDC_HAND           MAKEINTRESOURCE(32649)
#endif
#ifndef 	 DC_BRUSH
#define		 DC_BRUSH			18
#endif
#ifndef 	 DC_PEN
#define		 DC_PEN				19
#endif
#ifndef 	 AC_SRC_ALPHA
#define		 AC_SRC_ALPHA       0x01
#endif
#ifndef		 SHOW_INFO
#define		 SHOW_INFO			0
#endif
#ifndef		 OSM_LOG
#define		 OSM_LOG			0
#endif


extern "C"	WINGDIAPI COLORREF WINAPI SetDCPenColor  (HDC,COLORREF);
extern "C"	WINGDIAPI COLORREF WINAPI SetDCBrushColor(HDC,COLORREF);



BEGIN_MESSAGE_MAP(COsmCtrl,CBitmapWnd)
	//{{AFX_MSG_MAP(COsmCtrl)
	ON_WM_SIZE()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_KEYDOWN()
	ON_MESSAGE(WM_NEXT_JOB   ,OnNextJob)
	ON_MESSAGE(WM_REINIT_WND ,OnReinitWnd)
	ON_MESSAGE(WM_INVALIDATE ,OnInvalidate)
	ON_MESSAGE(WM_AREA_UPDATE,OnAreaUpdate)
	ON_MESSAGE(WM_SCROLL_DONE,OnScrollDone)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

#define ENTRIES(a)			(sizeof(a)/sizeof((a)[0]))

#define OSM_TIMEOUT			120000						// Standard-Timeout
#define OSM_TCP_PORT		80							// Standard-TCP-Port
#define OSM_MAX_TURNS		2							// Maximale Versuche beim Download
#define OSM_MAX_CONN		2							// Maximale Versuche beim Verbinden
#define OSM_ZOOM_MIN		0							// Minimaler Zoom
#define OSM_ZOOM_MAX		22							// Maximaler Zoom
#define OSM_BLOCK_AREA		256							// Abmessungen einer Kachel vom Server
#define OSM_MENU_WITDH		36							// Breite des Mens
#define OSM_MENU_HEIGHT		(sizeof(m_aMenuTool)/36)	// Hhe des Mens
#define OSM_AREA_MASK		((1<< OSM_AREA_BITS)-1)		// Maske einer Kachel
#define OSM_AREA			( 1<< OSM_AREA_BITS)		// Gre einer Kachel
#define OSM_ZOOM_ALL		((1<< OSM_ZOOM_BASE)  )		// Steps fr OSM-Koordinaten
#define OSM_ZOOM_MASK		((1<< OSM_ZOOM_BASE)-1)		// Maske fr OSM-Koordinaten
#define OSM_START_X			(34<<(OSM_ZOOM_BASE -6))	// Ist die Startposition frs Fenster
#define OSM_START_Y			(22<<(OSM_ZOOM_BASE -6))	//		*
#define OSM_START_Z			(9)							// Ist der Start-Zoom

#define OSM_PI				3.1415926535897932384626433832795
#define BIT_NOT_CLEAR		0x80000000					// Kachel nicht lschen bei DrawEmpty
#define BIT_TEMP_BIT		0x40000000					// Merkerbit fr Zoom


#define PIN_SIZE_X			(((BITMAPINFOHEADER*)m_aWayPoindDib)->biWidth)
#define PIN_SIZE_Y			(((BITMAPINFOHEADER*)m_aWayPoindDib)->biHeight)
#define PIN_OFF_X			12
#define PIN_OFF_Y			31
#define PATH_FRAME			5
#define WAS_CHANGED			-10000

#if		OSM_LOG
static	HANDLE	hOsmLogSem = 0;							// Semahore zu Loggen
#endif

int		COsmCtrl::m_iTempSizeX	= 0;					// Gre des virtuellen OSM-Fensters in Pixel
int		COsmCtrl::m_iTempSizeY	= 0;					//		*
int		COsmCtrl::m_iTempStepX	= 0;					// Gre des virtuellen OSM-Fensters in Kacheln
int		COsmCtrl::m_iTempStepY	= 0;					//		*
LONG	COsmCtrl::m_lCacheRun	= 0;					// Luft der Chache-Thread
LONG	COsmCtrl::m_lWindowCnt  = 0;					// Ist die anzahl der offenen Fenster
HBITMAP	COsmCtrl::m_hMenuBitmap	= 0;					// Bitmap fr's Men
HDC		COsmCtrl::m_hMenuDc		= 0;					// DC fr's Men
HBITMAP	COsmCtrl::m_hWayPoint	= 0;					// Bitmap fr einen Wegpunkt
HDC		COsmCtrl::m_hWayPointDc = 0;					// DC fr einen Wegpunkt
UINT	COsmCtrl::m_uMenuColor	= BGR(60,0,255);		// Farbe fr's Men
int		COsmCtrl::m_iCacheLen	= 0;					// Ist die Lnge vom Cache-Pfad
time_t	COsmCtrl::m_iChacheTime = 0;					// Ist die letzte Prfung des Chaches
UINT	COsmCtrl::m_uChacheSize	= 64;					// Gre des Caches in MB
char 	COsmCtrl::m_cCachePath[MAX_PATH];				// Ist der Cache-Pfad

static const BYTE	m_aMenuTool[] =
	{
	0x00,0x00,0x00,0x00,0x0F,0x4B,0x92,0xB8,0xBB,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,0xBA,
	0x00,0x00,0x00,0x38,0xAB,0xE2,0xE9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0x00,0x00,0x44,0xDE,0xF2,0xEC,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0x00,0x38,0xE0,0xF0,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0x0F,0xAE,0xF2,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0x48,0xE1,0xEC,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0x90,0xE9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB7,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xED,0xF2,0xF2,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xED,0xF2,0xF2,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xED,0xF3,0xE4,0xAE,0x6B,0x81,0xED,0xEB,0xEB,0xEB,0xEB,0xEE,0xF4,0xDF,0xA6,0x66,0x8A,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEC,0xF3,0xE4,0x94,0x32,0x00,0x19,0xB3,0xEF,0xEB,0xEB,0xEC,0xF3,0xDD,0x85,0x28,0x00,0x07,0xA4,0xF0,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF0,0xEE,0xA7,0x35,0x00,0x00,0x29,0xC5,0xF3,0xEB,0xEB,0xF1,0xEA,0x95,0x27,0x00,0x00,0x19,0xAF,0xF2,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF3,0xCF,0x58,0x03,0x00,0x00,0x33,0xCE,0xF2,0xEB,0xEC,0xF3,0xC0,0x46,0x00,0x00,0x00,0x27,0xC0,0xF4,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEC,0xF1,0xA1,0x1C,0x00,0x00,0x00,0x37,0xD5,0xF1,0xEB,0xEE,0xED,0x89,0x10,0x00,0x00,0x00,0x36,0xCE,0xF2,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEC,0xEE,0x81,0x05,0x00,0x00,0x00,0x2E,0xD5,0xF1,0xEB,0xEC,0xE8,0x64,0x00,0x00,0x00,0x00,0x36,0xD7,0xF1,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xF1,0x92,0x01,0x00,0x00,0x00,0x0D,0xB6,0xF3,0xEB,0xEB,0xF0,0x73,0x00,0x00,0x00,0x00,0x1A,0xCA,0xF2,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEF,0x36,0x00,0x00,0x00,0x00,0x34,0xED,0xEC,0xEB,0xED,0xDF,0x23,0x00,0x00,0x00,0x00,0x4D,0xF5,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEC,0xEB,0x33,0x00,0x00,0x00,0x00,0x3A,0xF2,0xEB,0xEB,0xEE,0xDA,0x22,0x00,0x00,0x00,0x00,0x59,0xF2,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xF1,0x39,0x00,0x00,0x00,0x00,0x36,0xF0,0xEB,0xEB,0xED,0xE3,0x28,0x00,0x00,0x00,0x00,0x4F,0xF5,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xF1,0x94,0x02,0x00,0x00,0x00,0x0B,0xAA,0xF5,0xEB,0xEB,0xF0,0x81,0x00,0x00,0x00,0x00,0x13,0xB8,0xF4,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEC,0xEF,0x88,0x08,0x00,0x00,0x00,0x17,0xB5,0xF4,0xEB,0xEC,0xEC,0x75,0x03,0x00,0x00,0x00,0x1F,0xBD,0xF4,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEC,0xF2,0xAA,0x24,0x00,0x00,0x00,0x13,0xA5,0xF3,0xEC,0xED,0xEF,0x97,0x15,0x00,0x00,0x00,0x18,0xAC,0xF4,0xEC,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF3,0xD6,0x64,0x07,0x00,0x00,0x09,0x90,0xF1,0xED,0xEB,0xF3,0xC7,0x4D,0x00,0x00,0x00,0x0C,0x97,0xF2,0xEC,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEF,0xF1,0xB0,0x3E,0x00,0x00,0x03,0x7C,0xEE,0xEC,0xEB,0xF0,0xEB,0x9A,0x29,0x00,0x00,0x05,0x83,0xEE,0xEC,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0xE8,0x9C,0x38,0x01,0x00,0x75,0xEF,0xEB,0xEB,0xEC,0xF3,0xDD,0x84,0x26,0x00,0x00,0x82,0xF1,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEC,0xF3,0xE8,0xB4,0x71,0x79,0xEB,0xEB,0xEB,0xEB,0xEB,0xEE,0xF4,0xDE,0xA3,0x68,0x8B,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEC,0xF1,0xF2,0xF1,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xED,0xF2,0xF2,0xF0,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEE,0xEF,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEE,0xEC,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEC,0xEF,0xDA,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD3,0xD4,0xE7,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xED,0x7F,0x23,0x1B,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1B,0x1C,0x3D,0xCA,0xEF,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEF,0xD0,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x71,0xF2,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xED,0xE0,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x98,0xF1,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEE,0xCE,0x86,0x78,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x79,0x78,0x7A,0xA4,0xE9,0xEC,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEF,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF1,0xEC,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEC,0xF2,0xF2,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEC,0xE6,0x83,0x5E,0xBE,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x96,0x00,0x00,0x2F,0xE2,0xED,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x72,0x00,0x00,0x17,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xED,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF9,0x77,0x00,0x00,0x19,0xD6,0xF6,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xED,0xE3,0xB8,0xB1,0xB1,0xB1,0xB1,0xB1,0xB1,0xB1,0xB6,0x57,0x00,0x00,0x12,0x9D,0xB4,0xB1,0xB1,0xB1,0xB1,0xB1,0xB1,0xB1,0xCC,0xEE,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEC,0xE8,0x56,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x14,0xB2,0xF1,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEF,0xCE,0x0E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x6B,0xF2,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEC,0xE9,0x58,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x16,0xB3,0xF0,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xED,0xE4,0xB9,0xB2,0xB2,0xB2,0xB2,0xB2,0xB2,0xB2,0xB7,0x57,0x00,0x00,0x12,0x9E,0xB5,0xB2,0xB2,0xB2,0xB2,0xB2,0xB2,0xB2,0xCD,0xEE,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEC,0xF1,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF9,0x76,0x00,0x00,0x19,0xD6,0xF6,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xF2,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB9,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xBA,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x73,0x00,0x00,0x18,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xB1,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x72,0x00,0x00,0x17,0xD0,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0xA4,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xF2,0x97,0x00,0x00,0x2F,0xE2,0xED,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0x70,0xE7,0xEC,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEC,0xE7,0x85,0x61,0xBF,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0x2B,0xD1,0xEE,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEC,0xF2,0xF2,0xEF,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0x01,0x7C,0xF4,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0x00,0x08,0x9E,0xF4,0xEC,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,0xEB,
	0x00,0x00,0x0B,0x95,0xEB,0xF3,0xEF,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,0xED,
	0x00,0x00,0x00,0x03,0x42,0x9A,0xC4,0xD5,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,0xD7,
	};

static const BYTE	m_aWayPoindDib[] =
	{
	0x28,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x01,0x00,0x20,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x03,0x23,0x22,0x23,0x4E,0x0E,0x0E,0x0E,0x18,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,
	0x00,0x00,0x00,0x16,0x1C,0x1B,0x1B,0x68,0x81,0x80,0x80,0xC1,0x05,0x05,0x05,0x0B,
	0x00,0x00,0x00,0x03,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,
	0x00,0x00,0x00,0x05,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x1C,
	0x00,0x00,0x00,0x2F,0x16,0x15,0x15,0x58,0x95,0x93,0x92,0xFF,0x76,0x76,0x76,0x9D,
	0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x1A,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x09,
	0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x13,
	0x00,0x00,0x00,0x1F,0x00,0x00,0x00,0x28,0x00,0x00,0x00,0x2F,0x00,0x00,0x00,0x35,
	0x00,0x00,0x00,0x3A,0x03,0x03,0x03,0x41,0x7B,0x79,0x77,0xED,0xBD,0xBD,0xBD,0xF9,
	0x0E,0x0E,0x0E,0x4E,0x00,0x00,0x00,0x3B,0x00,0x00,0x00,0x36,0x00,0x00,0x00,0x2A,
	0x00,0x00,0x00,0x17,0x00,0x00,0x00,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x1B,0x00,0x00,0x00,0x27,
	0x00,0x00,0x00,0x2E,0x00,0x00,0x00,0x31,0x00,0x00,0x00,0x34,0x00,0x00,0x00,0x36,
	0x00,0x00,0x00,0x37,0x00,0x00,0x00,0x39,0x4B,0x4A,0x49,0x9E,0x9C,0x9A,0x99,0xFF,
	0x4F,0x4F,0x4F,0x9D,0x00,0x00,0x00,0x40,0x00,0x00,0x00,0x41,0x00,0x00,0x00,0x3F,
	0x00,0x00,0x00,0x32,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x25,0x00,0x00,0x00,0x29,
	0x00,0x00,0x00,0x2B,0x00,0x00,0x00,0x2D,0x00,0x00,0x00,0x2E,0x00,0x00,0x00,0x30,
	0x00,0x00,0x00,0x32,0x00,0x00,0x00,0x34,0x10,0x10,0x10,0x49,0x6C,0x6A,0x69,0xFA,
	0x78,0x78,0x78,0xEB,0x03,0x03,0x03,0x3F,0x00,0x00,0x00,0x3C,0x00,0x00,0x00,0x3D,
	0x00,0x01,0x00,0x36,0x0C,0x1A,0x15,0x26,0x0F,0x1D,0x18,0x1D,0x0A,0x16,0x12,0x09,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x19,0x00,0x00,0x00,0x21,0x00,0x00,0x00,0x24,
	0x00,0x00,0x00,0x26,0x00,0x00,0x00,0x27,0x00,0x00,0x00,0x29,0x00,0x00,0x00,0x2B,
	0x00,0x00,0x00,0x2D,0x00,0x00,0x00,0x2E,0x00,0x00,0x00,0x30,0x42,0x41,0x40,0xB6,
	0x58,0x57,0x57,0xFF,0x31,0x4D,0x44,0x94,0x27,0x73,0x5A,0x97,0x4A,0xB4,0x90,0xCC,
	0x75,0xE5,0xBE,0xF3,0x95,0xF2,0xD1,0xFF,0xA9,0xF5,0xD9,0xFF,0x99,0xF3,0xD3,0xFD,
	0x63,0xC7,0xA5,0xC6,0x11,0x35,0x29,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x03,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x1D,
	0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x22,0x00,0x00,0x00,0x24,0x00,0x00,0x00,0x25,
	0x00,0x00,0x00,0x27,0x00,0x00,0x00,0x29,0x00,0x02,0x01,0x2B,0x22,0x5C,0x48,0x90,
	0x3B,0xB9,0x90,0xFF,0x45,0xE2,0xAF,0xFF,0x4E,0xE4,0xB3,0xFF,0x54,0xE6,0xB4,0xFF,
	0x5F,0xE9,0xB9,0xFF,0x6D,0xED,0xBE,0xFF,0x81,0xF1,0xC8,0xFF,0x94,0xF3,0xD1,0xFF,
	0x7A,0xEE,0xC5,0xFF,0x47,0xCF,0xA1,0xD7,0x00,0x03,0x02,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x09,0x00,0x00,0x00,0x14,
	0x00,0x00,0x00,0x1A,0x00,0x00,0x00,0x1D,0x00,0x00,0x00,0x1E,0x00,0x00,0x00,0x20,
	0x00,0x00,0x00,0x22,0x03,0x28,0x1B,0x50,0x1B,0x9D,0x71,0xD0,0x2D,0xCE,0x98,0xFF,
	0x2E,0xD9,0xA1,0xFF,0x34,0xDE,0xA7,0xFF,0x41,0xE2,0xAC,0xFF,0x4D,0xE5,0xB1,0xFF,
	0x5C,0xE8,0xB7,0xFF,0x74,0xED,0xC2,0xFF,0x8A,0xF1,0xCC,0xFF,0xA7,0xF5,0xD9,0xFF,
	0xA0,0xF3,0xD6,0xFF,0x5A,0xE8,0xB7,0xFF,0x0D,0x2D,0x23,0x24,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x0B,
	0x00,0x00,0x00,0x13,0x00,0x00,0x00,0x17,0x00,0x00,0x00,0x19,0x00,0x01,0x01,0x1B,
	0x04,0x52,0x34,0x89,0x1A,0xAB,0x76,0xFB,0x18,0xB5,0x7E,0xFF,0x1B,0xC1,0x89,0xFF,
	0x25,0xD0,0x97,0xFF,0x2F,0xDC,0xA4,0xFF,0x3C,0xE0,0xAA,0xFF,0x48,0xE3,0xAF,0xFF,
	0x5C,0xE8,0xB8,0xFF,0x7B,0xEE,0xC6,0xFF,0x95,0xF2,0xD1,0xFF,0xB7,0xF7,0xE0,0xFF,
	0xB5,0xF6,0xDF,0xFF,0x74,0xED,0xC2,0xFF,0x14,0x39,0x2D,0x37,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,
	0x00,0x00,0x00,0x08,0x00,0x00,0x00,0x0D,0x00,0x02,0x01,0x11,0x05,0x63,0x3F,0xA6,
	0x0F,0x9A,0x65,0xFF,0x08,0x9F,0x67,0xFF,0x0F,0xAB,0x73,0xFF,0x17,0xB8,0x80,0xFF,
	0x20,0xC7,0x8F,0xFF,0x2B,0xD6,0x9E,0xFF,0x36,0xDF,0xA8,0xFF,0x43,0xE2,0xAD,0xFF,
	0x5A,0xE7,0xB7,0xFF,0x7D,0xED,0xC6,0xFF,0x9D,0xF2,0xD5,0xFF,0xC6,0xF8,0xE6,0xFF,
	0xB7,0xF7,0xDF,0xFF,0x84,0xF0,0xC9,0xFF,0x12,0x2B,0x22,0x1C,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x04,0x57,0x38,0x80,0x0E,0x99,0x64,0xFF,
	0x00,0x92,0x59,0xFF,0x04,0x98,0x5F,0xFF,0x0C,0xA3,0x6C,0xFF,0x14,0xB0,0x79,0xFF,
	0x1D,0xBD,0x87,0xFF,0x26,0xCC,0x95,0xFF,0x31,0xDA,0xA3,0xFF,0x3D,0xE0,0xAA,0xFF,
	0x52,0xE4,0xB3,0xFF,0x75,0xEB,0xC3,0xFF,0x9B,0xF2,0xD4,0xFF,0xC5,0xF7,0xE6,0xFF,
	0xA7,0xF5,0xD9,0xFF,0x79,0xD7,0xB5,0xD0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x01,0x16,0x0E,0x16,0x14,0xA1,0x6D,0xF8,0x07,0x98,0x61,0xFF,
	0x00,0x92,0x5A,0xFF,0x01,0x93,0x5B,0xFF,0x08,0x9C,0x65,0xFF,0x11,0xA8,0x71,0xFF,
	0x19,0xB1,0x7D,0xFF,0x20,0xB9,0x85,0xFF,0x27,0xBD,0x8C,0xFF,0x36,0xCB,0x9A,0xFF,
	0x51,0xDB,0xAC,0xFF,0x74,0xE9,0xBF,0xFF,0x78,0xEC,0xC4,0xFF,0x9D,0xF2,0xD5,0xFF,
	0x95,0xF3,0xD1,0xFF,0x33,0x62,0x51,0x54,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x03,0x36,0x23,0x43,0x17,0xA6,0x71,0xFF,0x17,0xA2,0x6E,0xFF,
	0x04,0x96,0x5E,0xFF,0x00,0x92,0x59,0xFF,0x05,0x94,0x5E,0xFF,0x0C,0x95,0x63,0xFF,
	0x12,0x96,0x67,0xFF,0x15,0x9C,0x6D,0xFF,0x21,0xBE,0x89,0xFF,0x3A,0xDE,0xA9,0xFF,
	0x55,0xE7,0xB5,0xFF,0x84,0xF0,0xCA,0xFF,0x8E,0xF1,0xCE,0xFF,0x68,0xEA,0xBD,0xFF,
	0x54,0xAB,0x8C,0xA2,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x01,0x0F,0x0A,0x0D,0x0B,0xA0,0x68,0xED,0x26,0xAA,0x79,0xFF,
	0x21,0xA3,0x72,0xFF,0x07,0x8F,0x5C,0xFF,0x02,0x84,0x52,0xFF,0x08,0x80,0x53,0xFF,
	0x07,0x88,0x57,0xFF,0x08,0x95,0x60,0xFF,0x19,0xB2,0x7D,0xFF,0x2F,0xD5,0x9F,0xFF,
	0x4A,0xE4,0xB0,0xFF,0x6C,0xEB,0xBF,0xFF,0xA1,0xF4,0xD6,0xFF,0x55,0xE0,0xB0,0xEA,
	0x04,0x0B,0x08,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x43,0x2B,0x56,0x08,0xA2,0x68,0xFD,
	0x22,0xA3,0x73,0xFF,0x2C,0x9C,0x72,0xFF,0x18,0x87,0x5D,0xFF,0x07,0x7E,0x51,0xFF,
	0x12,0x9A,0x66,0xFF,0x02,0x8E,0x59,0xFF,0x11,0xA4,0x70,0xFF,0x24,0xC7,0x91,0xFF,
	0x3E,0xE0,0xAB,0xFF,0x59,0xE8,0xB7,0xFF,0x8D,0xF1,0xCE,0xFF,0x82,0xEE,0xC9,0xFF,
	0x10,0x42,0x32,0x3A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x3F,0x28,0x4C,
	0x04,0x93,0x5D,0xE2,0x02,0x91,0x59,0xFF,0x07,0x86,0x55,0xFF,0x06,0x7F,0x51,0xFF,
	0x22,0xA5,0x74,0xFF,0x01,0x8E,0x58,0xFF,0x0A,0x98,0x63,0xFF,0x1B,0xB7,0x82,0xFF,
	0x32,0xD9,0xA3,0xFF,0x4D,0xE5,0xB2,0xFF,0x73,0xED,0xC2,0xFF,0xA0,0xF4,0xD6,0xFF,
	0x36,0xA4,0x7F,0xA8,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x07,0x04,0x05,0x00,0x37,0x22,0x4A,0x00,0x56,0x34,0x83,0x00,0x5A,0x38,0x9C,
	0x13,0x88,0x5C,0xCF,0x13,0x99,0x65,0xFF,0x04,0x90,0x5A,0xFF,0x13,0xA9,0x74,0xFF,
	0x27,0xCC,0x96,0xFF,0x42,0xE2,0xAD,0xFF,0x5E,0xE9,0xB9,0xFF,0x88,0xF1,0xCB,0xFF,
	0x65,0xEA,0xBC,0xFA,0x08,0x22,0x1A,0x1B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x02,0x2D,0x1D,0x34,0x14,0x9E,0x6A,0xFF,0x01,0x8D,0x57,0xFF,0x0C,0x9C,0x67,0xFF,
	0x1E,0xBC,0x87,0xFF,0x34,0xDA,0xA4,0xFF,0x46,0xE2,0xAE,0xFF,0x53,0xE6,0xB4,0xFF,
	0x6D,0xEB,0xBF,0xFF,0x7F,0xE4,0xC1,0xEA,0x73,0xC5,0xA7,0xBF,0x3D,0x7B,0x64,0x76,
	0x09,0x16,0x11,0x11,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x06,0x84,0x55,0xC6,0x01,0x90,0x59,0xFF,0x08,0x96,0x61,0xFF,
	0x1B,0xB7,0x82,0xFF,0x28,0xCC,0x96,0xFF,0x36,0xD9,0xA4,0xFF,0x46,0xE3,0xAE,0xFF,
	0x55,0xE7,0xB5,0xFF,0x67,0xEB,0xBD,0xFF,0x8D,0xF1,0xCE,0xFF,0x99,0xF3,0xD3,0xFF,
	0x64,0xE0,0xB4,0xEB,0x17,0x4A,0x39,0x41,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x04,0x43,0x2C,0x58,0x05,0x97,0x60,0xFF,0x0E,0xA4,0x6D,0xFF,
	0x18,0xB4,0x7E,0xFF,0x23,0xC4,0x8F,0xFF,0x31,0xD5,0x9F,0xFF,0x58,0xE3,0xB6,0xFF,
	0x86,0xEE,0xCA,0xFF,0xA1,0xF3,0xD6,0xFF,0xA9,0xF5,0xD9,0xFF,0x9B,0xF3,0xD3,0xFF,
	0x94,0xF3,0xD0,0xFF,0x77,0xE9,0xC2,0xF5,0x13,0x32,0x28,0x29,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x01,0x30,0x1E,0x3E,0x05,0x96,0x5E,0xFF,0x0B,0x9F,0x68,0xFF,
	0x16,0xAE,0x78,0xFF,0x35,0xC8,0x96,0xFF,0x5C,0xE3,0xB5,0xFF,0x6E,0xEC,0xBF,0xFF,
	0x6B,0xED,0xBE,0xFF,0x6C,0xED,0xBF,0xFF,0x6D,0xEE,0xBF,0xFF,0x6F,0xEE,0xC0,0xFF,
	0x70,0xEE,0xC1,0xFF,0x72,0xEF,0xC1,0xFF,0x4F,0xA4,0x85,0xA5,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x04,0x03,0x04,0x10,0x8D,0x5D,0xD9,0x00,0x92,0x59,0xFF,0x16,0xA3,0x6F,0xFF,
	0x4A,0xD2,0xA3,0xFF,0x5E,0xE9,0xB8,0xFF,0x5E,0xEA,0xB9,0xFF,0x60,0xEA,0xBA,0xFF,
	0x61,0xEA,0xBA,0xFF,0x63,0xEB,0xBB,0xFF,0x64,0xEB,0xBC,0xFF,0x66,0xEC,0xBC,0xFF,
	0x67,0xEC,0xBD,0xFF,0x68,0xEC,0xBD,0xFF,0x4C,0xAA,0x88,0xAD,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x03,0x3D,0x27,0x52,0x14,0x9E,0x6A,0xFF,0x1F,0xAA,0x76,0xFF,0x4E,0xDF,0xAD,0xFF,
	0x52,0xE7,0xB4,0xFF,0x54,0xE7,0xB5,0xFF,0x55,0xE7,0xB5,0xFF,0x57,0xE8,0xB6,0xFF,
	0x58,0xE8,0xB6,0xFF,0x5A,0xE8,0xB7,0xFF,0x5B,0xE9,0xB8,0xFF,0x5C,0xE9,0xB8,0xFF,
	0x5E,0xEA,0xB9,0xFF,0x5F,0xEA,0xB9,0xFD,0x1E,0x49,0x3A,0x3D,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x06,0x64,0x41,0x94,0x23,0xB1,0x7E,0xFF,0x46,0xE1,0xAD,0xFF,0x48,0xE4,0xB0,0xFF,
	0x49,0xE4,0xB0,0xFF,0x4B,0xE5,0xB1,0xFF,0x4C,0xE5,0xB1,0xFF,0x4E,0xE5,0xB2,0xFF,
	0x4F,0xE6,0xB3,0xFF,0x51,0xE6,0xB3,0xFF,0x52,0xE6,0xB4,0xFF,0x53,0xE7,0xB4,0xFF,
	0x54,0xE7,0xB5,0xFE,0x2F,0x7E,0x63,0x6C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x10,0x7D,0x56,0xA6,0x3C,0xDB,0xA7,0xFF,0x3D,0xE1,0xAB,0xFF,0x3F,0xE1,0xAC,0xFF,
	0x40,0xE2,0xAC,0xFF,0x44,0xE2,0xAE,0xFF,0x47,0xE3,0xAF,0xFF,0x49,0xE4,0xB0,0xFF,
	0x48,0xE4,0xB0,0xFF,0x48,0xE4,0xB0,0xFF,0x48,0xE4,0xB0,0xFF,0x47,0xDD,0xAB,0xF0,
	0x20,0x60,0x4A,0x55,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x19,0x73,0x55,0x87,0x32,0xDE,0xA7,0xFF,0x34,0xDF,0xA7,0xFF,0x3C,0xE0,0xAB,0xFF,
	0x47,0xE2,0xB0,0xFF,0x51,0xE3,0xB4,0xFF,0x57,0xE5,0xB6,0xFF,0x57,0xE5,0xB6,0xFF,
	0x50,0xE4,0xB3,0xFF,0x40,0xE2,0xAD,0xFD,0x2E,0xA5,0x7E,0xA7,0x0A,0x23,0x1A,0x1C,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x0A,0x35,0x27,0x2E,0x2A,0xD9,0xA0,0xF9,0x37,0xDC,0xA7,0xFF,0x4B,0xE0,0xB0,0xFF,
	0x5B,0xE3,0xB7,0xFF,0x64,0xE5,0xBB,0xFF,0x5A,0xE4,0xB7,0xFF,0x44,0xDA,0xAA,0xEE,
	0x24,0x95,0x71,0x97,0x0C,0x34,0x27,0x2A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x08,0x31,0x24,0x2C,0x16,0x7A,0x59,0x84,0x1C,0x86,0x63,0x9E,
	0x1E,0x87,0x64,0x97,0x15,0x6F,0x52,0x74,0x0D,0x43,0x32,0x3E,0x01,0x0A,0x07,0x04,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	};



//*****************************************************************************
//*
//*		Konstruktor
//*
//*****************************************************************************
COsmCtrl::COsmCtrl()
{
int		iSize;
int		iPos;



	m_iTempPosX			= OSM_START_X & (((1<<OSM_START_Z)-1)<<(OSM_ZOOM_BASE-OSM_START_Z));
	m_iTempPosY			= OSM_START_Y & (((1<<OSM_START_Z)-1)<<(OSM_ZOOM_BASE-OSM_START_Z));

	m_pOsmMouseParam	= 0;							// Daten fr Maus-Callback
	m_pOsmMouseCB		= 0;							//		*
	m_pOsmDrawParam		= 0;							// Daten fr Draw-Callback
	m_pOsmDrawCB		= 0;							//		*
	m_pOsmPostParam		= 0;							// Daten fr Post-Callback
	m_pOsmPostCB		= 0;							//		*

	m_iOsmArea			= OSM_AREA;						// Aktuelle Gre einer Kachel in Pixel
	m_iOsmAreaBit		= OSM_AREA_BITS;				// Aktuelle Gre einer Kachel in 1<<N Pixel
	m_iOsmAreaMask		= OSM_AREA_MASK;				// Aktuelle Gre einer Kachel in Pixel-1

	m_iOsmScrollX		= OSM_START_X;					// OSM-Position im der Mitte mit Zoom OSM_ZOOM_BASE
	m_iOsmScrollY		= OSM_START_Y;					//		*
	m_iOsmZoom			= OSM_START_Z;					// Ist der aktuelle Zoom-Faktor
	m_iOsmZoomMin		= OSM_ZOOM_MIN;					// Ist der minimale Zoom-Faktor
	m_iOsmZoomMax		= OSM_ZOOM_MAX;					// Ist der minimale Zoom-Faktor
	m_iOsmZoomSize		= 1<<m_iOsmZoom;				// Ist die Weltgre in Kacheln
	m_iOsmZoomMask		= m_iOsmZoomSize-1;				// Ist die Make fr den Zoom
	m_iOsmZoomWitdh		= m_iOsmZoomSize<<m_iOsmAreaBit;// Ist die Weltgre in Pixel
	m_iOsmRangeX1		= 0;							// Ist der X-Scroll-Bereich
	m_iOsmRangeX2		= 0;							//		*
	m_iOsmRangeY1		= 0;							// Ist der Y-Scroll-Bereich
	m_iOsmRangeY2		= OSM_ZOOM_MASK;				//		*

	m_hWnd				= 0;							// Fenster Handle
	m_iJobThreads		= 0;							// Threads fr den Download
	m_iRunning			= 0;							// Ist das Fenster aktiv
	m_uTurnId			= 0;							// Ist die ID der Server-Einstellungen
	m_uJopTurn			= 0;							// Vortlaufende Job-Nummer
	m_uLayerMask		= 0;							// Welche Layer werden benutzt
	m_uLayerCount		= 0;							// Anzahl der aktiven Layer
	m_uWindowThread		= 0;							// Thread ID des Fenster Threads
	m_uWindowDual		= 0;							// Die Kacheln im Fenster doppelt so gro zeichnen
	m_bWindowOk			= 0;							// Ist das Fenster sichtbar
	m_iWayPointViews	= 0;							// Wie oft ist ein Wegpunkt sichtbar

	m_uNextMode			= 0;							// Modus fr das Suchen der nchsten Jobs
	m_uMouseMode		= 0;							// Modus fr Mausaktion (0=nichts 1=verschieben)
	m_uWndStyle			= 0;							// Sind die Fenster-Style-Bits
	m_uTickTurn			= 0;							// Ticker fr den Timer
	m_iXorMode			= 0;							// Aktueller XOR-Modus (0=keiner 1=Rechteck)
	m_iXorOn			= 0;							// Ist die XOR-Line sichtbar
	m_iRectColor		=-1;							// Ist die Rechteckfarbe (-1=aus)
	m_iLineColor		=-1;							// Ist die Linienfarbe (-1=aus)
	m_iPathSel			=-1;							// Welche Pfade ist gewhlt (-1= keiner)
	m_iPathSelPt		=-1;							// Welche Punkt ist gewhlt (-1= keiner)
	m_iPathMove			=-1;							// Auf welchen Pfad  ist die Maus (-1= keiner)
	m_iPathMovePt		=-1;							// Auf welchen Punkt ist die Maus (-1= keiner)
	m_iMenuHighLight	= 0;							// Welches Men ist aktiv
	m_aMouseAction[0]	= OSM_ACTION_NONE;				// Aktionen fr die Maustasten
	m_aMouseAction[1]	= OSM_ACTION_NONE;				// Aktionen fr die Maustasten
	m_aMouseAction[2]	= OSM_ACTION_NONE;				// Aktionen fr die Maustasten
	m_cDownload[0]		= 0;							// Ist der Pfad der extern heruntergelden wird


	m_sWayPoints.DataLength(sizeof(OSM_WAYPOINT));
	memset(m_aPathShow  ,1,sizeof(m_aPathShow  ));		// Sind Pfade sichtbar
	memset(m_aWayPointDc,0,sizeof(m_aWayPointDc));		// DC's der Wegpunktbilder

	m_aPathColor[0]		= 0x00FF0000;					// Farben fr Pfade
	m_aPathColor[1]		= 0x0000FF00;
	m_aPathColor[2]		= 0x000000FF;
	m_aPathColor[3]		= 0x00FF00FF;
	m_aPathColor[4]		= 0x0000FFFF;
	m_aPathColor[5]		= 0x00FFFF00;
	m_aPathColor[6]		= 0x00C04040;
	m_aPathColor[7]		= 0x004040C0;
	m_uPathValid		= 0;							// Welche Pfade sind aktiv
	m_uPathMode			= 0;							// Welche Pfade sind nderbar

	for(iPos=0;iPos<OSM_PATHS;iPos++)
		{
		m_aPathChg  [iPos] =  0;
		m_aPathWidth[iPos] =  3;
		m_aPaths[iPos].Len(sizeof(POINT));
		}


	for(iPos=0;iPos<OSM_LAYER;iPos++)
		{
		m_aLayers[iPos].cServer[0]	= 0;				// Ist der aktuelle Server-Name
		m_aLayers[iPos].cPath[0]	= 0;				// Ist der aktuelle Server-Pfad
		m_aLayers[iPos].usTcpPort	= OSM_TCP_PORT;		// Ist das aktuelle	Server-Port
		m_aLayers[iPos].uTimeout	= OSM_TIMEOUT;		// Ist das Timeout fr den Server
		m_aLayers[iPos].ucEnabled	= 0;				// Ist dieser Eintrag frei gegeben
		m_aLayers[iPos].ucTurnId	= 0;				// Ist eine vortlaufende ID
		m_aLayers[iPos].ucAlphaAdd	= 0;				// Ist die Anzahl der Filterfarben
		}


	for(iPos=0;iPos<OSM_JOBS;iPos++)
		{
		m_aJops[iPos].pStop			= NULL;				// Zeiger auf die Shutdown-Klasse
		m_aJops[iPos].pThis	 		= this;				// Zeiger auf die eigene Klasse
		m_aJops[iPos].uZoom	 		= 0;				// Zoom der Kachel
		m_aJops[iPos].uReady 		= 0;				// Ist der Thread bereit
		m_aJops[iPos].sNext.Lock();						// Semaphore fr nchsten Job sperren
		}


	if(ExistFile("D:\\Temp")==2)						// Temporren Pfad erstellen
		{
		strcpy(m_cCachePath,"D:\\Temp\\OsmCache\\");
		}
	else if(ExistFile("C:\\")==3)
		{
		strcpy(m_cCachePath,"C:\\Temp\\OsmCache\\");
		}
	else{
		iSize = GetTempPath(sizeof(m_cCachePath),m_cCachePath);
		strcpy(m_cCachePath+iSize,"\\OsmCache\\");
		}

	m_sOsmExit.Lock();
	CreatePath(m_cCachePath);							// Cache-Pfad erstellen
	m_iCacheLen = strlen(m_cCachePath);



	if(!m_iTempSizeX)
		{
		iSize			=	GetSystemMetrics(SM_CXSCREEN)/(OSM_BLOCK_AREA/2);
		iSize			=  (iSize+3)&~3;
		m_iTempSizeX	= 	iSize<<m_iOsmAreaBit;

		if(m_iTempSizeX<1024)m_iTempSizeX=1024;

		m_iTempStepX	= m_iTempSizeX>>m_iOsmAreaBit;

		iSize			=	GetSystemMetrics(SM_CYSCREEN)/(OSM_BLOCK_AREA/2);
		iSize			=  (iSize+3)&~3;
		m_iTempSizeY	= 	iSize<<m_iOsmAreaBit;

		if(m_iTempSizeY<1024)m_iTempSizeY=1024;

		m_iTempStepY	= m_iTempSizeY>>m_iOsmAreaBit;
		}


	m_pTempStates	= new DWORD[m_iTempStepX*m_iTempStepY];
	memset(m_pTempStates,0,sizeof(DWORD)*m_iTempStepX*m_iTempStepY);


	SetMinSize(m_iTempSizeX,m_iTempSizeY);


	#if OSM_LOG

	if(!hOsmLogSem)
		{
		hOsmLogSem = CreateSemaphore(NULL,1,0x7FFF,NULL);
		ShowConsole();
		}

	#endif



}

//*****************************************************************************
//*
//*		Destruktor
//*
//*****************************************************************************
COsmCtrl::~COsmCtrl()
{
int		iPos;


	m_iRunning = 0;

	while(m_iJobThreads>0)								// Warten bis alle Threads beendet sind
		{
		StopThreads();
		m_sOsmExit.Lock(100);
		}


	if(!m_lWindowCnt && m_hMenuBitmap)					// Men-Bitmap lschen
		{
		MenuExit();
		}


	if(m_uPathValid)									// Pens fr Pfade lschen
		{
		for(iPos=0;iPos<OSM_PATHS;iPos++,m_uPathValid>>=1)
			{
			if(!(m_uPathValid&1))continue;
			m_aPathPens[iPos].DeleteObject();
			}
		}

	for(iPos=0;iPos<64;iPos++)							// Wegpunktbilder lschen
		{
		if(!m_aWayPointDc[iPos])continue;
		DeleteDC(m_aWayPointDc[iPos]);
		m_aWayPointDc[iPos]=NULL;
		}

	delete m_pTempStates;
	m_pTempStates = 0;


}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
BOOL COsmCtrl::CreateEx(DWORD dwExStyle,DWORD dwStyle,const RECT &sRect,CWnd *pParentWnd,UINT nID)
{
BOOL	bOk;


	m_uWndStyle = dwStyle;

	bOk = CBitmapWnd::CreateEx(dwExStyle,(dwStyle&0xFFFF0000)|CBW_FIXED_SIZE,sRect,pParentWnd,nID);

	if(bOk)
		{
		m_bWindowOk		= 1;
		m_uWindowThread	= GetCurrentThreadId();
		m_sXorPen.CreatePen(PS_SOLID,1,RGB(255,255,0));
		CBitmapWnd::SetMouseCallback(MouseCallback,this);
		CBitmapWnd::SetDrawCallback (DrawCallback ,this);
		StartThreads();
		}


return bOk;
}

//*****************************************************************************
//*
//*		CreateEx
//*
//*****************************************************************************
//	Erzeugt das Fenster ber einem Dialog-Item.
BOOL COsmCtrl::CreateEx(DWORD dwExStyle,DWORD dwStyle,unsigned uId,CDialog *pDlg,UINT nID)
{
BOOL	bOk;


	m_uWndStyle = dwStyle;

	bOk = CBitmapWnd::CreateEx(dwExStyle,(dwStyle&0xFFFF0000)|CBW_FIXED_SIZE,uId,pDlg,nID);

	if(bOk)
		{
		m_bWindowOk		= 1;
		m_uWindowThread	= GetCurrentThreadId();
		m_sXorPen.CreatePen(PS_SOLID,1,RGB(255,255,0));
		CBitmapWnd::SetMouseCallback(MouseCallback,this);
		CBitmapWnd::SetDrawCallback (DrawCallback ,this);
		StartThreads();
		}



return bOk;
}


//*****************************************************************************
//*
//*		OnDestroy
//*
//*****************************************************************************
void COsmCtrl::OnDestroy()
{
int		iPos;


	m_bWindowOk = 0;

	m_sXorPen.DeleteObject();

	while(m_iJobThreads>0)									// Warten bis alle Threads beendet sind
		{
		StopThreads();
		m_sOsmExit.Lock(100);
		}

	for(iPos=0;iPos<OSM_JOBS;iPos++)						// Sicherheitshalber auf Thread-Ende warten
		{
		WaitForSingleObject(m_aJops[iPos].hThread,100);
		CloseHandle(m_aJops[iPos].hThread);
		}

	CBitmapWnd::OnDestroy();


	StopThreads();

}

//*****************************************************************************
//*
//*		OnSize
//*
//*****************************************************************************
void COsmCtrl::OnSize(UINT nType,int iSizeX,int iSizeY)
{
int			iOldX;



	CBitmapWnd::OnSize(nType,iSizeX,iSizeY);

	iOldX = m_iOsmScrollX;
	m_iOsmScrollX++;
	SetScrollPos(iOldX,m_iOsmScrollY,FALSE);

	if(iSizeX && iSizeY)
		{
		m_uNextMode			= 1;
		m_iWayPointViews	= m_iOsmZoomWitdh+m_iWndSizeX+256;
		m_iWayPointViews  >>= m_iOsmZoom+m_iOsmAreaBit;

		if(!StartNextJop(m_uNextMode) && m_uNextMode)
			{
			m_uNextMode = 0;
			StartNextJop(m_uNextMode);
			}
		}


}

//*****************************************************************************
//*
//*		OnTimer
//*
//*****************************************************************************
void COsmCtrl::OnTimer(UINT_PTR nIdEvent)
{
DWORD		dwThread;
HANDLE		hThread;
time_t		iTime;
time_t		iDiff;


	if(nIdEvent==ID_TICKER)
		{
		m_uTickTurn++;

		if(m_uTickTurn%8==0)
			{
			OnNextJob(0,0);
			}

		if(m_uTickTurn%32==0)
			{
			iTime	= time(0);
			iDiff	= iTime-m_iChacheTime;

			if(iDiff>3600 || iDiff<0)
			if(!InterlockedExchange(&m_lCacheRun,1))
				{
				m_iChacheTime = time(0);
				hThread = CreateThread(NULL,0x20000,CacheThread,&m_lWindowCnt,0,&dwThread);
				CloseHandle(hThread);
				}
			}

		return;
		}


	CBitmapWnd::OnTimer(nIdEvent);
}

//*****************************************************************************
//*
//*		OnReinitWnd
//*
//*****************************************************************************
//	Das Fenster muss neu gezeichnet werden
LRESULT COsmCtrl::OnReinitWnd(WPARAM wParam,LPARAM lParam)
{


	m_sOsmSem.Lock();

	m_uNextMode = 1;
	memset(m_pTempStates,0,sizeof(DWORD)*m_iTempStepX*m_iTempStepY);

	m_sOsmSem.Unlock();

	DrawEmpty();
	RedrawWindow();

	if(m_iRunning)										// Jobs wieder starten
		{
		if(!PostMessage(WM_NEXT_JOB,0))
			{
			Sleep(20);
			PostMessage(WM_NEXT_JOB,0);
			}
		}




return 0;
}

//*****************************************************************************
//*
//*		OnAreaUpdate
//*
//*****************************************************************************
//	Eine Kachel wurde neu gezeichnet
LRESULT COsmCtrl::OnAreaUpdate(WPARAM wParam,LPARAM lParam)
{
int		iPosX;
int		iPosY;


	if(IndexToStep((int)wParam,(int)lParam,iPosX,iPosY))
		{
		InvalidateRect(iPosX<<m_iOsmAreaBit,iPosY<<m_iOsmAreaBit,m_iOsmArea,m_iOsmArea);
		}



return 0;
}

//*****************************************************************************
//*
//*		OnKeyDown
//*
//*****************************************************************************
void COsmCtrl::OnKeyDown(UINT nChar,UINT nRepCnt,UINT nFlags)
{
OSM_NOTIFY	sNotify;
int			iPath;
int			iNum;
CWnd	   *pWnd;



	if(nChar==VK_ESCAPE)CancelAction();

	if(nChar==VK_CONTROL)								// Kreuz ausblenden	wenn STRG gedrckt
		{
		if(m_uMouseMode==OSM_ACTION_NONE && m_iPathMove>=0)
			{
			::SetCursor(IDC_ARROW);
			m_iPathMove	  = -1;
			m_iPathMovePt = -1;
			}
		}


	if(nChar==VK_DELETE)
		{												// Einen Punkt lschen
		if((m_uWndStyle&OSM_POINT_DELETE) && m_uMouseMode==OSM_ACTION_NONE)
		if((UINT)m_iPathSel<OSM_PATHS && m_iPathSelPt>=0)
			{
			iPath	= m_iPathSel;
			iNum	= m_iPathSelPt;

			if(m_aPaths[iPath].Count()!=2 || iNum>=2)	// Nur einen Punkt lschen
				{
				if(iNum+1<m_aPaths[iPath].Count())
						m_iPathSelPt = iNum+1;
				else	m_iPathSelPt = iNum-1;

				DelPathPoint(iPath,iNum);

				if(m_iPathSelPt>=0)
					{
					PathInvalidate(m_iPathSel,m_iPathSelPt,1);
					}

				sNotify.iMode = iNum;
				}
			else{										// Ganzen Pfad lschen
				sNotify.iMode = 0x80000000;
				m_iPathSel	  = -1;
				m_iPathSelPt  = -1;
				DelPath(iPath);
				}

			m_aPathChg[iPath] = 1;

			sNotify.sHdr.code		= OSM_NM_DELDONE;
			sNotify.sHdr.hwndFrom	= m_hWnd;
			sNotify.sHdr.idFrom		= GetWindowLong(m_hWnd,GWL_ID);
			sNotify.iIndex			= iPath;
			sNotify.iPos1X			= 0;
			sNotify.iPos1Y			= 0;
			sNotify.iPos2X			= 0;
			sNotify.iPos2Y			= 0;
			sNotify.dPos1X			= 0;
			sNotify.dPos1Y			= 0;
			sNotify.dPos2X			= 0;
			sNotify.dPos2Y			= 0;

			   pWnd = GetParent();
			if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);

			}
		}


	CBitmapWnd::OnKeyDown(nChar,nRepCnt,nFlags);

}

//*****************************************************************************
//*
//*		CancelAction
//*
//*****************************************************************************
//	Bricht die aktuelle Aktion ab.
//	Ergibt TRUE wenn eine Aktion abgebrochen wurde.
BOOL COsmCtrl::CancelAction()
{
CWnd	   *pWnd;
OSM_NOTIFY	sNotify;
LPPOINT		pItems;
POINT		sLast;
int			iPath;
int			iNum;



	if(m_uMouseMode==OSM_ACTION_RECT)
		{
		sNotify.sHdr.code = OSM_NM_RECTDONE;
		}
	else if(m_uMouseMode==OSM_ACTION_LINE)
		{
		sNotify.sHdr.code = OSM_NM_LINEDONE;
		}
	else if(m_uMouseMode==OSM_ACTION_PLACE)
		{
		sNotify.sHdr.code = OSM_NM_PLACEDONE;
		}
	else if(m_uMouseMode==OSM_ACTION_DRAW)
		{
		ReleaseCapture();

		iNum					= m_iPathSelPt;
		iPath					= m_iPathSel;

		if(m_aPaths[iPath].Count()!=2 || iNum>=2)
			{
			m_iPathSelPt		= iNum-1;
			if(m_iPathSelPt<0)m_iPathSelPt=0;
			DelPathPoint(iPath,iNum);
			}
		else{
			m_iPathSel	 = -1;
			m_iPathSelPt = -1;
			DelPath(iPath);
			}

		sNotify.sHdr.code		= OSM_NM_DRAWDONE;
		sNotify.sHdr.hwndFrom	= m_hWnd;
		sNotify.sHdr.idFrom		= GetWindowLong(m_hWnd,GWL_ID);
		sNotify.iMode			= iNum|0x80000000;
		sNotify.iIndex			= iPath;
		sNotify.iPos1X			= 0;
		sNotify.iPos1Y			= 0;
		sNotify.iPos2X			= 0;
		sNotify.iPos2Y			= 0;
		sNotify.dPos1X			= 0;
		sNotify.dPos1Y			= 0;
		sNotify.dPos2X			= 0;
		sNotify.dPos2Y			= 0;

		   pWnd = GetParent();
		if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);

		m_iXorOn		= 0;
		m_iXorMode		= 0;
		m_uMouseMode	= 0;
		}
	else if(m_uMouseMode==OSM_ACTION_POINT)
		{
		ReleaseCapture();

		iNum					= m_iPathSelPt;
		iPath					= m_iPathSel;
		pItems					= (LPPOINT)m_aPaths[iPath].Get(iNum);
		sLast					= *pItems;
		*pItems					= m_aPathPoint;
		sNotify.sHdr.code		= OSM_NM_POINTDONE;
		sNotify.sHdr.hwndFrom	= m_hWnd;
		sNotify.sHdr.idFrom		= GetWindowLong(m_hWnd,GWL_ID);
		sNotify.iMode			= TRUE;
		sNotify.iIndex			= iPath;
		sNotify.iPos1X			= m_aPathPoint.x;
		sNotify.iPos1Y			= m_aPathPoint.y;
		sNotify.iPos2X			= m_aPathPoint.x;
		sNotify.iPos2Y			= m_aPathPoint.y;

		PosToCoord(sNotify.iPos1X,sNotify.iPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
		PosToCoord(sNotify.iPos2X,sNotify.iPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

		   pWnd = GetParent();
		if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);

		m_iXorOn		= 0;
		m_iXorMode		= 0;
		m_uMouseMode	= 0;

		PathInvalidate(iPath,iNum-1,3,&sLast);
		}
	else{
		return FALSE;
		}



	ReleaseCapture();

	if(m_iXorOn)DrawXor();

	m_iXorOn		= 0;
	m_iXorMode		= 0;
	m_uMouseMode	= 0;

	sNotify.sHdr.hwndFrom	= m_hWnd;
	sNotify.sHdr.idFrom		= GetWindowLong(m_hWnd,GWL_ID);
	sNotify.iMode			= TRUE;
	sNotify.iIndex			= 0;
	sNotify.iPos1X			= m_iXorPos1X;
	sNotify.iPos1Y			= m_iXorPos1Y;
	sNotify.iPos2X			= m_iXorPos2X;
	sNotify.iPos2Y			= m_iXorPos2Y;

	PosToCoord(m_iXorPos1X,m_iXorPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
	PosToCoord(m_iXorPos2X,m_iXorPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

	   pWnd = GetParent();
	if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);


return TRUE;
}

//*****************************************************************************
//*
//*		OnNextJob
//*
//*****************************************************************************
//	Ein Job ist fertig, jetz kann der nchste beginnen
LRESULT COsmCtrl::OnNextJob(WPARAM wParam,LPARAM lParam)
{
int		iRet;


	iRet = StartNextJop(m_uNextMode);

	if(!iRet && m_uNextMode)
		{
		m_uNextMode = 0;
		StartNextJop(m_uNextMode);
		}


return 0;
}

//*****************************************************************************
//*
//*		OnScrollDone
//*
//*****************************************************************************
//	Ein Lob ist fertig, jetz kann der nchste beginnen
LRESULT COsmCtrl::OnScrollDone(WPARAM wParam,LPARAM lParam)
{
OSM_NOTIFY	sNotify;
CWnd	   *pWnd;


	sNotify.sHdr.code		= OSM_NM_RECTDONE;
	sNotify.sHdr.hwndFrom	= m_hWnd;
	sNotify.sHdr.idFrom		= GetWindowLong(m_hWnd,GWL_ID);
	sNotify.iMode			= FALSE;
	sNotify.iIndex			= 0;
	sNotify.iIndex			= 0;
	sNotify.iPos1X			= m_iOsmScrollX;
	sNotify.iPos1Y			= m_iOsmScrollY;

	PixelToCoord(m_iOsmScrollX,m_iOsmScrollY,sNotify.dPos1X,sNotify.dPos1Y);

	sNotify.dPos2X			= 0;
	sNotify.dPos2Y			= 0;

	   pWnd = GetParent();
	if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);


return 0;
}

//*****************************************************************************
//*
//*		SetServer
//*
//*****************************************************************************
//	bernimmt die Verbindung zum OSM-Server fr alle Layer von einem anderen Fenster
BOOL COsmCtrl::SetServer(const COsmCtrl *pMain)
{
TCHAR		cInfo[OSM_ALPHA_ADD*12];
unsigned	uLayer;
unsigned	uLen;
unsigned	uPos;


	if(!pMain)return FALSE;

	for(uLayer=0;uLayer<OSM_LAYER;uLayer++)
		{
		uLen = 0;

		if(uLayer)
			{
			for(uPos=0;uPos<pMain->m_aLayers[uLayer].ucAlphaAdd;uPos++)
				{
				uLen += itostr(pMain->m_aLayers[uLayer].aAlphaAdd[uPos],cInfo+uLen,1);
				cInfo[uLen] = ' ';
				uLen++;
				}
			}

		cInfo[uLen] = 0;

		SetServer     (uLayer,pMain->m_aLayers[uLayer].cServer  ,pMain->m_aLayers[uLayer].cPath,pMain->m_aLayers[uLayer].usTcpPort,pMain->m_aLayers[uLayer].ucFixed,pMain->m_aLayers[uLayer].ucInterface,cInfo,pMain->m_aLayers[uLayer].uTimeout);
		SetServerAlpha(uLayer,pMain->m_aLayers[uLayer].iAlphaOff,pMain->m_aLayers[uLayer].iAlphaMul);
		}


return TRUE;
}


//*****************************************************************************
//*
//*		SetServer
//*
//*****************************************************************************
//	Stellt die Verbindung zum OSM-Server fr eine Layer ein.
//	uLayer			: Ist die Layer-Nummer (0..7)
//	pServerName		: Ist der Name oder die IP fr vom Server. Mit NULL
//					  wird der Layer und alle anschlieenden deaktiviert.
//	pPath			: Ist der HTTP Pfad fr den Download
//	iPort			: Ist das TCP-Port vom Servver (-1=default)
//	bFixed			: Sollen die Dateien im fixen Cache gespeichert werden
//	ucInterface		: Benutztes Interface (0=OSM 1=ESRI 2=YANDEX 3=MAPY)
//	pInfo			: Ist der Info Text vom Server 
//						"0x123 0x123..."   fr Filerfarben oder 
//						"GRAY  0x00112233" fr Graustufen-Alpha-Maske (Byte4=MaxZoom)
//						"TRANS 0xDD112233" fr transparente Layer (DD=Diff)
//	iTimeout		: Ist das Timeout in Millisekunden fr (0=default)
//	Ergibt TRUE wenn die Verbindung konfiguriert wurde
BOOL COsmCtrl::SetServer(UINT uLayer,LPCTSTR pServerName,LPCTSTR pPath,int iPort,BOOL bFixed,unsigned char ucInterface,LPCTSTR pInfo,int iTimeout)
{
char		cPath  [OSM_SRV_PATH];
char		cSuffix[256];
BOOL		bChanged;
COLORRGB	uColor;
LPCTSTR 	pName;
int			iLen;
int			iPos;


	if(uLayer>=OSM_LAYER)								// Layer prfen
		{
		return FALSE;
		}


	if(!pServerName || !*pServerName)					// Layer abschalten
		{
		if(uLayer==0)return FALSE;

		if(!m_aLayers[uLayer].ucEnabled)
			{
			return TRUE;
			}

		m_sOsmSem.Lock();

		m_uTurnId++;

		m_aLayers[uLayer].cServer[0]	= 0;
		m_aLayers[uLayer].cPath[0]		= 0;
		m_aLayers[uLayer].ucEnabled		= 0;
		m_aLayers[uLayer].ucTurnId		= 0;
		m_aLayers[uLayer].ucFixed		= 0;
		m_aLayers[uLayer].ucInterface	= 0;
		m_aLayers[uLayer].ucAlphaAdd	= 0;

		bChanged = TRUE;
		}
	else{												// Layer einschalten
		m_sOsmSem.Lock();

														// Neuer Servername
		if(strawcmp(m_aLayers[uLayer].cServer,pServerName,ENTRIES(m_aLayers[uLayer].cServer)))
			{
			strawcpy(m_aLayers[uLayer].cServer,pServerName,ENTRIES(m_aLayers[uLayer].cServer));
			bChanged = TRUE;
			}
		else{
			bChanged = FALSE;
			}

		if(m_aLayers[uLayer].ucFixed!=bFixed)
			{
			m_aLayers[uLayer].ucFixed = bFixed;
			bChanged = TRUE;
			}

		if(m_aLayers[uLayer].ucInterface!=ucInterface)
			{
			m_aLayers[uLayer].ucInterface = ucInterface;
			bChanged = TRUE;
			}

		if(*pPath=='/')pPath++;

		pName = GetFileName(pPath);

		if(pName[0]=='*' && pName-pPath>0)				// Ist ein Suffix angehngt "cycle/*.png?apikey=6170aad10dfd42a38d4d8c709a536f38"
			{
			strawcpy(cSuffix,pName+1,sizeof(cSuffix));
			iLen = pName-pPath-1;
			}
		else if(pName[0]=='?' && pName-pPath>0)			// Sind Parameter angehngt "cycle/?apikey=6170aad10dfd42a38d4d8c709a536f38"
			{
			strcpy  (cSuffix,".png");
			strawcpy(cSuffix+4,pName,sizeof(cSuffix)-4);
			iLen = pName-pPath-1;
			}
		else{											// Keine Parameter angehngt "tiles/1.0.0/osm"
			iLen = _tcslen(pPath);
			strcpy(cSuffix,".png");
			}

		if(iLen)
			{
			strawcpy(cPath+1,pPath,sizeof(cPath)-1,iLen);
			cPath[0]='/';
			}
		else{
			cPath[0] = 0;
			}

		if(strcmp(m_aLayers[uLayer].cPath,cPath))
			{
			strcpy(m_aLayers[uLayer].cPath,cPath);
			bChanged = TRUE;
			}

		if(strcmp(m_aLayers[uLayer].cSuffix,cSuffix))
			{
			strcpy(m_aLayers[uLayer].cSuffix,cSuffix);
			bChanged = TRUE;
			}

		if(pInfo && *pInfo)								// Filterfarbe aus Infos holen
			{
			iPos = 0;
			
			if((*pInfo=='T' || *pInfo=='t') && !_tcsnicmp(pInfo,_T("TRANS"),5) && strwchar(pInfo[5],0x800000E0))
				{
				if(m_aLayers[uLayer].aAlphaAdd[iPos]!=TRANS_COLOR)
					{
					m_aLayers[uLayer].aAlphaAdd[iPos] = TRANS_COLOR;
					bChanged = TRUE;
					}

				pInfo  += 5; 
				iPos   += 1;
				uColor	= strtoi(pInfo,0,&iLen);
				pInfo  += iLen;
														// Farbe und Differnz fr Transparenz-Layer
				if(m_aLayers[uLayer].aAlphaAdd[iPos]!=uColor)
					{
					m_aLayers[uLayer].aAlphaAdd[iPos] = uColor;
					bChanged = TRUE;
					}

				iPos   += 1;
				}

			if((*pInfo=='G' || *pInfo=='g') && !_tcsnicmp(pInfo,_T("GRAY"),4) && strwchar(pInfo[4],0x800000E0))
				{
				if(m_aLayers[uLayer].aAlphaAdd[iPos]!=GRAY_COLOR)
					{
					m_aLayers[uLayer].aAlphaAdd[iPos] = GRAY_COLOR;
					bChanged = TRUE;
					}

				pInfo  += 4; 
				iPos   += 1;
				uColor	= strtoi(pInfo,0,&iLen);
				pInfo  += iLen;
														// Farbe fr Layer
				if(m_aLayers[uLayer].aAlphaAdd[iPos]!=uColor)
					{
					m_aLayers[uLayer].aAlphaAdd[iPos] = uColor;
					bChanged = TRUE;
					}

				iPos   += 1;
				}


			for(;iPos<OSM_ALPHA_ADD;iPos++)
				{
				pInfo += strwlen(pInfo,0x10A0);
				if(!strwchar(*pInfo,0x01))break;

				uColor	= strtoi(pInfo,0,&iLen);
				pInfo  += iLen;

				if(m_aLayers[uLayer].aAlphaAdd[iPos]==uColor)
					{
					continue;
					}

				m_aLayers[uLayer].aAlphaAdd[iPos] = uColor;
				bChanged = TRUE;
				}
			}
		else{
			iPos = 0;
			}

		if(m_aLayers[uLayer].ucAlphaAdd!=iPos)
			{
			m_aLayers[uLayer].ucAlphaAdd = iPos;
			bChanged = TRUE;
			}

		m_aLayers[uLayer].uTimeout		= (iTimeout<=0)? OSM_TIMEOUT :iTimeout;
		m_aLayers[uLayer].usTcpPort		= (iPort   <=0)? OSM_TCP_PORT:iPort;
		m_aLayers[uLayer].ucEnabled		= 1;

		if(bChanged)									// Haben sich die Einstellungen verndert
			{
			m_aLayers[uLayer].ucTurnId = 0;
			}
		}


	if(bChanged)										// Hat sich die Konfiguration verndert
		{
		m_bServerChanged = TRUE;
		}


	m_sOsmSem.Unlock();



return TRUE;
}

//*****************************************************************************
//*
//*		SetCacheSize
//*
//*****************************************************************************
//	Stellt eine neue Cache-Gre ein
//	uSize	: Ist die neue Cache-Gre in MB
void COsmCtrl::SetCacheSize(unsigned uSize)
{

	m_uChacheSize = uSize;

}

//*****************************************************************************
//*
//*		SetServerAlpha
//*
//*****************************************************************************
//	Stellt den Alpha-Kanal fr einen Sub-Layer ein.
//	iOffset		: Ist der Offset fr den Alpha-Kanal
//	iScale		: Ist der Skalierungwert in 1/256 Schritten
//	Ergibt TRUE wenn die Verbindung konfiguriert wurde
BOOL COsmCtrl::SetServerAlpha(UINT uLayer,int iOffset,int iScale)
{
BOOL	bChanged;


	if(uLayer>=OSM_LAYER || uLayer==0)					// Layer prfen
		{
		return FALSE;
		}

	m_sOsmSem.Lock();

	bChanged = FALSE;

	if(m_aLayers[uLayer].iAlphaOff!=iOffset)
		{
		m_aLayers[uLayer].iAlphaOff = iOffset;
		bChanged = TRUE;
		}

	if(m_aLayers[uLayer].iAlphaMul!=iScale)
		{
		m_aLayers[uLayer].iAlphaMul = iScale;
		bChanged = TRUE;
		}

	if(bChanged)										// Hat sich die Konfiguration verndert
		{
		m_bServerChanged = TRUE;
		}

	m_sOsmSem.Unlock();


return TRUE;
}

//*****************************************************************************
//*
//*		MultiplyAlpha
//*
//*****************************************************************************
//	Skkaliert die Alpha-Maske eines Bildes ( Alpha'=(Alpha+Off)*Mul )
//	sImage		: Ist das Bild das korrigiert werden soll (muss 32 Bit sein)
//	iMul		: Ist der Sklierungs-Faktor
//	iOff		: Ist der Sklierungs-Offset
void COsmCtrl::MultiplyAlpha(BitmapImage &sImage,int iMul,int iOff)
{
unsigned char  *pPixels;
unsigned		uValue;
unsigned		uIndex;
int				iAlpha;
int				iOffX;
int				iOffY;


	if(sImage.BitCount()==32)
		{
		for(iOffY=0;iOffY<OSM_BLOCK_AREA;iOffY++)
			{
			pPixels = sImage.GetRow(iOffY)+3;

			for(iOffX=OSM_BLOCK_AREA;iOffX>0;iOffX--,pPixels+=4)
				{
				iAlpha	= *pPixels;
				iAlpha += iOff;
				iAlpha *= iMul;
				iAlpha /= 256;

				if(iAlpha>255)iAlpha=255;
				if(iAlpha<  0)iAlpha=0;

				*pPixels = iAlpha;
				}
			}
		}
	else if(sImage.BitCount()<=8)
		{
		for(uIndex=0;uIndex<sImage.ColorCount();uIndex++)
			{
			uValue  = sImage.GetColor(uIndex);
			iAlpha	= uValue>>24;
			iAlpha += iOff;
			iAlpha *= iMul;
			iAlpha /= 256;

			if(iAlpha>255)iAlpha=255;
			if(iAlpha<  0)iAlpha=0;

			uValue &= 0x00FFFFFF;
			uValue |= iAlpha<<24;
			
			sImage.SetColor(uIndex,uValue);
			}
		}

}

//*****************************************************************************
//*
//*		HighlightFonts
//*
//*****************************************************************************
//	Hebt Beschriftungen in einem Bild hervor.
//	sImage		: Ist das Bild das korrigiert werden soll (muss 32 Bit sein)
void COsmCtrl::HighlightFonts(BitmapImage &sImage)
{
unsigned		uColor;
unsigned		uValue;
int				iOffX;
int				iOffY;
int				iMin;


	for(iOffY=0;iOffY<OSM_BLOCK_AREA;iOffY++)
	for(iOffX=0;iOffX<OSM_BLOCK_AREA;iOffX++)
		{
		uColor = sImage.GetPixel(iOffX,iOffY);
		if(!(uColor&0xFF000000))continue;

		iMin=0;
		uValue = sImage.GetPixel(iOffX+1,iOffY  );if(uValue>0x1F000000)iMin+=BitmapImage::ColorDiff(uColor,uValue);
		uValue = sImage.GetPixel(iOffX-1,iOffY  );if(uValue>0x1F000000)iMin+=BitmapImage::ColorDiff(uColor,uValue);
		uValue = sImage.GetPixel(iOffX  ,iOffY+1);if(uValue>0x1F000000)iMin+=BitmapImage::ColorDiff(uColor,uValue);
		uValue = sImage.GetPixel(iOffX  ,iOffY-1);if(uValue>0x1F000000)iMin+=BitmapImage::ColorDiff(uColor,uValue);

		if(iMin<500)continue;

		sImage.SetPixel(iOffX,iOffY,uColor|0xFF000000);
		}

	for(iOffY=0;iOffY<OSM_BLOCK_AREA;iOffY++)
	for(iOffX=0;iOffX<OSM_BLOCK_AREA;iOffX++)
		{
		   uColor = sImage.GetPixel(iOffX,iOffY);
		if(uColor<0xFF000000)continue;

		uValue = sImage.GetPixel(iOffX+1,iOffY  );if(uValue>0x0F000000 && uValue<0x81000000)sImage.SetPixel(iOffX+1,iOffY  ,uValue+(uValue&0xFF000000));
		uValue = sImage.GetPixel(iOffX-1,iOffY  );if(uValue>0x0F000000 && uValue<0x81000000)sImage.SetPixel(iOffX-1,iOffY  ,uValue+(uValue&0xFF000000));
		uValue = sImage.GetPixel(iOffX  ,iOffY+1);if(uValue>0x0F000000 && uValue<0x81000000)sImage.SetPixel(iOffX  ,iOffY+1,uValue+(uValue&0xFF000000));
		uValue = sImage.GetPixel(iOffX  ,iOffY-1);if(uValue>0x0F000000 && uValue<0x81000000)sImage.SetPixel(iOffX  ,iOffY-1,uValue+(uValue&0xFF000000));
		}

}

//*****************************************************************************
//*
//*		DoTranspatent
//*
//*****************************************************************************
//	Macht Teile eines Bildes transparent.
//	sImage		: Ist das Bild das korrigiert werden soll
//	pColors		: Zeiger auf die Farben
//	iCount		: Ist die Anzahl der Farben
void COsmCtrl::DoTranspatent(BitmapImage &sImage,unsigned *pColors,int iCount)
{
unsigned char  *pPixels;
unsigned		uColor;
int				iOffX;
int				iMin;
int				iVal;
int				iNum;	


	if(sImage.BitCount()!=32)
		{
		sImage.ConvertToRGB(32);
		}


	pPixels = sImage[OSM_BLOCK_AREA-1];

	for(iOffX=OSM_BLOCK_AREA*OSM_BLOCK_AREA;iOffX>0;iOffX--,pPixels+=4)
		{
		uColor	= *(LPDWORD)pPixels;
		iMin	= BitmapImage::ColorDiff(uColor,pColors[0]);

		for(iNum=1;iNum<iCount;iNum++)
			{
			iVal = BitmapImage::ColorDiff(uColor,pColors[iNum]);
			if(iMin>iVal)iMin=iVal;
			}

		   iMin -= 3;
		if(iMin<0)iMin=0;
		if(iMin<8)
				pPixels[3]=iMin*16;
		else	pPixels[3]=127;
		}


}

//*****************************************************************************
//*
//*		DoTranspatent
//*
//*****************************************************************************
//	Macht Teile eines Bildes transparent.
//	sImage		: Ist das Bild das korrigiert werden soll
//	uColor		: Ist die Hintergrundfarbe 
//	iDiff		: Ist die Differenz zwischen den Farben 
void COsmCtrl::DoTranspatent(BitmapImage &sImage,unsigned uColor,int iDiff)
{
unsigned char  *pPixels;
unsigned		uValue;
int				iOffX;
int				iOffY;
int				iMin;
int				iPos;


	sImage.SetUserData(uColor&0x00FFFFFF);


	if(iDiff<=0)											// Nur die Farbmaske hinterlegen			
		{
		return;
		}
	
															
	if(sImage.BitCount()==32)								// 32-Bit-Bild
		{
		pPixels = sImage[OSM_BLOCK_AREA-1];
		uColor &= 0x00FFFFFF;

		for(iPos=OSM_BLOCK_AREA*OSM_BLOCK_AREA;iPos>0;iPos--,pPixels+=4)
			{
			uValue	= *(LPDWORD)pPixels;
			iMin	= BitmapImage::ColorDiff(uColor,uValue);
			
			if(iMin>iDiff)
				{
				pPixels[3] = 0xFF;
				continue;
				}
			
			*(LPDWORD)pPixels = uColor;

			if((iMin*3)/2>iDiff)
				{
				pPixels[3] = 0x80;
				}
			}
		}
	else if(sImage.BitCount()==24)							// 24-Bit-Bild
		{
		pPixels = sImage[OSM_BLOCK_AREA-1];
		uColor &= 0x00FFFFFF;

		for(iPos=OSM_BLOCK_AREA*OSM_BLOCK_AREA;iPos>0;iPos--,pPixels+=3)
			{
			uValue	= *(LPDWORD)pPixels;
			iMin	= BitmapImage::ColorDiff(uColor,uValue);
			
			if(iMin>iDiff || iMin==0)continue;
			
			*(LPDWORD)pPixels &= 0xFF000000;
			*(LPDWORD)pPixels |= uColor;
			}
		}
	else if(sImage.BitCount()>8)							// 12,15,16-Bit-Bild
		{
		uColor &= 0x00FFFFFF;

		for(iOffY=0;iOffY<OSM_BLOCK_AREA;iOffY++)
		for(iOffX=0;iOffX<OSM_BLOCK_AREA;iOffX++)
			{
			uValue	= sImage.GetRGB(iOffX,iOffY);
			iMin	= BitmapImage::ColorDiff(uColor,uValue);
			
			if(iMin>iDiff || iMin==0)continue;
			
			uValue = sImage.RgbToPixel(uColor);
			sImage.SetPixel(iOffX,iOffY,uValue);
			}
		}
	else{													// Bild mit Pallete
		uColor &= 0x00FFFFFF;

		for(iPos=sImage.ColorCount()-1;iPos>=0;iPos--)
			{
			uValue	= sImage.GetColor(iPos);
			iMin	= BitmapImage::ColorDiff(uColor,uValue);
			
			if(iMin>iDiff || iMin==0)
					sImage.SetColor(iPos,uColor|0xFF000000);
			else	sImage.SetColor(iPos,uColor);
			}
		}


}

//*****************************************************************************
//*
//*		GrayToAlpha
//*
//*****************************************************************************
//	Macht ein Graustufenbild zu einer Alpha-Maske.
//	sImage		: Ist das Bild das korrigiert werden soll
//	uColor		: Ist die Hintergrundfarbe 
void COsmCtrl::GrayToAlpha(BitmapImage &sImage,unsigned uColor)
{
unsigned char  *pPixels;
unsigned		uValue;
unsigned		uFlag;
int				iPos;



	if(sImage.BitCount()<32 && sImage.BitCount()>8)			// RGB auf Alpha-Maske
		{
		sImage.ConvertToRGB(32);
		uFlag = 1;
		}
	else{
		uFlag = 0;
		}
	
	uColor &= 0x00FFFFFF;

															
	if(sImage.BitCount()==32)								// 32-Bit-Bild
		{
		pPixels = sImage[OSM_BLOCK_AREA-1];
		uColor &= 0x00FFFFFF;


		if(uFlag==0 && sImage.HasAlphaEx()>=1)				// Ist bereits eine Alphamaske vorhanden
			{
			for(iPos=OSM_BLOCK_AREA*OSM_BLOCK_AREA;iPos>0;iPos--,pPixels+=4)
				{
				uValue	 = (*(LPDWORD)pPixels)&0xFF000000;
				uValue  |= uColor;
				
				*(LPDWORD)pPixels = uValue;
				}
			}
		else{												// Alphamaske aus Blauanteil holen
			for(iPos=OSM_BLOCK_AREA*OSM_BLOCK_AREA;iPos>0;iPos--,pPixels+=4)
				{
				uValue	 = (*(LPDWORD)pPixels)<<24;
				uValue  |= uColor;
				
				*(LPDWORD)pPixels = uValue;
				}
			}
		}
	else{													// Bild mit Pallete
		for(iPos=sImage.ColorCount()-1;iPos>=0;iPos--)
			{
			uValue	= sImage.GetColor(iPos)<<24;
			uValue |= uColor;

			sImage.SetColor(iPos,uValue);
			}
		}


}

//*****************************************************************************
//*
//*		Reinit
//*
//*****************************************************************************
//	Startet das Laden aller Kacheln nach einer Server-nderung neu.
//	Ergibt TRUE wenn sich Server-Einstellungen verndert haben.
BOOL COsmCtrl::Reinit()
{
unsigned	uLayer;


	if(!m_bServerChanged)return FALSE;

	m_sOsmSem.Lock();

		m_uTurnId++;
	if(!m_uTurnId)m_uTurnId++;

	for(uLayer=0;uLayer<8;uLayer++)
		{
		if(!m_aLayers[uLayer].ucEnabled)break;

		m_aLayers[uLayer].ucTurnId = m_uTurnId;
		}

	m_uLayerCount	= uLayer;
	m_uLayerMask	= (1<<uLayer)-1;

	memset(m_pTempStates,0,sizeof(DWORD)*m_iTempStepX*m_iTempStepY);

	m_bServerChanged = 0;

	m_sOsmSem.Unlock();


	if(IsWindow(m_hWnd))
		{
		if(m_iRunning)									// Nachricht zum Neuzeichnen
			{
			if(!PostMessage(WM_REINIT_WND))
				{
				Sleep(20);
				PostMessage(WM_REINIT_WND);
				}
			}
		}


return TRUE;
}

//*****************************************************************************
//*
//*		StepToIndex
//*
//*****************************************************************************
//	Rechnet die Position einer Kachel im virtuellen OSM-Fensters in
//	den Index fr die PNG-Datei um.
//	iStepX/Y	: Position der Kachel
//	iIndexX/Y	: Hier wird der PNG-Index gespeichert
void COsmCtrl::StepToIndex(int iStepX,int iStepY,int &iIndexX,int &iIndexY)
{
unsigned	uShift;


	uShift	= OSM_ZOOM_BASE-m_iOsmZoom;
	iIndexX	= ((m_iTempPosX>>uShift)-m_iTempStepX/2+iStepX)&m_iOsmZoomMask;
	iIndexY	= ((m_iTempPosY>>uShift)-m_iTempStepY/2+iStepY)&m_iOsmZoomMask;

}

//*****************************************************************************
//*
//*		IndexToStep
//*
//*****************************************************************************
//	Berechnet die Position einer Kachel im virtuellen OSM-Fensters aus
//	den Index fr die PNG-Datei um.
//	iIndexX/Y	: Ist der PNG-Index der umgerechnet werden soll
//	iStepX/Y	: Hier wird die Position der Kachel gespeichert
//	Ergibt 1 wenn die Kachel im virtuellen OSM-Fensters enthalten ist.
int COsmCtrl::IndexToStep(int iIndexX,int iIndexY,int &iStepX,int &iStepY)
{
unsigned	uShift;
int			iIdx;
int			iOk;


	iOk		= 1;
	uShift	= OSM_ZOOM_BASE-m_iOsmZoom;


	iIdx	= (iIndexX+m_iTempStepX/2-(m_iTempPosX>>uShift))&m_iOsmZoomMask;

	if(m_iOsmZoomSize<m_iTempStepX)						// Passt die Weltkarte mehrmals ins Fenster
		{
		if(iIdx>=m_iOsmZoomSize)iIdx=iIdx%m_iOsmZoomSize;
		if(iIdx<              0)iIdx=iIdx%m_iOsmZoomSize+m_iOsmZoomSize;
		}

	if((unsigned)iIdx>=(unsigned)m_iTempStepX)
		{
		iOk = 0;
		}

	iStepX	= iIdx;

	iIdx	= (iIndexY+m_iTempStepY/2-(m_iTempPosY>>uShift));

	if((unsigned)iIdx>=(unsigned)m_iTempStepY)
		{
		iOk = 0;
		}

	iStepY  = iIdx;


return iOk;
}

//*****************************************************************************
//*
//*		PixelToPos
//*
//*****************************************************************************
//	Rechnet eine Pixelposition auf der virtuellen OSM-Karte in eine globale
//	OSM-Position bei OSM_ZOOM_BASE um
//	iPixelX/Y		: Ist die Pixelposition
//	iPosX/Y			: Hier wird die OSM-Position gespeichert
void COsmCtrl::PixelToPos(int iPixelX,int iPixelY,int &iPosX,int &iPosY)
{
int		iDiff;


	iDiff	= iPixelX-m_iTempSizeX/2;
	iDiff <<= OSM_ZOOM_BASE-m_iOsmAreaBit-m_iOsmZoom;
	iPosX	= (m_iTempPosX+iDiff)&OSM_ZOOM_MASK;

	iDiff	= iPixelY-m_iTempSizeY/2;
	iDiff <<= OSM_ZOOM_BASE-m_iOsmAreaBit-m_iOsmZoom;
	iPosY	= (m_iTempPosY+iDiff)&OSM_ZOOM_MASK;

}

//*****************************************************************************
//*
//*		PixelToCoord
//*
//*****************************************************************************
//	Rechnet eine Pixelposition auf der virtuellen OSM-Karte in eine globale
//	Koordinate um
//	iPixelX/Y		: Ist die Pixelposition
//	dCoordX/Y		: Hier wird die Koordinate gespeichert
void COsmCtrl::PixelToCoord(int iPixelX,int iPixelY,double &dCoordX,double &dCoordY)
{
int		iDiff;
int		iPosX;
int		iPosY;


	iDiff	= iPixelX-m_iTempSizeX/2;
	iDiff <<= OSM_ZOOM_BASE-m_iOsmAreaBit-m_iOsmZoom;
	iPosX	= (m_iTempPosX+iDiff)&OSM_ZOOM_MASK;
	iPosX  -= OSM_ZOOM_ALL/2;
	dCoordX	= (double)iPosX*(360.0/((double)OSM_ZOOM_ALL));

	iDiff	= iPixelY-m_iTempSizeY/2;
	iDiff <<= OSM_ZOOM_BASE-m_iOsmAreaBit-m_iOsmZoom;
	iPosY	= (m_iTempPosY+iDiff)&OSM_ZOOM_MASK;
	iPosY   = OSM_ZOOM_ALL/2-iPosY;
	dCoordY	= atan(sinh((double)iPosY*(OSM_PI*2.0/((double)OSM_ZOOM_ALL))))*(180.0/OSM_PI);

}



//*****************************************************************************
//*
//*		PixelToPos
//*
//*****************************************************************************
//	Rechnet eine globale OSM-Position bei OSM_ZOOM_BASE in eine Pixelposition
//	auf der virtuellen OSM-Karte um
//	iPosX/Y			: Ist die globale OSM-Position OSM-Position
//	iPixelX/Y		: Hier wird die Pixelposition gespeichert
void COsmCtrl::PosToPixel(int iPosX,int iPosY,int &iPixelX,int &iPixelY)
{
int		iDiff;


	iPosX  &= OSM_ZOOM_MASK;
	iDiff	= iPosX-m_iTempPosX;
	iDiff >>= OSM_ZOOM_BASE-m_iOsmAreaBit-m_iOsmZoom;
	iPixelX	= iDiff+m_iTempSizeX/2;

	iPosY  &= OSM_ZOOM_MASK;
	iDiff	= iPosY-m_iTempPosY;
	iDiff >>= OSM_ZOOM_BASE-m_iOsmAreaBit-m_iOsmZoom;
	iPixelY	= iDiff+m_iTempSizeY/2;

}


//*****************************************************************************
//*
//*		PosToCoord
//*
//*****************************************************************************
//	Rechnet eine globale OSM-Position bei OSM_ZOOM_BASE in eine Koordinate um.
//	iPosX/Y			: Ist die globale OSM-Position OSM-Position
//	iPixelX/Y		: Hier wird die Pixelposition gespeichert
void COsmCtrl::PosToCoord(int iPosX,int iPosY,double &dCoordX,double &dCoordY)
{


	iPosX  &= OSM_ZOOM_MASK;
	iPosX  -= OSM_ZOOM_ALL/2;
	dCoordX	= (double)iPosX*(360.0/((double)OSM_ZOOM_ALL));

	iPosY  &= OSM_ZOOM_MASK;
	iPosY   = OSM_ZOOM_ALL/2-iPosY;
	dCoordY	= atan(sinh((double)iPosY*(OSM_PI*2.0/((double)OSM_ZOOM_ALL))))*(180.0/OSM_PI);

}

//*****************************************************************************
//*
//*		CoordToPixel
//*
//*****************************************************************************
//	Rechnet eine globale Koordinate in eine Pixelposition
//	auf der virtuellen OSM-Karte um
//	iPosX/Y			: Ist die globale OSM-Position OSM-Position
//	iPixelX/Y		: Hier wird die Pixelposition gespeichert
void COsmCtrl::CoordToPixel(double dCoordX,double dCoordY,int &iPixelX,int &iPixelY)
{
double	dValue;
int		iDiff;
int		iPosX;
int		iPosY;


	iPosX	= (int)(dCoordX*(((double)OSM_ZOOM_ALL)/360.0));
	iPosX  += OSM_ZOOM_ALL/2;
	iPosX  &= OSM_ZOOM_MASK;
	iDiff	= iPosX-m_iTempPosX;
	iDiff >>= OSM_ZOOM_BASE-m_iOsmAreaBit-m_iOsmZoom;
	iPixelX	= iDiff+m_iTempSizeX/2;



	if(dCoordY>85.051128779806589)
		{
		iPosY	= -1;
		}
	else if(dCoordY<-85.051128721960239)
		{
		iPosY	= OSM_ZOOM_MASK;
		}
	else{
  		dValue	= tan(dCoordY*(OSM_PI/180.0));
		dValue	= log(dValue+sqrt(dValue*dValue+1.0));				// = asinh(dVal)
		iPosY	= (int)(dValue*(((double)OSM_ZOOM_ALL)/OSM_PI/2.0));
		iPosY   = OSM_ZOOM_ALL/2-iPosY;
		iPosY  &= OSM_ZOOM_MASK;
		}

	iDiff	= iPosY-m_iTempPosY;
	iDiff >>= OSM_ZOOM_BASE-m_iOsmAreaBit-m_iOsmZoom;
	iPixelY	= iDiff+m_iTempSizeY/2;


}

//*****************************************************************************
//*
//*		CoordToPos
//*
//*****************************************************************************
//	Rechnet eine globale Koordinate in eine globale OSM-Position bezogen
//  auf OSM_ZOOM_BASE um
//	iPosX/Y			: Ist die globale OSM-Position OSM-Position
//	iPixelX/Y		: Hier wird die Pixelposition gespeichert
void COsmCtrl::CoordToPos(double dCoordX,double dCoordY,int &iPosX,int &iPosY)
{
double	dValue;


	iPosX	= (int)(dCoordX*(((double)OSM_ZOOM_ALL)/360.0));
	iPosX  += OSM_ZOOM_ALL/2;
	iPosX  &= OSM_ZOOM_MASK;

	dValue	= tan(dCoordY*(OSM_PI/180.0));
	dValue	= log(dValue+sqrt(dValue*dValue+1.0));				// = asinh(dVal)
	iPosY	= (int)(dValue*(((double)OSM_ZOOM_ALL)/OSM_PI/2.0));
	iPosY   = OSM_ZOOM_ALL/2-iPosY;
	iPosY  &= OSM_ZOOM_MASK;

}


//*****************************************************************************
//*
//*		StartNextJop
//*
//*****************************************************************************
//	Sucht nach einem einem nicht geladenen
//	uMode		: Ist der Suchmodus
//					Bit0 : 1=nur in sichtbaren Teilen suchen
//					Bit1 : 1=Bei Ladefehlern nochmals versuchen
//	Ergibt die Anzahl der gestarteten Jops, bzw. -1 wenn alle Threads gesperrt sind
int COsmCtrl::StartNextJop(unsigned uMode)
{
DWORD		dwState;
int			iStart;
int			iIdxX;
int			iIdxY;
int			iPosX;
int			iPosY;
int			iMaxX;
int			iMaxY;
int			iMinX;
int			iMinY;
int			iStep;
int			iTurn;
int			iFree;
int			iSet;
int			iPos;
int			iEnd;



	m_sOsmSem.Lock();

	for(iPos=0,iFree=0;iPos<OSM_JOBS;iPos++)			// Suche freie Job-Thread
		{
		if(m_aJops[iPos].uReady)iFree++;
		}

	if(!iFree || m_bServerChanged)
		{
		m_sOsmSem.Unlock();
		return -1;
		}


	iPosX		= (m_iWndSizeX/2+m_iScrollX)>>m_iOsmAreaBit;
	iPosY		= (m_iWndSizeY/2+m_iScrollY)>>m_iOsmAreaBit;

	if(uMode&1)											// Nur die sichbaren Teile absuchen
		{
		iMinX	= (m_iScrollX            )>>m_iOsmAreaBit;
		iMinY	= (m_iScrollY            )>>m_iOsmAreaBit;
		iMaxX	= (m_iScrollX+m_iWndSizeX)>>m_iOsmAreaBit;
		iMaxY	= (m_iScrollY+m_iWndSizeY)>>m_iOsmAreaBit;

		if(iMinX<0)iMinX=0;
		if(iMinY<0)iMinY=0;
		if(iMaxX>=m_iTempStepX)iMaxX=m_iTempStepX-1;
		if(iMaxY>=m_iTempStepX)iMaxY=m_iTempStepY-1;
		}
	else{												// Alles absuchen
		iMinX	= 0;
		iMinY	= 0;
		iMaxX	= m_iTempStepX-1;
		iMaxY	= m_iTempStepY-1;
		}

	if(m_iOsmZoomSize<m_iTempStepY)						// Virtuelle Karte gre als relle
		{
		iSet	= m_iTempStepY-m_iOsmZoomSize;
		iPos	= (iSet+1)/2;

		if(iMinY<iPos)iMinY=iPos;

		iPos   += m_iOsmZoomSize-1;

		if(iMaxY>iPos)iMaxY=iPos;
		}



	if(iPosX<iMinX)iPosX=iMinX;
	if(iPosX>iMaxX)iPosX=iMaxX;
	if(iPosY<iMinY)iPosY=iMinY;
	if(iPosY>iMaxY)iPosY=iMaxY;

						iEnd=iMaxX-iMinX;
	if(iMaxY-iMinY>iEnd)iEnd=iMaxY-iMinY;

	iStart	= 0;
	iTurn	= 1;
	iEnd   += 1;

	for(iStep=0;iStep<=iEnd;iStep--)					// Such Schleife
		{
		if(iStep<=0)
			{
			iTurn  += 1;
			iStep	= iTurn/2;
			}

		if(iPosX>=iMinX	&& iPosX<=iMaxX)
		if(iPosY>=iMinY	&& iPosY<=iMaxY)
			{
			dwState	= m_pTempStates[iPosX+iPosY*m_iTempStepX];

			if(!(uMode&2))								// Fehler nicht nochmals versuchen
				{
				dwState |= (dwState>>8)&0xFF;
				}


			if((dwState^m_uLayerMask)&m_uLayerMask)		// Ist ein Teil der Kachel nicht geladen
				{
				StepToIndex(iPosX,iPosY,iIdxX,iIdxY);

				iSet = -1;

				for(iPos=0;iPos<OSM_JOBS;iPos++)		// Suche anderem Thread fr Kachel
					{
					if(m_aJops[iPos].uReady)
						{
						iSet = iPos;
						continue;
						}

					if(m_aJops[iPos].uIdxX==(UINT)iIdxX)
					if(m_aJops[iPos].uIdxY==(UINT)iIdxY)
						{
						break;
						}
					}

				if(iPos<OSM_JOBS || iSet<0)				// Wird die Kachel gerade geladen
					{
					goto Next;
					}

				iStart++;
				m_uJopTurn++;
				m_aJops[iSet].uIdxX		= iIdxX;
				m_aJops[iSet].uIdxY		= iIdxY;
				m_aJops[iSet].uZoom		= m_iOsmZoom;
				m_aJops[iSet].uTurn		= m_uJopTurn;
				m_aJops[iSet].uTurnId	= m_uTurnId;
				m_aJops[iSet].uExtern	= 0;
				m_aJops[iSet].uReady	= 0;
				m_aJops[iSet].sNext.Unlock();			// Den Job starten

				iStart++;
				iFree--;

				if(iFree<=0)break;
				}
			}

Next:

		switch(iTurn&3)
			{
		case 0:	iPosX--;								// Nach links abbiegen
				break;
		case 1:	iPosY--;								// Nach oben abbiegen
				break;
		case 2:	iPosX++;								// Nach rechst abbiegen
				break;
		case 3:	iPosY++;								// Nach unten abbiegen
				break;
			}
		}

	m_sOsmSem.Unlock();



return iStart;
}

//*****************************************************************************
//*
//*		DrawEmpty
//*
//*****************************************************************************
void COsmCtrl::DrawEmpty()
{
DWORD	uStates;
int		iPosX;
int		iPosY;



	m_sOsmSem.Lock();

	Lock();


	for(iPosY=0;iPosY<m_iTempStepY;iPosY++)
	for(iPosX=0;iPosX<m_iTempStepX;iPosX++)
		{
		uStates = m_pTempStates[iPosX+iPosY*m_iTempStepX];

		if(uStates&0xFF)continue;

		if(!(uStates&BIT_NOT_CLEAR))
			{
			m_pImage->Box(iPosX<<m_iOsmAreaBit,iPosY<<m_iOsmAreaBit,m_iOsmArea,m_iOsmArea,m_uBkColor);
			}

		#if SHOW_INFO
		char	cText[64];
		int		iIdxX;
		int		iIdxY;
		static unsigned char aFont[]=
			{
			#include "Image/Fonts/Arial10.h"
			};


		m_pImage->Frame(iPosX<<m_iOsmAreaBit,iPosY<<m_iOsmAreaBit,m_iOsmArea,m_iOsmArea,BGR(0,0,255));
		sprintf(cText,"X=%i Y=%i Z=%i",iPosX,iPosY,m_iOsmZoom);
		m_pImage->SetFont(aFont);
		m_pImage->SetPixelFunction(TEXT_NORMAL,BGR(0,0,0),0);
		m_pImage->SetPixelFunction(TEXT_NONE  ,BGR(0,0,0),1);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+5,(iPosY<<m_iOsmAreaBit)+5,cText);

		StepToIndex(iPosX,iPosY,iIdxX,iIdxY);

		sprintf(cText,"OX=%i OY=%i",iIdxX,iIdxY);
		m_pImage->SetPixelFunction(TEXT_NORMAL,BGR(255,255,255),0);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+4,(iPosY<<m_iOsmAreaBit)+24,cText);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+6,(iPosY<<m_iOsmAreaBit)+24,cText);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+4,(iPosY<<m_iOsmAreaBit)+26,cText);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+6,(iPosY<<m_iOsmAreaBit)+26,cText);
		m_pImage->SetPixelFunction(TEXT_NORMAL,BGR(0,0,255),0);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+5,(iPosY<<m_iOsmAreaBit)+25,cText);
		#endif
		}


	Unlock();

	m_sOsmSem.Unlock();

}

//*****************************************************************************
//*
//*		DrawArea
//*
//*****************************************************************************
//	Zeichnet eine einzelne Kachel im virtuellen Fenster
//	iPosX/Y		: Ist die Position der Kachel
//	sImage		: Ist das Bild der die Kachel
//	uLayer		: Ist die Layernummer
void COsmCtrl::DrawArea(int iPosX,int iPosY,BitmapImage &sImage,int uLayer,int iIdxX,int iIdxY)
{
BitmapImage	   *pStrech;
BitmapImage	   *pImage;
unsigned		uColor;
int				iLoop;


	



	if(m_iOsmArea!=OSM_BLOCK_AREA)						// Muss das Bild gezoomt werden
		{
		pStrech = new BitmapImage;
		
		pStrech->Stretch(sImage,m_iOsmArea,m_iOsmArea,3);
		pImage = pStrech;
		}
	else{
		pImage = &sImage;
		}


	Lock();


	if(m_iOsmZoomSize<m_iTempStepX)						// Passt die Weltkarte mehrmals ins Fenster
		{
		if(iPosX>=m_iOsmZoomSize)iPosX=iPosX%m_iOsmZoomSize;
		if(iPosX<              0)iPosX=iPosX%m_iOsmZoomSize+m_iOsmZoomSize;

		iLoop = (m_iTempStepX+m_iOsmZoomMask-iPosX)/m_iOsmZoomSize;
		}
	else{
		iLoop = 1;
		}

	for(;iLoop>0;iLoop--,iPosX+=m_iOsmZoomSize)
		{
		if(uLayer)										// Bild ber die anderen zeichnen
			{
			if(pImage->BitCount()==32)
				{
				m_pImage->PutAlphaEx(*pImage,iPosX<<m_iOsmAreaBit,iPosY<<m_iOsmAreaBit);
				}
			else{
				uColor = sImage.GetUserData();

				if(uColor==ALPHA_COLOR && pImage->BitCount()<=8)
					{
					m_pImage->PutAlphaEx(*pImage,iPosX<<m_iOsmAreaBit,iPosY<<m_iOsmAreaBit);
					}
				else{
					m_pImage->Put(*pImage,iPosX<<m_iOsmAreaBit,iPosY<<m_iOsmAreaBit,(uColor&0x00FFFFFF)|0x01000000);
					}
				}
			}
		else{											// Basis Bild
			m_pImage->Put(*pImage,iPosX<<m_iOsmAreaBit,iPosY<<m_iOsmAreaBit);
			}

		#if SHOW_INFO
		char	cText[64];
		int		iIdxX;
		int		iIdxY;

		static unsigned char aFont[]=
			{
			#include "Image/Fonts/Arial14.h"
			};

		m_pImage->Frame(iPosX<<m_iOsmAreaBit,iPosY<<m_iOsmAreaBit,m_iOsmArea,m_iOsmArea,BGR(0,0,255));
		sprintf(cText,"X=%i Y=%i Z=%i",iPosX,iPosY,m_iOsmZoom);
		m_pImage->SetFont(aFont);
		m_pImage->SetPixelFunction(TEXT_NONE  ,BGR(0,0,0),1);

		m_pImage->SetPixelFunction(TEXT_SMOTH,BGR(255,255,255),0);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+4,(iPosY<<m_iOsmAreaBit)+4,cText);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+6,(iPosY<<m_iOsmAreaBit)+4,cText);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+4,(iPosY<<m_iOsmAreaBit)+6,cText);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+6,(iPosY<<m_iOsmAreaBit)+6,cText);
		m_pImage->SetPixelFunction(TEXT_NORMAL,BGR(0,0,0),0);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+5,(iPosY<<m_iOsmAreaBit)+5,cText);

		StepToIndex(iPosX,iPosY,iIdxX,iIdxY);

		sprintf(cText,"OX=%i OY=%i",iIdxX,iIdxY);
		m_pImage->SetPixelFunction(TEXT_SMOTH,BGR(255,255,255),0);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+4,(iPosY<<m_iOsmAreaBit)+24,cText);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+6,(iPosY<<m_iOsmAreaBit)+24,cText);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+4,(iPosY<<m_iOsmAreaBit)+26,cText);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+6,(iPosY<<m_iOsmAreaBit)+26,cText);
		m_pImage->SetPixelFunction(TEXT_NORMAL,BGR(0,0,255),0);
		m_pImage->Print((iPosX<<m_iOsmAreaBit)+5,(iPosY<<m_iOsmAreaBit)+25,cText);
		#endif
				 
		InvalidateRect(iPosX<<m_iOsmAreaBit,iPosY<<m_iOsmAreaBit,m_iOsmArea,m_iOsmArea);

		m_pTempStates[iPosX+iPosY*m_iTempStepX] |= 1<<uLayer;
		m_pTempStates[iPosX+iPosY*m_iTempStepX] &= ~BIT_NOT_CLEAR;


		if(m_iRunning)									// Nachricht zum Neuzeichnen
			{
			if(!PostMessage(WM_AREA_UPDATE,iIdxX,iIdxY))
				{
				Sleep(20);
				PostMessage(WM_AREA_UPDATE,iIdxX,iIdxY);
				}
			}
		}

	Unlock();

	if(pImage!=&sImage)
		{
		delete pStrech;
		}


}

//*****************************************************************************
//*
//*		DrawXor
//*
//*****************************************************************************
void COsmCtrl::DrawXor(CDC *pDc)
{
CDC    *pMyDc=0;
int		iPosX1;
int		iPosX2;
int		iTemp;
int		iXchg;
int		iEnd;
int		iOff;


	if(!pDc)
		{
		pMyDc = GetDC();
		pDc   = pMyDc;
		if(!pMyDc)return;
		}

	pDc->OffsetWindowOrg(m_iScrollX,m_iScrollY);
	pDc->SelectStockObject(NULL_BRUSH);
	pDc->SelectObject(m_sXorPen);
	pDc->SetROP2(R2_XORPEN);

	switch(m_iXorMode)
		{
	case 1:												// Auswahlrechteck

		if(!m_iXorOn)
			{
			PosToPixel(m_iXorPos1X,m_iXorPos1Y,m_iXorOld1X,m_iXorOld1Y);
			PosToPixel(m_iXorPos2X,m_iXorPos2Y,m_iXorOld2X,m_iXorOld2Y);

			if(m_iWayPointViews>1)
				{
				iEnd = (m_iScrollX+m_iWndSizeX)/2+m_iOsmZoomWitdh;
				iOff = (m_iScrollX+m_iWndSizeX)/2-m_iOsmZoomWitdh;

				if(m_iXorOld1X>m_iXorOld2X)
					{
					iTemp       = m_iXorOld1X;
					m_iXorOld1X = m_iXorOld2X;
					m_iXorOld2X = iTemp;
					iXchg       = 1;
					}
				else{
					iXchg       = 1;
					}

				while(m_iXorOld2X>=iEnd && m_iXorOld1X>=iOff)
					{
					iPosX1 = m_iXorOld1X-m_iOsmZoomWitdh;
					iPosX2 = m_iXorOld2X-m_iOsmZoomWitdh;

					if(iPosX1<iOff)break;
					if(iPosX2<iOff)break;

					m_iXorOld1X = iPosX1;
					m_iXorOld2X = iPosX2;
					}

				while(m_iXorOld1X<iOff && m_iXorOld2X<iEnd)
					{
					iPosX1 = m_iXorOld1X+m_iOsmZoomWitdh;
					iPosX2 = m_iXorOld2X+m_iOsmZoomWitdh;

					if(iPosX1>iEnd)break;
					if(iPosX2>iEnd)break;

					m_iXorOld1X = iPosX1;
					m_iXorOld2X = iPosX2;
					}

				if(iXchg)
					{
					iTemp       = m_iXorOld1X;
					m_iXorOld1X = m_iXorOld2X;
					m_iXorOld2X = iTemp;
					}
				}
			}

		pDc->Rectangle(m_iXorOld1X,m_iXorOld1Y,m_iXorOld2X,m_iXorOld2Y);
		m_iXorOn = !m_iXorOn;
		break;

	case 2:												// Linie

		if(!m_iXorOn)
			{
			PosToPixel(m_iXorPos1X,m_iXorPos1Y,m_iXorOld1X,m_iXorOld1Y);
			PosToPixel(m_iXorPos2X,m_iXorPos2Y,m_iXorOld2X,m_iXorOld2Y);
			}

		pDc->MoveTo(m_iXorOld1X,m_iXorOld1Y);
		pDc->LineTo(m_iXorOld2X,m_iXorOld2Y);
		m_iXorOn = !m_iXorOn;
		break;
		}



	if(pMyDc)ReleaseDC(pMyDc);

}

//*****************************************************************************
//*
//*		ShowMenu
//*
//*****************************************************************************
//	Die Menanzeige bei einem Fenster ein bzw. ausschalten
//	bOn			: Neuer Modus frs men
//	Ergibt den alten Modus
BOOL COsmCtrl::ShowMenu(BOOL bOn)
{

	if(bOn)
		{
		if(m_uWndStyle&OSM_MENU)return TRUE;

		if(!m_hMenuBitmap)MenuInit();

		m_uWndStyle |= OSM_MENU;

		InvalidateRect(m_iScrollX+m_iWndSizeX-OSM_MENU_WITDH,m_iScrollY+10,OSM_MENU_WITDH,OSM_MENU_HEIGHT);

		return FALSE;
		}

	if(!(m_uWndStyle&OSM_MENU))return FALSE;


	m_uWndStyle &= ~OSM_MENU;

	InvalidateRect(m_iScrollX+m_iWndSizeX-OSM_MENU_WITDH,m_iScrollY+10,OSM_MENU_WITDH,OSM_MENU_HEIGHT);



return TRUE;
}

//*****************************************************************************
//*
//*		SetMenuColor
//*
//*****************************************************************************
//	Stellt die Menefarbe ein, achtung alle Fenster mit Mens mssen
//	danach mit Redraw() neu gezeichnet werden.
//	uColor		: Ist die neue Farbe
void COsmCtrl::SetMenuColor(COLORREF uColor)
{

	m_uMenuColor = BitmapImage::SwapRGB(uColor);

	if(m_hMenuBitmap)
		{
		MenuExit();
		MenuInit();
		}

}

//*****************************************************************************
//*
//*		MenuExit
//*
//*****************************************************************************
//	Lscht alle Men-Recourcen
void COsmCtrl::MenuExit()
{

	if(m_hMenuBitmap){DeleteObject(m_hMenuBitmap);m_hMenuBitmap=NULL;}
	if(m_hMenuDc    ){DeleteDC    (m_hMenuDc    );m_hMenuDc    =NULL;}
	if(m_hWayPoint  ){DeleteObject(m_hWayPoint  );m_hWayPoint  =NULL;}
	if(m_hWayPointDc){DeleteDC    (m_hWayPointDc);m_hWayPointDc=NULL;}

}

//*****************************************************************************
//*
//*		MenuInit
//*
//*****************************************************************************
//	Erstellt alle Men-Recourcen
void COsmCtrl::MenuInit(HWND hWnd)
{
BitmapImage	sImage;
unsigned	uColR;
unsigned	uColG;
unsigned	uColB;
int			iPosX;
int			iPosY;
int			iAlpha;
int			iStep;
int			iPos;
int			iSub;


	uColB	= ((255+((m_uMenuColor    )&0xFF))/2);
	uColG	= ((255+((m_uMenuColor>> 8)&0xFF))/2);
	uColR	= ((255+((m_uMenuColor>>16)&0xFF))/2);

	sImage.SetSize(32,OSM_MENU_WITDH,OSM_MENU_HEIGHT*4,m_uMenuColor);

	for(iPosY=0;iPosY<OSM_MENU_HEIGHT*4;iPosY++)
	for(iPosX=0;iPosX<OSM_MENU_WITDH   ;iPosX++)
		{
		iStep	= iPosY/OSM_MENU_HEIGHT;
		iSub	= iPosY%OSM_MENU_HEIGHT;
		iPos	= OSM_MENU_HEIGHT-1-iSub;
		iAlpha	= m_aMenuTool[iPosX+iPos*OSM_MENU_WITDH];

		if(iStep==0 || (int)(iSub/(OSM_MENU_HEIGHT/3)+1)!=iStep)
			{
			iAlpha = iAlpha*2/3;
			}

		if(iAlpha==0)
			{
			sImage.SetPixel(iPosX,iPosY,BGR(0,0,0));
			}
		else if(iAlpha==255)
			{
			sImage.SetRGB(iPosX,iPosY,uColR,uColG,uColB,255);
			}
		else{
			sImage.SetRGB(iPosX,iPosY,(uColR*iAlpha)/255,(uColG*iAlpha)/255,(uColB*iAlpha)/255,iAlpha);
			}
		}


	m_hMenuDc	  = CreateCompatibleDC(NULL);
	m_hMenuBitmap = sImage.CreateBitmap();

	SelectObject(m_hMenuDc,m_hMenuBitmap);

	m_hWayPointDc = CreateCompatibleDC(NULL);
	m_hWayPoint	  = DibToBitmap(m_aWayPoindDib,2);
	SelectObject(m_hWayPointDc,m_hWayPoint);


}

//*****************************************************************************
//*
//*		HttpConnect
//*
//*****************************************************************************
//	Baut eine Verbindung zu einem HTTP-Server auf
//	pHttp		: Zeiger auf eine CHttpClient-Instanz
//	pSrvName	: Ist der Name des Servers
//	uPort		: Ist das benutzte TCP-Port
//	Ergibt TRUE wenn eine Verbindung geffnet werden konnte.
BOOL COsmCtrl::HttpConnect(void *pHttp,const char *pSrvName,unsigned uPort)
{
CHttpClient	   *pClient=(CHttpClient*)pHttp;
unsigned		uTurn;


	for(uTurn=0;uTurn<OSM_MAX_CONN;uTurn++)
		{
		if(m_iRunning==0)break;
														// Neu verbinden
		if(!pClient->Connect(pSrvName,uPort))
			{
			return TRUE;
			}
		}


return FALSE;
}

//*****************************************************************************
//*
//*		GetHttpPath
//*
//*****************************************************************************
//	Erstellt den Pfad fr den HTTP-Server
//	pName			: Hier wird der Name gespeichert
//	ucInterface		: Ist die Art des HTTP-Interfaces (0=OSM 1=ESRI 2=YANDEX 3=MAPY)
//	pSrvPath		: Ist der Pfad am HTTP-Server (z.B.: "transport")
//	uZoom			: Ist der aktuelle Zoom-Level (0..30)
//	uIdxX/Y			: Ist die Nummer der Kachel
//	pSuffix			: Suffix fr den Pfad (z.B.: ".png")
//	Ergibt die lnge des erstellen Pfades
int COsmCtrl::GetHttpPath(char *pName,unsigned char ucInterface,const char *pSrvPath,unsigned uZoom,unsigned uIdxX,unsigned uIdxY,const char *pSuffix)
{
int		iLen;


	switch(ucInterface)
		{
	case 3:	iLen=sprintf(pName,"%s/%u-%u-%u%s",pSrvPath,uZoom,uIdxX,uIdxY,pSuffix);	break;	// MAPY
	case 2:	iLen=sprintf(pName,"%s&x=%i&y=%i&z=%i",pSrvPath,uIdxX,uIdxY,uZoom);		break;	// YANDEX
	case 1:	iLen=sprintf(pName,"%s/%u/%u/%u%s",pSrvPath,uZoom,uIdxY,uIdxX,pSuffix);	break;	// ESRI
	default:iLen=sprintf(pName,"%s/%u/%u/%u%s",pSrvPath,uZoom,uIdxX,uIdxY,pSuffix);	break;	// OSM
		}


return iLen;
}

//*****************************************************************************
//*
//*		GetSubTopoArea
//*
//*****************************************************************************
//	Erstellt eine Topologie Kachel aus einem niedrigeren Zoom
//	pCachePath		: Ist der Cache-Pfad der Unterkachel die generiert werden soll
//	pCacheBase		: Ist der Cache-Pfad der Kachel aus der die Daten geholt werden sollen
//	pHttp			: Zeiger auf eine CHttpClient-Instanz
//	ucInterface		: Ist die Art des HTTP-Interfaces (0=OSM 1=ESRI 2=YANDEX 3=MAPY)
//	pSrvPath		: Ist der Pfad am HTTP-Server (z.B.: "transport")
//	uZoom			: Ist der aktuelle Zoom-Level (0..30)
//	uIdxX/Y			: Ist die Nummer der Kachel
//	pSuffix			: Suffix fr den Pfad (z.B.: ".png")
//	uTimeout		: HTTP-Timeout in Millisekunden
//	Ergibt die Lnge der erstellen Datei, bzw. -1 bei einem Fehler
int	COsmCtrl::GetSubTopoArea(char *pCachePath,char *pCacheBase,void *pHttp,unsigned uZoomDiff,unsigned char ucInterface,const char *pSrvPath,unsigned uZoom,unsigned uIdxX,unsigned uIdxY,const char *pSuffix,unsigned uTimeout)
{
CHttpClient	   *pClient=(CHttpClient*)pHttp;
char			cName[MAX_PATH*2];
PngEncoder		sPngEnc;
BitmapImage		sImage;
BitmapImage		sBlock;
LPBYTE			pData;
unsigned		uStep;
unsigned		uMask;
int				iSize;



	   iSize = LengthFile(pCacheBase);
	if(iSize<=0)										// Basiskachel vom Server holen		
		{
		GetHttpPath(cName,ucInterface,pSrvPath,uZoom-uZoomDiff,uIdxX>>uZoomDiff,uIdxY>>uZoomDiff,pSuffix);
		CreatePath(pCacheBase,CP_PATH_LEN);

		iSize = pClient->Get(pCacheBase,cName,uTimeout);
		
		if(iSize<=0)
			{
			RemoveFile(pCacheBase);
			return -1;
			}
		}

		pData = new BYTE[iSize+16];
	if(!pData)return -1;


	   iSize = LoadMemory(pCacheBase,pData,iSize+16);	// Das Bilddaten aus Datei laden
	if(iSize<=0)
		{
		goto Error;
		}
	
	if(ImageLoader(pData,iSize,&sImage))				// Das Bild aus den Bilddaten laden
		{
		goto Error;
		}

	if(sImage.Width ()!=OSM_BLOCK_AREA)goto Error;
	if(sImage.Height()!=OSM_BLOCK_AREA)goto Error;


	if(sImage.BitCount()==8)							// Bild ist nur Alpha-Maske
		{
		sImage.ConvertToAlpha(2);
		}
	else if(sImage.BitCount()!=32)	
		{
		sImage.ConvertToRGB(32);
		}

	uStep	= OSM_BLOCK_AREA>>uZoomDiff;
	uMask	= (1<<uZoomDiff)-1;

	sBlock.Get(sImage,uStep*(uIdxX&uMask),uStep*(uIdxY&uMask),uStep,uStep);
	sBlock.Stretch(OSM_BLOCK_AREA,OSM_BLOCK_AREA,0x01);

	CreatePath(pCachePath,CP_PATH_LEN);

	try	{												// Die Kachel speichern
		sPngEnc.WriteImage(pCachePath,sBlock,0x01);
		}
	catch(...)
		{
		goto Error;
		}

	return LengthFile(pCachePath);

Error:

	delete pData;

return -1;
}

//*****************************************************************************
//*
//*		StartThreads
//*
//*****************************************************************************
//	Alle JOB-Thread starten
void COsmCtrl::StartThreads()
{
DWORD	dwThread;
HANDLE	hThread;
int		iPos;



	InterlockedIncrement(&m_lWindowCnt);


	if(!m_hMenuBitmap && (m_uWndStyle&OSM_MENU))		// Men Bitmap erzeugen
		{
		MenuInit(m_hWnd);
		}

	m_iRunning = 1;

	DrawEmpty();

	m_sOsmSem.Lock();


	for(iPos=0;iPos<OSM_JOBS;iPos++)
		{
		m_aJops[iPos].uReady = 1;

			hThread = CreateThread(NULL,0x20000,DownloadThread,m_aJops+iPos,0,&dwThread);
		if(!hThread)break;

		m_aJops[iPos].hThread	= hThread;
		m_iJobThreads++;
		}

	m_sOsmSem.Unlock();

	SetTimer(ID_TICKER,2000,0);

	if(m_uWndStyle&OSM_SCROLL_NM)						// Notify Message beim Scrollen
	if(!PostMessage(WM_SCROLL_DONE,0))
		{
		Sleep(20);
		PostMessage(WM_SCROLL_DONE,0);
		}


}

//*****************************************************************************
//*
//*		StopThreads
//*
//*****************************************************************************
//	Alle JOB-Thread beenden
void COsmCtrl::StopThreads()
{
CHttpClient	   *pHttp;
int				iPos;


	m_sOsmSem.Lock();

	if(m_iRunning && m_lWindowCnt>0)
		{
		InterlockedDecrement(&m_lWindowCnt);
		}

	m_iRunning = 0;


	for(iPos=0;iPos<OSM_JOBS;iPos++)
		{
		m_aJops[iPos].uIdxX	 = 0xFFFFFFFF;
		m_aJops[iPos].uIdxX	 = 0xFFFFFFFF;
		m_aJops[iPos].sNext.Unlock();						// Job-Thread freigeben

		if(m_aJops[iPos].pStop)
			{
			pHttp	= (CHttpClient*)m_aJops[iPos].pStop;
			pHttp->Shutdown();
			}
		}

	m_sOsmSem.Unlock();

}

//*****************************************************************************
//*
//*		CacheThread
//*
//*****************************************************************************
//	Prft die Chache-Gre
DWORD WINAPI COsmCtrl::CacheThread(LPVOID pParam)
{
PLONG volatile	pWndCount=(PLONG volatile)pParam;
CFileFinder		sFind;
unsigned		uTurn;
unsigned		uByte;
__int64			iSize;
__int64			iMax;
int				iPos;



	iSize = 0;
	uTurn = 0;
	sFind.Set(m_cCachePath,CFF_FIND_SUBDIR|CFF_FIND_ALL,"*");

	for(;;)											// Suche alles im Cache Ordner
		{
		if(!*pWndCount  )break;
		if(!sFind.Next())break;

		uTurn++;
		iSize += sFind.Len();

		if(uTurn%256==0)Sleep(100);
		}


	if(*pWndCount)
		{
		iMax	= m_uChacheSize;
		iMax  <<= 20;

		if(iSize>iMax)								// Mssen Dateien gelscht werden
			{
			sFind.Sort(CFF_SORT_WRITE,CFF_SORT_INVERT);

			for(iPos=sFind.Count()-1;iPos>=0;iPos--)
				{
				if(!*pWndCount)return 0;

				uByte	= sFind.Len(iPos);

				if(RemoveFile(sFind.Name(iPos,1)))
					{
					iSize -= uByte;
					}

				uTurn++;

				if(iSize<=iMax)break;
				if(uTurn%32==0)Sleep(20);
				}
			}

		if(*pWndCount)
			{
			sFind.Set(m_cCachePath,CFF_FIND_SUBDIR|CFF_FIND_DIR,"*");

			for(;;)									// Leere Verzeichnisse lschen
				{
				if(!*pWndCount  )break;
				if(!sFind.Next())break;


				RemoveDirectory(sFind.Name(-1,1));

				uTurn++;

				if(uTurn%256==0)Sleep(100);
				}
			}
	   }


	InterlockedExchange(&m_lCacheRun,0);



return 0;
}

//*****************************************************************************
//*
//*		DownloadThread
//*
//*****************************************************************************
//	Downlad-Funktion fr die OSM-Kachelsn
DWORD WINAPI COsmCtrl::DownloadThread(LPVOID    pParam){JobEntry *pJob=(JobEntry*)pParam;return ((COsmCtrl*)(pJob->pThis))->DownloadThread(pJob);}
DWORD        COsmCtrl::DownloadThread(JobEntry *pJob)
{
char			cSrvName[OSM_SRV_NAME];
char			cSrvPath[OSM_SRV_PATH];
char			cName[MAX_PATH*2];
char			cTemp[MAX_PATH];
char			cPath[MAX_PATH];
char			cFixedPath[32];
char			cSuffix[256];
char			cDummy [16];
ServerEntry	   *pServer;
char	       *pBuffer;
unsigned	   *pCol;
BitmapImage		sImage;
LPCSTR			pData;
CHttpClient		sHttp;
unsigned		uTimeout;
unsigned		uExtern;
unsigned		uLayer;
unsigned		uMyId;
unsigned		uLoad;
unsigned		uDiff;
unsigned		uTurn;
unsigned		uPort;
unsigned		uBit;
BYTE			uZoom;
DWORD			uIdxX;
DWORD			uIdxY;
DWORD			uState;
BYTE			uTurnId;
BOOL			bRetry;
BOOL			bConn;
int				iSize;
int				iByte;
int				iSetX;
int				iSetY;
int				iPosX;
int				iPosY;
int				iOff;
int				iMul;
int				iAdd;
int				iLen;
int				iVal;
int				iPos;



	memcpy(cPath,m_cCachePath,m_iCacheLen);

	pJob->pStop	= &sHttp;
	uMyId		= 0xFFFFFFFF;
	uPort		= OSM_TCP_PORT;
	uTimeout 	= OSM_TIMEOUT;
	bConn	 	= FALSE;
	cDummy[0]	= 0;
	pServer	 	= 0;
	pBuffer		= 0;
	iByte		= 0;

	for(;;)
		{
		pJob->sNext.Lock();								// Warte auf den nchsten Job

		m_sOsmSem.Lock();

		if(!m_iRunning)									// Wurde das Fenster geschlossen
			{
			break;
			}

		uIdxX	= pJob->uIdxX;
		uIdxY	= pJob->uIdxY;
		uZoom	= pJob->uZoom;
		uTurnId	= pJob->uTurnId;
		uExtern	= pJob->uExtern;
				  pJob->uReady=0;


		if(uIdxX==0xFFFFFFFF)							// Spezial Job
			{
			if(uIdxY==0xFFFFFFFF)						// Thread beenden
				{
				break;
				}
			}
														// Ist die Kachel im vitruellen Fenster
		if(!uExtern && !IndexToStep(uIdxX,uIdxY,iSetX,iSetY))
			{
			pJob->uReady=1;
			m_sOsmSem.Unlock();
			continue;
			}

		uState = (uExtern)? 0:m_pTempStates[iSetX+iSetY*m_iTempStepX];

		m_sOsmSem.Unlock();

		#if OSM_LOG
		WaitForSingleObject(hOsmLogSem,INFINITE);
		printf("jop=%i\tx=%i\ty=%i\tz=%i\n",pJob-m_aJops,uIdxX,uIdxY,uZoom);
		ReleaseSemaphore(hOsmLogSem,1,NULL);
		#endif

														// Alle Layer prfen
		for(uLayer=0,uBit=1;m_uLayerMask&uBit;uLayer++,uBit<<=1)
			{
			bRetry = 0;
Retry:
			if(m_iRunning==0)break;
			if(uExtern==0 && (uZoom^m_iOsmZoom))
				{
				break;
				}

			if(uState&uBit)								// Ist dieser Layer bereits ok
				{
				if(m_iTempStepX>=m_iOsmZoomSize)
					{
					   iSetX += m_iOsmZoomSize;
					if(iSetX<m_iTempStepX)
						{
						uState = m_pTempStates[iSetX+iSetY*m_iTempStepX];
						goto Retry;
						}

					iSetX %= m_iOsmZoomSize;
					}

				if(m_iTempStepY>=m_iOsmZoomSize)
					{
					   iSetY += m_iOsmZoomSize;
					if(iSetY<m_iTempStepY)
						{
						uState = m_pTempStates[iSetX+iSetY*m_iTempStepX];
						goto Retry;
						}

					iSetY %= m_iOsmZoomSize;
					}

				continue;
				}

			if(uTurnId!=m_uTurnId)						// Gibt es andere Server-Einstellungen
				{
				break;
				}

			uState &= ~0xFF;
			pServer	= m_aLayers+uLayer;

			if(uMyId!=pServer->ucTurnId+uLayer*256)		// Einen anderen Server-benutzen
				{
				if(!pServer->ucTurnId)break;

				m_sOsmSem.Lock();

				strlcpy(cSrvName,pServer->cServer,sizeof(cSrvName));
				strlcpy(cSrvPath,pServer->cPath  ,sizeof(cSrvPath));
				strlcpy(cSuffix ,pServer->cSuffix,sizeof(cSuffix ));

				iPos = strlchar(cSuffix,'.');

				if(cSuffix[iPos])						// Gibt es ein Suffix
					{
					iVal = strwlen(cSuffix+iPos,0x07);
					strlcpy(cDummy,cSuffix+iPos,sizeof(cDummy),iVal);
					}
				else{
					memcpy(cDummy,"png",4);
					}


				if(pServer->ucFixed)					// In fixen Cache legen
					{
					strcpy(cFixedPath,"../OsmFixed/");
					}
				else{
					cFixedPath[0]=0;
					}

				uMyId	 = pServer->ucTurnId+uLayer*256;
				uTimeout = pServer->uTimeout;
				uPort	 = pServer->usTcpPort;
				iOff	 = pServer->iAlphaOff;
				iMul	 = pServer->iAlphaMul;
				iAdd	 = pServer->ucAlphaAdd;
				pCol	 = pServer->aAlphaAdd;
				bConn	 = FALSE;

				m_sOsmSem.Unlock();
				}

														// Pfad im Chache erstellen
			sprintf(cPath+m_iCacheLen,"%s%s%s/%u/%u/%u.%s",cFixedPath,cSrvName,cSrvPath,uZoom,uIdxX,uIdxY,cDummy);
			strreplace(cPath+m_iCacheLen,'?','_');
			strreplace(cPath+m_iCacheLen,'&','_');

			iSize = LengthFile(cPath);

														// Topo-Layer fr diesen Zoom nicht vorhanden
			if(iSize<=0 && iAdd>=2 && pCol[0]==GRAY_COLOR)
				{
				   iVal = pCol[1]>>24;
				if(iVal<=0)iVal=28;	

				 
				if(m_iOsmZoom>iVal)						// Ist dieser Zoom-Level nicht verfgbar
					{
					   uDiff = m_iOsmZoom-iVal;
					if(uDiff<=4 && m_iOsmZoom<20)
						{
						if(!bConn)
							{
							bConn = HttpConnect(&sHttp,cSrvName,uPort);
							}
						
						if(bConn)
							{				
							sprintf(cName,"%.*s%s%s%s/%u/%u/%u.%s",m_iCacheLen,cPath,cFixedPath,cSrvName,cSrvPath,uZoom-uDiff,uIdxX>>uDiff,uIdxY>>uDiff,cDummy);
							strreplace(cName+m_iCacheLen,'?','_');
							strreplace(cName+m_iCacheLen,'&','_');

							iSize = GetSubTopoArea(cPath,cName,&sHttp,uDiff,pServer->ucInterface,cSrvPath,uZoom,uIdxX,uIdxY,cSuffix,uTimeout);
							}
						}

					if(iSize<=0)
						{
						if(iSetX<m_iTempStepX && iSetY<m_iTempStepY)
							{
							m_pTempStates[iSetX+iSetY*m_iTempStepX] |= 1<<uLayer;
							m_pTempStates[iSetX+iSetY*m_iTempStepX] &= ~BIT_NOT_CLEAR;
							}

						continue;
						}
					}
				}


			if(iSize>=1)								// Ist die Datei bereits im Cache
				{
				#if OSM_LOG
				WaitForSingleObject(hOsmLogSem,INFINITE);
				printf("jop=%i\tx=%i\ty=%i\tz=%i\tfile=%i\n",pJob-m_aJops,uIdxX,uIdxY,uZoom,iSize);
				ReleaseSemaphore(hOsmLogSem,1,NULL);
				#endif

				if(uExtern){m_sOsmCall.Unlock();continue;}

				if(iSize>iByte)
					{
					if(pBuffer)delete pBuffer;
					iByte	=  (iSize+256+0x3FFF)&~0x3FFF;
					pBuffer	=  new char[iByte];
					iByte  -= 256;
					}

				if(LoadMemory(cPath,pBuffer,iByte+256,1)!=iSize)
					{
					goto Error;
					}

				ChangeFileTime(cPath,time(0));
				pData = pBuffer;
				}
			else{										// Datei herunter laden
				if(!bConn)
					{
						bConn = HttpConnect(&sHttp,cSrvName,uPort);
					if(!bConn)							// Keine Verbindung
						{
						#if OSM_LOG
						WaitForSingleObject(hOsmLogSem,INFINITE);
						printf("jop=%i\tx=%i\ty=%i\tz=%i\tconn err\n",pJob-m_aJops,uIdxX,uIdxY,uZoom);
						ReleaseSemaphore(hOsmLogSem,1,NULL);
						#endif
						continue;
						}
					}

														// Pfad im Cache erzeugen
				iLen = sprintf(cPath+m_iCacheLen,"%s%s%s/%u/%u",cFixedPath,cSrvName,cSrvPath,uZoom,uIdxX);
				strreplace(cPath+m_iCacheLen,'?','_');
				strreplace(cPath+m_iCacheLen,'&','_');
				CreatePath(cPath);

				sprintf(cPath+m_iCacheLen+iLen,"/%u.tmp",uIdxY);

				GetHttpPath(cName,pServer->ucInterface,cSrvPath,uZoom,uIdxX,uIdxY,cSuffix);

				iSize = 0;
				uLoad = 0;

				for(uTurn=0;uTurn<OSM_MAX_TURNS;uTurn++)
					{
					if(m_iRunning==0)
						{
						iSize = 0;
						break;
						}

					if(uExtern==0 && (uZoom^m_iOsmZoom))
						{
						iSize = 0;
						break;
						}

					#if OSM_LOG
					WaitForSingleObject(hOsmLogSem,INFINITE);
					printf("jop=%i\tx=%i\ty=%i\tz=%i\thttp\n",pJob-m_aJops,uIdxX,uIdxY,uZoom);
					ReleaseSemaphore(hOsmLogSem,1,NULL);
					#endif

					iSize = sHttp.Get(cPath,cName,uTimeout);

					#if OSM_LOG
					WaitForSingleObject(hOsmLogSem,INFINITE);
					printf("jop=%i\tx=%i\ty=%i\tz=%i\thttp=%i\n",pJob-m_aJops,uIdxX,uIdxY,uZoom,iSize);
					ReleaseSemaphore(hOsmLogSem,1,NULL);
					#endif

					if(iSize<=0)
						{
						sHttp.ForceReconnect();
						bConn = FALSE;
						continue;
						}

						pData = sHttp.Data();
					if(!pData || memcmp(pData,"\x89PNG",4))
						{
						if(memcmp(pData,"\xFF\xD8\xFF\xE0",4) || memcmp(pData+6,"JFIF",4))
							{
							iSize = 0;					// Kein PNG/JPG-Format
							continue;
							}

						uLoad = 1;
						}


					if(m_iRunning==0)
						{
						iSize = 0;
						break;
						}

					SetFileType(cTemp,cPath,cDummy);

					if(MoveFile(cPath,cTemp))
						{
						break;
						}

					RemoveFile(cTemp);					// Eine vorhandene Datei lschen

					if(MoveFile(cPath,cTemp))
						{
						break;
						}

					if(m_iRunning==0)
						{
						iSize = 0;
						break;
						}

					Sleep(200);

					if(MoveFile(cPath,cTemp))
						{
	 					break;
						}
					}

				if(iSize<=0 || !pData)					// Schlug Download fehl
					{
					goto Error;
					}
				}

			if(ImageLoader(pData,iSize,&sImage))		// Das Bild laden
				{
				goto Error;
				}

			if(sImage.Width ()!=OSM_BLOCK_AREA)goto Error;
			if(sImage.Height()!=OSM_BLOCK_AREA)goto Error;
			if(uExtern){m_sOsmCall.Unlock();continue;}


			sImage.SetUserData(0);
			m_sOsmSem.Lock();							// Das Bild zeichnen

			if(uTurnId!=m_uTurnId || uZoom!=m_iOsmZoom)	// Gibt es andere Server-Einstellungen
				{
				m_sOsmSem.Unlock();
				break;
				}
														// Ist die Kachel im vitruellen Fenster
			if((uBit&m_uLayerMask) && IndexToStep(uIdxX,uIdxY,iPosX,iPosY))
				{
				if(!m_iRunning)break;

			Jmp:uState = m_pTempStates[iPosX+iPosY*m_iTempStepX];

				if((uState&0xFF)!=uBit-1)
					{
					if(m_iTempStepX>=m_iOsmZoomSize)
						{
						   iPosX += m_iOsmZoomSize;
						if(iPosX<m_iTempStepX)goto Jmp;
						   iPosX %= m_iOsmZoomSize;
						}

					if(m_iTempStepY>=m_iOsmZoomSize)
						{
						   iPosY += m_iOsmZoomSize;
						if(iPosY<m_iTempStepY)goto Jmp;
						}

					m_sOsmSem.Unlock();
					break;
					}


				if(uLayer && iAdd)						// Gibt es spezial Arbeiten
					{
					if(pCol[0]==GRAY_COLOR && iAdd>=2)	// Graustufenbild zu Tranparenz	
						{
						GrayToAlpha(sImage,pCol[1]);
						sImage.SetUserData(ALPHA_COLOR);
						}								// Transparenz-Layer 
					else if(pCol[0]==TRANS_COLOR && iAdd>=2)		
						{
						if(iMul && sImage.BitCount()<32 && sImage.BitCount()>8)
							{
							sImage.ConvertToRGB(32);
							}

						DoTranspatent(sImage,pCol[1],pCol[1]>>24);
						}
					else{								// Bildteile durchsichtig machen
						DoTranspatent(sImage,pCol,iAdd);
						}
					}

				if(uLayer && iAdd && iOff==1 && !iMul)	// Beschriftungen hervorheben
					{
					HighlightFonts(sImage);
					}

				if(uLayer!=0 && iMul)					// Alpha-Werte des Bildes bei Layern anpassen
					{
					MultiplyAlpha(sImage,iMul,iOff);
					sImage.SetUserData(ALPHA_COLOR);
					}

				if(uLayer==0 && sImage.HasAlphaEx()>=2)	// Alpha-Maske im Grundbild korrigieren
					{
					sImage.PutAlphaArea(COL_BG);
					}


														// Die Kachel zeichnen
				DrawArea(iPosX,iPosY,sImage,uLayer,uIdxX,uIdxY);
				}

			m_sOsmSem.Unlock();
			continue;

Error:
			if(RemoveFile(cPath) && !bRetry)			// Die Datei nochmals herunter laden
				{
				bRetry = TRUE;
				goto Retry;
				}

			if(uExtern){m_sOsmCall.Unlock();continue;}

			m_sOsmSem.Lock();

			if(IndexToStep(uIdxX,uIdxY,iPosX,iPosY))
				{
				while(m_uLayerMask&uBit)				// Fehlerbits senden
					{
					if(!m_iRunning)break;

				Do:	m_pTempStates[iPosX+iPosY*m_iTempStepX] |= uBit<<8;

					if(m_iTempStepX>=m_iOsmZoomSize)
						{
						   iPosX += m_iOsmZoomSize;
						if(iPosX<m_iTempStepX)goto Do;
						   iPosX %= m_iOsmZoomSize;
						}

					if(m_iTempStepY>=m_iOsmZoomSize)
						{
						   iPosY += m_iOsmZoomSize;
						if(iPosY<m_iTempStepY)goto Do;
						}

					uBit<<=1;
					uLayer++;
					}
				}

			m_sOsmSem.Unlock();
			}

		m_sOsmSem.Lock();
		pJob->uReady = 1;								// Der Thread ist wieder bereit
		m_sOsmSem.Unlock();

		if(m_iRunning)									// Nachricht das ein Job frei ist
			{
			if(!PostMessage(WM_NEXT_JOB,pJob-m_aJops))
				{
				Sleep(20);
				PostMessage(WM_NEXT_JOB,pJob-m_aJops);
				}
			}
		}

	if(pBuffer)delete pBuffer;

	pJob->uReady = 0;
	pJob->pStop	 = 0;

	m_iJobThreads--;
	m_sOsmSem.Unlock();

	if(m_iJobThreads<=0)
		{
		m_sOsmExit.Unlock();
		}



return 0;
}


//*****************************************************************************
//*
//*		Reload
//*
//*****************************************************************************
void COsmCtrl::Reload()
{
int		iPos;



	m_sOsmSem.Lock();

	memset(m_pTempStates,0,sizeof(DWORD)*m_iTempStepX*m_iTempStepY);

	for(iPos=0;iPos<OSM_JOBS;iPos++)					// Aktuelle Jop's beenden
		{
		CHttpClient	*pHttp;

		if(m_aJops[iPos].uReady)continue;

		   pHttp=(CHttpClient*)m_aJops[iPos].pStop;
		if(pHttp)
			{
			pHttp->Shutdown();
			}
		}

	m_sOsmSem.Unlock();

	if(m_iRunning)										// Jobs wieder starten
		{
		if(!PostMessage(WM_NEXT_JOB,0))
			{
			Sleep(20);
			PostMessage(WM_NEXT_JOB,0);
			}
		}


}

//*****************************************************************************
//*
//*		ClearCache
//*
//*****************************************************************************
//	Den gesamte Chache lschen
BOOL COsmCtrl::ClearCache()
{
BOOL	bRet;


	if(ExistFile(m_cCachePath)!=2)
		{
		return FALSE;
		}


	m_sOsmSem.Lock();

	bRet = RemovePath(m_cCachePath);

	m_sOsmSem.Unlock();

	Reload();


return bRet;
}

//*****************************************************************************
//*
//*		SetScrollEx
//*
//*****************************************************************************
//	Scrollt einen Pixlel im virtuellen OSM-Fenster zu einer Position in der OSM-Karte
//	iPosX/iPosY		: Ist die Koordinate in der OSM-Karte bezogen zu OSM_ZOOM_BASE
//	iPixelX/iPixelY	: Ist der Pixel im im virtuellen OSM-Fenster
//	bRedraw			: Soll das Fenster neu gezeichnet werden
//	Ergibt TRUE wenn die Scroll-Position verndert wurde.
BOOL COsmCtrl::SetScrollEx(int iPosX,int iPosY,int iPixelX,int iPixelY,BOOL bRedraw)
{
int		iDeltaX;
int		iDeltaY;
int		iNowX;
int		iNowY;


	PixelToPos(iPixelX,iPixelY,iNowX,iNowY);

	iDeltaX		= iPosX-iNowX;
	iDeltaY		= iPosY-iNowY;

return SetScrollPos(m_iOsmScrollX+iDeltaX,m_iOsmScrollY+iDeltaY,bRedraw);
}

//*****************************************************************************
//*
//*		SetScrollPos
//*
//*****************************************************************************
//	Scrollt die Bildmitte zu einer Position in der OSM-Karte
//	iPosX/iPosY		: Ist die Koordinate in der OSM-Karte bezogen zu OSM_ZOOM_BASE
//	bRedraw			: Soll das Fenster neu gezeichnet werden
//	Ergibt TRUE wenn die Scroll-Position verndert wurde.
BOOL COsmCtrl::SetScrollPos(int iPosX,int iPosY,BOOL bRedraw)
{
int		iScrollY;
int		iScrollX;
int		iDeltaX;
int		iDeltaY;
int		iRangeX;
int		iStepsX;
int		iStepsY;
int		iSizeX;
int		iSizeY;
int		iShift;
int		iMask;
int		iNumX;
int		iNumY;
int		iFlag;
int		iTurn;
int		iDif1;
int		iDif2;
int		iPos1;
int		iPos2;
int		iPosW;



	iShift		= OSM_ZOOM_BASE-m_iOsmAreaBit-m_iOsmZoom;
	iPosX	   &= OSM_ZOOM_MASK;
	iPosY	   &= OSM_ZOOM_MASK;
	iFlag		= 0;
	iTurn		= 0;

	iDeltaY		= m_iOsmRangeY2-m_iOsmRangeY1;			// Y-Scroll-Bereich prfen
	iDeltaY	  >>= iShift;
	iNumY		= iPosY>>iShift;
	iMask		= (1<<(m_iOsmAreaBit+m_iOsmZoom))-1;

	if(m_iWndSizeY>=iDeltaY)
		{
		iPosY = (m_iOsmRangeY2+m_iOsmRangeY1)/2;
		}
	else if(iNumY<(m_iOsmRangeY1>>iShift)+m_iWndSizeY/2)
		{
		iPosY = m_iOsmRangeY1+(m_iWndSizeY<<(iShift-1));
		}
	else if(iNumY>(m_iOsmRangeY2>>iShift)-m_iWndSizeY/2)
		{
		iPosY = m_iOsmRangeY2-(m_iWndSizeY<<(iShift-1));
		}

	if(m_iOsmRangeX2 || m_iOsmRangeX1)					// X-Scroll-Bereich prfen
		{
		iDeltaX	= (m_iOsmRangeX2-m_iOsmRangeX1)&OSM_ZOOM_MASK;
		iRangeX =  iDeltaX>>iShift;

		if(m_iWndSizeX>=iRangeX)
			{
			iPosX = (m_iOsmRangeX1+iDeltaX/2)&OSM_ZOOM_MASK;
			}
		else{
			   iDif1 = (iPosX-m_iOsmRangeX1)&OSM_ZOOM_MASK;
			if(iDif1>=OSM_ZOOM_ALL/2)iDif1=(-iDif1)&(OSM_ZOOM_MASK>>1);
			   iDif2 = (iPosX-m_iOsmRangeX2)&OSM_ZOOM_MASK;
			if(iDif2>=OSM_ZOOM_ALL/2)iDif2=(-iDif2)&(OSM_ZOOM_MASK>>1);

			if(m_iOsmRangeX1<m_iOsmRangeX2)
				{
				iPos1 = (m_iOsmRangeX1>>iShift)+m_iWndSizeX/2;
				iPos2 = (m_iOsmRangeX2>>iShift)-m_iWndSizeX/2;
				iPosW = (iPosX        >>iShift);

				if(iPos1>iPos2)
					{
					iPos1 = (m_iOsmRangeX1+m_iOsmRangeX2)>>(iShift+1);
					iPos2 = iPos2;
					}
				}
			else{
				iPos1 = ((m_iOsmRangeX1             )>>iShift)+m_iWndSizeX/2;
				iPos2 = ((m_iOsmRangeX2+OSM_ZOOM_ALL)>>iShift)-m_iWndSizeX/2;

				if(iPosX<m_iOsmRangeX2)
						iPosW = (iPosX+OSM_ZOOM_ALL)>>iShift;
				else	iPosW = (iPosX             )>>iShift;

				if(iPos1>iPos2)
					{
					iPos1  = (m_iOsmRangeX1+m_iOsmRangeX2)>>(iShift+1);
					iPos1 += (OSM_ZOOM_ALL				 )>>(iShift+1);
					iPos2  = iPos2;
					}
				}

			if(iPosW<iPos1 || iPosW>iPos2)
				{
				iPosX  = (iDif1<iDif2)? iPos1<<iShift:iPos2<<iShift;
				iPosX &= OSM_ZOOM_MASK;
				}
			}
		}


	if(m_iOsmScrollX==iPosX && m_iOsmScrollY==iPosY)	// Gab es nderungen
		{
		return FALSE;
		}

	m_iOsmScrollX	= iPosX;
	m_iOsmScrollY	= iPosY;

	if(m_iWndSizeX<=0 || m_iWndSizeY<=0)
		{
		return FALSE;
		}

Retry:

	m_iOsmScrollX  &= OSM_ZOOM_MASK;
	m_iOsmScrollY  &= OSM_ZOOM_MASK;
	iScrollX		= m_iOsmScrollX-m_iTempPosX;
	iScrollY		= m_iOsmScrollY-m_iTempPosY;
	iScrollX	  >>= iShift;
	iScrollY	  >>= iShift;
	iScrollX	   += m_iTempSizeX/2;
	iScrollY	   += m_iTempSizeY/2;
	iScrollX	   -= m_iWndSizeX /2;
	iScrollY	   -= m_iWndSizeY /2;
	iScrollX	   &= iMask;

	if(iScrollX==m_iScrollX && iScrollY==m_iScrollY)	// Hat sich der Pixel-Scroll verndert
		{
		return FALSE;
		}

	if(m_iTempStepX>=m_iOsmZoomSize)
		{
		while(iScrollX>m_iTempSizeX-m_iWndSizeX)iScrollX-=m_iOsmZoomSize<<m_iOsmAreaBit;
		while(iScrollX<0                       )iScrollX+=m_iOsmZoomSize<<m_iOsmAreaBit;
		}

	if(iScrollX>=0 && iScrollY>=0)						// Nur virtuelles OSM-Fensters scrollen
	if(iScrollX<m_iTempSizeX-m_iWndSizeX)
	if(iScrollY<m_iTempSizeY-m_iWndSizeY)
		{
		iTurn = 16;
		}

	if(iTurn>=16)
		{
		if(iScrollX==m_iScrollX)
		if(iScrollY==m_iScrollY)
			{
			return FALSE;
			}

		CBitmapWnd::SetScrollPos(iScrollX,iScrollY,bRedraw);


		if(iFlag && m_iRunning)
			{
			DrawEmpty();

			if(m_iWndSizeX && m_iWndSizeY)				// Weiter Jobs starten
			if(!PostMessage(WM_NEXT_JOB,0))
				{
				Sleep(20);
				PostMessage(WM_NEXT_JOB,0);
				}

			if(m_uWndStyle&OSM_SCROLL_NM)				// Notify Message beim Scrollen
			if(!PostMessage(WM_SCROLL_DONE,0))
				{
				Sleep(20);
				PostMessage(WM_SCROLL_DONE,0);
				}
			}

		return TRUE;
		}


	m_sOsmSem.Lock();


	iDeltaX		  = m_iTempPosX-m_iOsmScrollX;
	iDeltaX		>>= iShift;
	iDeltaX		 &=~m_iOsmAreaMask;
	m_iTempPosX	 -= iDeltaX<<iShift;
	m_iTempPosX  &= OSM_ZOOM_MASK;

	iDeltaY		  = m_iTempPosY-m_iOsmScrollY;
	iDeltaY 	>>= iShift;
	iDeltaY		 &=~m_iOsmAreaMask;
	m_iTempPosY	 -= iDeltaY<<iShift;
	m_iTempPosY  &= OSM_ZOOM_MASK;


	Lock();
														// Alles lschen
	if(iDeltaX>=m_iTempSizeX || iDeltaX<=-m_iTempSizeX || iDeltaY>=m_iTempSizeY || iDeltaY<=-m_iTempSizeY)
		{
		m_pImage->Box(0,0,m_iTempSizeX,m_iTempSizeY,m_uBkColor);
		memset(m_pTempStates,0,sizeof(DWORD)*m_iTempStepX*m_iTempStepY);
		}
	else{												// Nur einen Teil
		ASSERT(m_pTempStates[-1]==0xFDFDFDFD);
		ASSERT(m_pTempStates[m_iTempStepX*m_iTempStepY]==0xFDFDFDFD);


		m_pImage->Move(0,0,m_iTempSizeX,m_iTempSizeY,iDeltaX,iDeltaY);


		if(iDeltaY<0 && iDeltaX<0)
			{
			iSizeX	= m_iTempSizeX+iDeltaX;
			iSizeY	= m_iTempSizeY+iDeltaY;
			iDeltaX	=-iDeltaX;
			iDeltaY	=-iDeltaY;
			iStepsX	= iDeltaX>>m_iOsmAreaBit;
			iStepsY	= iDeltaY>>m_iOsmAreaBit;
			iNumX	= iSizeX >>m_iOsmAreaBit;
			iNumY	= iSizeY >>m_iOsmAreaBit;

			m_pImage->Box(0     ,iSizeY,m_iTempSizeX,iDeltaY,m_uBkColor);
			m_pImage->Box(iSizeX,0     ,iDeltaX     ,iSizeY ,m_uBkColor);

			for(iPosY=0;iPosY<iNumY;iPosY++)
				{
				memset(	m_pTempStates+(iPosY        )*m_iTempStepX+iNumX,0,sizeof(DWORD)*iStepsX);
				memcpy(	m_pTempStates+(iPosY        )*m_iTempStepX,
						m_pTempStates+(iPosY+iStepsY)*m_iTempStepX+iStepsX,sizeof(DWORD)*iNumX);
				}

			memset(m_pTempStates+iNumY*m_iTempStepX,0,sizeof(DWORD)*m_iTempStepX*iStepsY);

			ASSERT(m_pTempStates[-1]==0xFDFDFDFD);
			ASSERT(m_pTempStates[m_iTempStepX*m_iTempStepY]==0xFDFDFDFD);
			}
		else if(iDeltaY<0 && iDeltaX>=0)
			{
			iSizeX	= m_iTempSizeX-iDeltaX;
			iSizeY	= m_iTempSizeY+iDeltaY;
			iDeltaY	=-iDeltaY;
			iStepsX	= iDeltaX>>m_iOsmAreaBit;
			iStepsY	= iDeltaY>>m_iOsmAreaBit;
			iNumX	= iSizeX >>m_iOsmAreaBit;
			iNumY	= iSizeY >>m_iOsmAreaBit;

			m_pImage->Box(0,iSizeY,m_iTempSizeX,iDeltaY,m_uBkColor);
			m_pImage->Box(0,0     ,iDeltaX     ,iSizeY ,m_uBkColor);

			for(iPosY=0;iPosY<iNumY;iPosY++)
				{
				memset(	m_pTempStates+(iPosY        )*m_iTempStepX,0,sizeof(DWORD)*iStepsX);
				memcpy(	m_pTempStates+(iPosY        )*m_iTempStepX+iStepsX,
						m_pTempStates+(iPosY+iStepsY)*m_iTempStepX,  sizeof(DWORD)*iNumX);
				}

			memset(m_pTempStates+iNumY*m_iTempStepX,0,sizeof(DWORD)*m_iTempStepX*iStepsY);

			ASSERT(m_pTempStates[-1]==0xFDFDFDFD);
			ASSERT(m_pTempStates[m_iTempStepX*m_iTempStepY]==0xFDFDFDFD);
			}
		else if(iDeltaY>0 && iDeltaX<0)
			{
			iSizeX	= m_iTempSizeX+iDeltaX;
			iSizeY	= m_iTempSizeY-iDeltaY;
			iDeltaX	=-iDeltaX;
			iStepsX	= iDeltaX>>m_iOsmAreaBit;
			iStepsY	= iDeltaY>>m_iOsmAreaBit;
			iNumX	= iSizeX >>m_iOsmAreaBit;
			iNumY	= iSizeY >>m_iOsmAreaBit;

			m_pImage->Box(0     ,0      ,m_iTempSizeX,iDeltaY,m_uBkColor);
			m_pImage->Box(iSizeX,iDeltaY,iDeltaX     ,iSizeY ,m_uBkColor);

			for(iPosY=iNumY;iPosY>0;)
				{
				iPosY--;
				memset(	m_pTempStates+(iPosY+iStepsY)*m_iTempStepX+iNumX,0,sizeof(DWORD)*iStepsX);
				memcpy(	m_pTempStates+(iPosY+iStepsY)*m_iTempStepX,
						m_pTempStates+(iPosY        )*m_iTempStepX+iStepsX,sizeof(DWORD)*iNumX);
				}

			memset(m_pTempStates,0,sizeof(DWORD)*m_iTempStepX*iStepsY);

			ASSERT(m_pTempStates[-1]==0xFDFDFDFD);
			ASSERT(m_pTempStates[m_iTempStepX*m_iTempStepY]==0xFDFDFDFD);
			}
		else if(iDeltaY>0 && iDeltaX>=0)
			{
			iSizeX	= m_iTempSizeX-iDeltaX;
			iSizeY	= m_iTempSizeY-iDeltaY;
			iStepsX	= iDeltaX>>m_iOsmAreaBit;
			iStepsY	= iDeltaY>>m_iOsmAreaBit;
			iNumX	= iSizeX >>m_iOsmAreaBit;
			iNumY	= iSizeY >>m_iOsmAreaBit;

			m_pImage->Box(0     ,0,m_iTempSizeX,iDeltaY,m_uBkColor);
			m_pImage->Box(iSizeX,0,iDeltaX     ,iSizeY ,m_uBkColor);

			for(iPosY=iNumY;iPosY>0;)
				{
				iPosY--;
				memset(	m_pTempStates+(iPosY+iStepsY)*m_iTempStepX,0,sizeof(DWORD)*iStepsX);
				memcpy(	m_pTempStates+(iPosY+iStepsY)*m_iTempStepX+iStepsX,
						m_pTempStates+(iPosY        )*m_iTempStepX  ,sizeof(DWORD)*iNumX);
				}

			memset(m_pTempStates,0,sizeof(DWORD)*m_iTempStepX*iStepsY);

			ASSERT(m_pTempStates[-1]==0xFDFDFDFD);
			ASSERT(m_pTempStates[m_iTempStepX*m_iTempStepY]==0xFDFDFDFD);
			}
		else{
			iSizeX	= m_iTempSizeX-abs(iDeltaX);
			iStepsX	= abs(iDeltaX)>>m_iOsmAreaBit;
			iNumX	= iSizeX      >>m_iOsmAreaBit;

			if(iDeltaX>0)
				{
				m_pImage->Box(0     ,0, iDeltaX,m_iTempSizeY,m_uBkColor);
				iScrollX = iStepsX;
				iScrollY = 0;
				iPosX	 = 0;
				}
			else{
				m_pImage->Box(iSizeX,0,-iDeltaX,m_iTempSizeY,m_uBkColor);
				iScrollX = 0;
				iScrollY = iStepsX;
				iPosX	 = iNumX;
				}

			for(iPosY=0;iPosY<m_iTempStepY;iPosY++)
				{
				memmove(m_pTempStates+iPosY*m_iTempStepX+iScrollX,
						m_pTempStates+iPosY*m_iTempStepX+iScrollY,sizeof(DWORD)*iNumX);
				memset (m_pTempStates+iPosY*m_iTempStepX+iPosX ,0,sizeof(DWORD)*iStepsX);
				}

			ASSERT(m_pTempStates[-1]==0xFDFDFDFD);
			ASSERT(m_pTempStates[m_iTempStepX*m_iTempStepY]==0xFDFDFDFD);
			}
		}

	Unlock();

	m_uNextMode		= 1;
	iFlag			= 1;


	m_sOsmSem.Unlock();
	iTurn++;

	goto Retry;


return TRUE;
}

//*****************************************************************************
//*
//*		SetScrollPath
//*
//*****************************************************************************
//	Scrollt so das das ein ganze Pfad sichtbar ist.
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	iSizeX,iSizeY	: Ist die Fenstergre (bei 0 wird die aktuelle benutzt)
BOOL COsmCtrl::SetScrollPath(unsigned uPath,BOOL bRedraw)
{
LPPOINT	pItems;
double	dMinX;
double	dMaxX;
double	dMinY;
double	dMaxY;
int		iMinX;
int		iMaxX;
int		iMinY;
int		iMaxY;
int		iLen;


	if(uPath>=OSM_PATHS)return FALSE;

	iLen	= m_aPaths[uPath].Count();

	if(iLen==0)return FALSE;

	pItems	= (LPPOINT)m_aPaths[uPath].Get();

	iMinX	= pItems->x;
	iMaxX	= pItems->x;
	iMinY	= pItems->y;
	iMaxY	= pItems->y;

	for(iLen--;iLen>=0;iLen--)
		{
		if(iMinX>pItems[iLen].x)iMinX=pItems[iLen].x;
		if(iMaxX<pItems[iLen].x)iMaxX=pItems[iLen].x;
		if(iMinY>pItems[iLen].y)iMinY=pItems[iLen].y;
		if(iMaxY<pItems[iLen].y)iMaxY=pItems[iLen].y;
		}

	PosToCoord(iMinX,iMinY,dMinX,dMinY);
	PosToCoord(iMaxX,iMaxY,dMaxX,dMaxY);


return SetScrollRect(dMinX,dMinY,dMaxX,dMaxY,bRedraw);
}


//*****************************************************************************
//*
//*		SetScrollRect
//*
//*****************************************************************************
//	Scrollt so das das ein ganzes Rechteck sichtbar ist.
//	dPosX1/dPosY1	: Linke obere   Ecke des Rechtecks
//	dPosX2/dPosY2	: Rechte untere Ecke des Rechtecks
//	bRedraw			: Soll das Fenster neu gezeichnet werden
//	iSizeX,iSizeY	: Ist die Fenstergre (bei 0 wird die aktuelle benutzt)
//	Ergibt TRUE wenn die Scroll-Position verndert wurde.
BOOL COsmCtrl::SetScrollRect(double dPosX1,double dPosY1,double dPosX2,double dPosY2,BOOL bRedraw,int iSizeX,int iSizeY)
{
double	dValX;
double	dValY;
int		iFrameX1;
int		iFrameY1;
int		iFrameX2;
int		iFrameY2;
int		iDifX;
int		iDifY;
int		iPosX;
int		iPosY;
int		iTemp;
int		iZoom;


	if(iSizeX<=0)
		{
		   iSizeX = m_iWndSizeX;
		if(iSizeX<40)iSizeX=40;
		}

	if(iSizeY<=0)
		{
		   iSizeY = m_iWndSizeY;
		if(iSizeY<30)iSizeY=30;
		}

	if(dPosY1> 85.0)dPosY1= 85.0;
	if(dPosY2> 85.0)dPosY2= 85.0;
	if(dPosY1<-85.0)dPosY1=-85.0;
	if(dPosY2<-85.0)dPosY2=-85.0;

	CoordToPos(dPosX1,dPosY1,iFrameX1,iFrameY1);
	CoordToPos(dPosX2,dPosY2,iFrameX2,iFrameY2);


	if(iFrameY1>iFrameY2)
		{
		iTemp		= iFrameY1;
		iFrameY1	= iFrameY2;
		iFrameY2	= iTemp;
		}

	iDifY	=  iFrameY2-iFrameY1;
	iPosY	= (iFrameY1+iFrameY2)/2;

	if(iFrameX1>iFrameX2)
		{
		iPosX	= (iFrameX1+iFrameX2+OSM_ZOOM_MASK)/2;
		iPosX  &=  OSM_ZOOM_MASK;
		iDifX	=  iFrameX2+OSM_ZOOM_MASK-iFrameX1;
		}
	else{
		iPosX	= (iFrameX1+iFrameX2)/2;
		iDifX	=  iFrameX2-iFrameX1;
		}


	if(iDifX<=0)iDifX=1;
	if(iDifY<=0)iDifY=1;

	dValX		= iDifX;
	dValY		= iDifY;
	dValX	   /= iSizeX;
	dValY	   /= iSizeY;

	iZoom		= OSM_ZOOM_BASE-m_iOsmAreaBit;


	if(dValX>=dValY)
		{
		while(iDifX>=iSizeX-30)
			{
			iZoom -= 1;
			iDifX /= 2;
			}
		}
	else{
		while(iDifY>=iSizeY-30)
			{
			iZoom -= 1;
			iDifY /= 2;
			}
		}

	SetZoom(iZoom);


return SetScrollPos(iPosX,iPosY);
}

//*****************************************************************************
//*
//*		SetRangeZoom
//*
//*****************************************************************************
//	Stellt den Bereich fr den Zoom-Faktor ein
void COsmCtrl::SetRangeZoom(int iMin,int iMax)
{

	if(iMin==0 && iMax==0)
		{
		iMin = OSM_ZOOM_MIN;
		iMax = OSM_ZOOM_MAX;
		}

	if(m_iOsmZoomMin<OSM_ZOOM_MIN)
		{
		m_iOsmZoomMin = OSM_ZOOM_MIN;
		}

	if(m_iOsmZoomMin>OSM_ZOOM_MAX)
		{
		m_iOsmZoomMin = OSM_ZOOM_MAX;
		}

	if(m_iOsmZoomMax<OSM_ZOOM_MIN)
		{
		m_iOsmZoomMax = OSM_ZOOM_MIN;
		}

	if(m_iOsmZoomMax>OSM_ZOOM_MAX)
		{
		m_iOsmZoomMax = OSM_ZOOM_MAX;
		}


	if(iMax<iMin)iMax=iMin;

	m_iOsmZoomMin = iMin;
	m_iOsmZoomMax = iMax;

	SetZoom(m_iOsmZoom);

}

//*****************************************************************************
//*
//*		SetRangeX
//*
//*****************************************************************************
//	Stellt den Scroll-Bereich ein fr die X-Richtung
//	iPosX1		: Ist die linke  Koordinate in der OSM-Karte bezogen zu OSM_ZOOM_BASE
//	iPosX2		: Ist die rechte Koordinate in der OSM-Karte bezogen zu OSM_ZOOM_BASE
void COsmCtrl::SetRangeX(int iPosX1,int iPosX2)
{
int		iOldX;


	m_iOsmRangeX1 = iPosX1&OSM_ZOOM_MASK;
	m_iOsmRangeX2 = iPosX2&OSM_ZOOM_MASK;

	iOldX = m_iOsmScrollX;
	m_iOsmScrollX++;
	SetScrollPos(iOldX,m_iOsmScrollY);

}


//*****************************************************************************
//*
//*		SetRangeY
//*
//*****************************************************************************
//	Stellt den Scroll-Bereich ein fr die Y-Richtung
//	iPosY1		: Ist die obere  Koordinate in der OSM-Karte bezogen zu OSM_ZOOM_BASE
//	iPosY2		: Ist die untere Koordinate in der OSM-Karte bezogen zu OSM_ZOOM_BASE
void COsmCtrl::SetRangeY(int iPosY1,int iPosY2)
{
int		iOldX;


	if(iPosY1<0            )iPosY1=0;
	if(iPosY1>OSM_ZOOM_MASK)iPosY1=OSM_ZOOM_MASK;
	if(iPosY2<0            )iPosY2=0;
	if(iPosY2>OSM_ZOOM_MASK)iPosY2=OSM_ZOOM_MASK;
	if(iPosY2<iPosY1       )iPosY2=iPosY1;

	m_iOsmRangeY1 = iPosY1&OSM_ZOOM_MASK;
	m_iOsmRangeY2 = iPosY2&OSM_ZOOM_MASK;

	iOldX = m_iOsmScrollX;
	m_iOsmScrollX++;
	SetScrollPos(iOldX,m_iOsmScrollY);

}


//*****************************************************************************
//*
//*		SetZoom
//*
//*****************************************************************************
//	Stellt den Zoom-Faktor ein (0..N 0=Weltkarte)
//	iZoom		: Ist der neue Zoom-Faktor
//	Ergibt den neuen Zoom-Faktor
int COsmCtrl::SetZoom(int iZoom)
{
DWORD	uStates;
DWORD	uMask;
int		iPixel;
int		iStopX;
int		iStopY;
int		iOldX;
int		iPosX;
int		iPosY;
int		iEndX;
int		iEndY;
int		iPos;


	if(iZoom <m_iOsmZoomMin)iZoom=m_iOsmZoomMin;
	if(iZoom >m_iOsmZoomMax)iZoom=m_iOsmZoomMax;
	if(iZoom==m_iOsmZoom  )return m_iOsmZoom;


	m_sOsmSem.Lock();

	if(m_iOsmZoom+1==iZoom)								// Um eine Stufe nher zoomen
		{
		iEndX	= m_iTempSizeX/2;
		iEndY	= m_iTempSizeY/2;

		for(iPosY=0;iPosY<iEndY;iPosY+=2)
		for(iPosX=0;iPosX<iEndX;iPosX+=2)
			{
			iPixel	= m_pImage->GetPixel(iPosX/2+iEndX/2,iPosY/2+iEndY/2);

			m_pImage->SetPixel(iPosX  ,iPosY  ,iPixel);
			m_pImage->SetPixel(iPosX+1,iPosY  ,iPixel);
			m_pImage->SetPixel(iPosX  ,iPosY+1,iPixel);
			m_pImage->SetPixel(iPosX+1,iPosY+1,iPixel);
			}

		for(iPosY=0             ;iPosY< iEndY;iPosY+=2)
		for(iPosX=m_iTempSizeX-1;iPosX>=iEndX;iPosX-=2)
			{
			iPixel	= m_pImage->GetPixel(iPosX/2+iEndX/2,iPosY/2+iEndY/2);

			m_pImage->SetPixel(iPosX  ,iPosY  ,iPixel);
			m_pImage->SetPixel(iPosX+1,iPosY  ,iPixel);
			m_pImage->SetPixel(iPosX  ,iPosY+1,iPixel);
			m_pImage->SetPixel(iPosX+1,iPosY+1,iPixel);
			}

		for(iPosY=m_iTempSizeY-1;iPosY>=iEndY;iPosY-=2)
		for(iPosX=             0;iPosX< iEndX;iPosX+=2)
			{
			iPixel	= m_pImage->GetPixel(iPosX/2+iEndX/2,iPosY/2+iEndY/2);

			m_pImage->SetPixel(iPosX  ,iPosY  ,iPixel);
			m_pImage->SetPixel(iPosX+1,iPosY  ,iPixel);
			m_pImage->SetPixel(iPosX  ,iPosY+1,iPixel);
			m_pImage->SetPixel(iPosX+1,iPosY+1,iPixel);
			}

		for(iPosY=m_iTempSizeY-1;iPosY>=iEndY;iPosY-=2)
		for(iPosX=m_iTempSizeX-1;iPosX>=iEndX;iPosX-=2)
			{
			iPixel	= m_pImage->GetPixel(iPosX/2+iEndX/2,iPosY/2+iEndY/2);

			m_pImage->SetPixel(iPosX  ,iPosY  ,iPixel);
			m_pImage->SetPixel(iPosX+1,iPosY  ,iPixel);
			m_pImage->SetPixel(iPosX  ,iPosY+1,iPixel);
			m_pImage->SetPixel(iPosX+1,iPosY+1,iPixel);
			}

		for(iPosY=0;iPosY<m_iTempStepY;iPosY++)			// Merker lschen
		for(iPosX=0;iPosX<m_iTempStepX;iPosX++)
			{
			m_pTempStates[iPosX+iPosY*m_iTempStepX] &= ~BIT_TEMP_BIT;
			}

		uMask = m_uLayerMask|BIT_NOT_CLEAR;

		for(iPosY=0;iPosY<m_iTempStepY;iPosY++)			// Merker setzen
		for(iPosX=0;iPosX<m_iTempStepX;iPosX++)
			{
			uStates	= m_pTempStates[iPosX/2+m_iTempStepX/4+(iPosY/2+m_iTempStepY/4)*m_iTempStepX];

			if(uStates&uMask)
				{
				m_pTempStates[iPosX+iPosY*m_iTempStepX]|=BIT_TEMP_BIT;
				}
			}

		for(iPosY=0;iPosY<m_iTempStepY;iPosY++)			// Merker auswerten
		for(iPosX=0;iPosX<m_iTempStepX;iPosX++)
			{
			   uStates = m_pTempStates[iPosX+iPosY*m_iTempStepX];
			if(uStates&BIT_TEMP_BIT)
					m_pTempStates[iPosX+iPosY*m_iTempStepX]=BIT_NOT_CLEAR;
			else	m_pTempStates[iPosX+iPosY*m_iTempStepX]=0;
			}

		}
	else if(m_iOsmZoom-1==iZoom)						// Um eine Stufe weiter weg zoomen
		{
		iEndX	= m_iTempSizeX/2;
		iEndY	= m_iTempSizeY/2;
		iStopX	= m_iTempSizeX*3/4;
		iStopY	= m_iTempSizeY*3/4;

		for(iPosY=iEndY-1;iPosY>=m_iTempSizeY/4;iPosY--)
		for(iPosX=iEndX-1;iPosX>=m_iTempSizeX/4;iPosX--)
			{
			iPixel	= m_pImage->GetPixel(iPosX*2-iEndX,iPosY*2-iEndY);
			m_pImage->SetPixel(iPosX,iPosY,iPixel);
			}

		for(iPosY=iEndY-1;iPosY>=m_iTempSizeY/4;iPosY--)
		for(iPosX=iEndX  ;iPosX< iStopX        ;iPosX++)
			{
			iPixel	= m_pImage->GetPixel(iPosX*2-iEndX,iPosY*2-iEndY);
			m_pImage->SetPixel(iPosX,iPosY,iPixel);
			}

		for(iPosY=iEndY  ;iPosY< iStopY        ;iPosY++)
		for(iPosX=iEndX-1;iPosX>=m_iTempSizeX/4;iPosX--)
			{
			iPixel	= m_pImage->GetPixel(iPosX*2-iEndX,iPosY*2-iEndY);
			m_pImage->SetPixel(iPosX,iPosY,iPixel);
			}

		for(iPosY=iEndY;iPosY<iStopY;iPosY++)
		for(iPosX=iEndX;iPosX<iStopX;iPosX++)
			{
			iPixel	= m_pImage->GetPixel(iPosX*2-iEndX,iPosY*2-iEndY);
			m_pImage->SetPixel(iPosX,iPosY,iPixel);
			}

		for(iPosY=0;iPosY<m_iTempStepY;iPosY++)			// Merker lschen
		for(iPosX=0;iPosX<m_iTempStepX;iPosX++)
			{
			m_pTempStates[iPosX+iPosY*m_iTempStepX] &= ~BIT_TEMP_BIT;
			}

		uMask = m_uLayerMask|BIT_NOT_CLEAR;

		for(iPosY=0;iPosY<m_iTempStepY/2;iPosY++)		// Merker setzen
		for(iPosX=0;iPosX<m_iTempStepX/2;iPosX++)
			{
			uStates	 = m_pTempStates[iPosX*2+  (iPosY*2  )*m_iTempStepX];
			uStates	|= m_pTempStates[iPosX*2+1+(iPosY*2  )*m_iTempStepX];
			uStates	|= m_pTempStates[iPosX*2+  (iPosY*2+1)*m_iTempStepX];
			uStates	|= m_pTempStates[iPosX*2+1+(iPosY*2+1)*m_iTempStepX];

			if(uStates&uMask)
				{
				m_pTempStates[iPosX+m_iTempStepX/4+(iPosY+m_iTempStepY/4)*m_iTempStepX]|=BIT_TEMP_BIT;
				}
			}

		for(iPosY=0;iPosY<m_iTempStepY;iPosY++)			// Merker auswerten
		for(iPosX=0;iPosX<m_iTempStepX;iPosX++)
			{
			   uStates = m_pTempStates[iPosX+iPosY*m_iTempStepX];
			if(uStates&BIT_TEMP_BIT)
					m_pTempStates[iPosX+iPosY*m_iTempStepX]=BIT_NOT_CLEAR;
			else	m_pTempStates[iPosX+iPosY*m_iTempStepX]=0;
			}
		}
	else{
		memset(m_pTempStates,0,sizeof(DWORD)*m_iTempStepX*m_iTempStepY);
		}

	m_iOsmZoom			= iZoom;
	m_iOsmZoomSize		= 1<<m_iOsmZoom;
	m_iOsmZoomMask		= m_iOsmZoomSize-1;
	m_iOsmZoomWitdh		= m_iOsmZoomSize<<m_iOsmAreaBit;
	m_iWayPointViews	= m_iOsmZoomWitdh+m_iWndSizeX+256;
	m_iWayPointViews  >>= m_iOsmZoom+m_iOsmAreaBit;

	if(m_iOsmZoomSize<=m_iTempStepX)
		{
		m_iTempPosX	= 1<<(OSM_ZOOM_BASE-1);

		if(m_iOsmZoom==0)
			{
			m_iTempPosX = 0;
			}
		}
	else{
		m_iTempPosX&= m_iOsmZoomMask<<(OSM_ZOOM_BASE-m_iOsmZoom);
		}

	if(m_iOsmZoomSize<=m_iTempStepY)
		{
		if(m_iOsmZoom!=0)
			{
			m_iTempPosY = 1<<(OSM_ZOOM_BASE-1);

			if(m_iWndSizeX && m_iWndSizeY)
				{
				Lock();
				iPixel = (m_iTempStepY-m_iOsmZoomSize)*(m_iOsmArea/2);
				m_pImage->Box(0,0             ,m_iTempSizeX, iPixel,m_uBkColor);
				m_pImage->Box(0,m_iTempSizeY-1,m_iTempSizeX,-iPixel,m_uBkColor);
				Unlock();
				}
			}
		else{
			m_iTempPosY = 0;

			if(m_iWndSizeX && m_iWndSizeY)
				{
				Lock();
				iPixel = (m_iTempStepY/2)<<m_iOsmAreaBit;
				m_pImage->Box(0,0             ,m_iTempSizeX, iPixel,m_uBkColor);iPixel-=m_iOsmArea;
				m_pImage->Box(0,m_iTempSizeY-1,m_iTempSizeX,-iPixel,m_uBkColor);
				Unlock();
				}
			}
		}
	else{
		m_iTempPosY &= m_iOsmZoomMask<<(OSM_ZOOM_BASE-m_iOsmZoom);
		}

	for(iPos=0;iPos<OSM_JOBS;iPos++)					// Aktuelle Jop's beenden
		{
		CHttpClient	*pHttp;

		if(m_aJops[iPos].uReady)continue;

		   pHttp=(CHttpClient*)m_aJops[iPos].pStop;
		if(pHttp)
			{
			pHttp->Shutdown();
			}
		}

	m_sOsmSem.Unlock();

	iOldX		= m_iOsmScrollX;
	m_iOsmScrollX++;
	SetScrollPos(iOldX,m_iOsmScrollY);


	if(m_iRunning && m_iWndSizeX && m_iWndSizeY)		// Nachricht das ein Job frei ist
		{
		DrawEmpty();

		if(!PostMessage(WM_NEXT_JOB,0))
			{
			Sleep(20);
			PostMessage(WM_NEXT_JOB,0);
			}
		}


return m_iOsmZoom;
}

//*****************************************************************************
//*
//*		SetDrawCallback
//*
//*****************************************************************************
void COsmCtrl::SetDrawCallback(DrawCB pProc,void *pParam)
{
	m_pOsmDrawCB	 = pProc;
	m_pOsmDrawParam  = pParam;
}

//*****************************************************************************
//*
//*		SetPostCallback
//*
//*****************************************************************************
void COsmCtrl::SetPostCallback(DrawCB pProc,void *pParam)
{
	m_pOsmPostCB	 = pProc;
	m_pOsmPostParam  = pParam;
}

//*****************************************************************************
//*
//*		SetMouseCallback
//*
//*****************************************************************************
void COsmCtrl::SetMouseCallback(MouseCB pProc,void *pParam)
{
	m_pOsmMouseCB	 = pProc;
	m_pOsmMouseParam = pParam;
}


//*****************************************************************************
//*
//*		StartSelect
//*
//*****************************************************************************
//	Startet mit einem Auswahlrechteck auf einer OSM-Position
//	iPosX/Y		: OSM-Position der ersten Ecke
//	iOffCmd		: Ist die Abschalte-Message
//	Ergibt 1 wenn die Auswahl gestartet werden konnte
int COsmCtrl::StartSelect(int iPosX,int iPosY,int iOffCmd)
{
OSM_NOTIFY	sNotify;
CPoint		sPoint;
CWnd	   *pWnd;


	if(m_uMouseMode)
		{
		return 0;
		}

	if(m_iXorMode && m_iXorOn)
		{
		DrawXor();
		}

	m_iXorPos1X = iPosX&OSM_ZOOM_MASK;
	m_iXorPos1Y = iPosY&OSM_ZOOM_MASK;

	GetCursorPos  (&sPoint);
	ScreenToClient(&sPoint);

	PixelToPos(sPoint.x,sPoint.y,m_iXorPos2X,m_iXorPos2Y);

	m_uMouseMode= OSM_ACTION_RECT;
	m_uMouseOff	= iOffCmd;
	m_iXorMode  = 1;
	m_iXorOn    = 0;

	SetCapture();
	DrawXor();


	if(m_uWndStyle&OSM_CHANGE_NM)						// XOR-Notify Nachricht
		{
		sNotify.sHdr.code		= OSM_NM_CHANGEXOR;
		sNotify.sHdr.hwndFrom	= m_hWnd;
		sNotify.sHdr.idFrom		= GetWindowLong(m_hWnd,GWL_ID);
		sNotify.iMode			= OSM_ACTION_RECT;
		sNotify.iIndex			= 0;
		sNotify.iPos1X			= m_iXorPos1X;
		sNotify.iPos1Y			= m_iXorPos1Y;
		sNotify.iPos2X			= m_iXorPos2X;
		sNotify.iPos2Y			= m_iXorPos2Y;

		PosToCoord(m_iXorPos1X,m_iXorPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
		PosToCoord(m_iXorPos2X,m_iXorPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

		   pWnd = GetParent();
		if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);
		}




return 1;
}

//*****************************************************************************
//*
//*		StartDraw
//*
//*****************************************************************************
//	Startet mit dem Zeichnen eines Pfades
//	iPath		: Ist die Pfadnummer (0..OSM_PATHS-1, -1 fr den ersten)
//	iIndex		: Ist die Nummer des Punktes (-1 fr am Ende)
//	pPixel		: Ist die Position des neuen Punktes als Pixel im Bild
//	iOffCmd		: Ist die Abschalte-Message
//	Ergibt 1 wenn das Verschieben gestartet werden konnte
int COsmCtrl::StartDraw(int iPath,int iIndex,LPPOINT pPixel,int iOffCmd)
{
OSM_NOTIFY	sNotify;
CPoint		sPoint;
CWnd	   *pWnd;


	if(m_uMouseMode)
		{
		return 0;
		}

	if(m_iXorMode && m_iXorOn)
		{
		DrawXor();
		}

	if(iPath<0)											// Den ersten bearbeitbaren Pfad ndern
		{
		for(iPath=0;;iPath++)
			{
			if(iPath>=OSM_PATHS)return 0;
			if((m_uPathMode>>iPath)&1)break;
			}

		iIndex = m_aPaths[iPath].Count()-1;
		}


	if(iPath>=OSM_PATHS || !((m_uPathMode>>iPath)&1))
		{
		return 0;
		}


	PixelToPos(pPixel->x,pPixel->y,&sPoint);


	if(iIndex<0)										// Am Ende anhngen
		{
		iIndex = m_aPaths[iPath].Count()-1;

		if(iIndex<0)
			{											// Einen neuen Pfad beginnen
			if(AddPathPoint(iPath,sPoint.x,sPoint.y,0,FALSE)<0)
				{
				return 0;
				}

			sNotify.sHdr.code		= OSM_NM_DRAWSTART;
			sNotify.sHdr.hwndFrom	= m_hWnd;
			sNotify.sHdr.idFrom		= GetWindowLong(m_hWnd,GWL_ID);
			sNotify.iMode			= 0;
			sNotify.iIndex			= iPath;
			sNotify.iPos1X			= sPoint.x;
			sNotify.iPos1Y			= sPoint.y;
			sNotify.iPos2X			= 0;
			sNotify.iPos2Y			= 0;
			sNotify.dPos2X			= 0;
			sNotify.dPos2Y			= 0;

			PosToCoord(sNotify.iPos1X,sNotify.iPos1Y,sNotify.dPos1X,sNotify.dPos1Y);

			   pWnd = GetParent();
			if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);

			iIndex = 0;
			}
		}


	if(!m_aPaths[iPath].Check(iIndex))					// Existiert der Punkt
		{
		return 0;
		}

	if(iIndex || m_aPaths[iPath].Count()<2)
		{
		iIndex++;
		}

	   iIndex = AddPathPoint(iPath,sPoint.x,sPoint.y,iIndex,FALSE);
	if(iIndex<0)
		{
		return 0;
		}

	if(m_iPathSelPt>=0)									// Zuvor gewhlten Punkt neu zeichnen
	if(m_iPathSelPt!=m_iPathMovePt || m_iPathSel!=m_iPathMove)
		{
		PathInvalidate(m_iPathSel,m_iPathSelPt,1);
		}

	m_iPathSel		= iPath;
	m_iPathSelPt	= iIndex;
	m_iPathMove		= iPath;
	m_iPathMovePt	= iIndex;
	m_uMouseOff		= iOffCmd;
	m_uMouseMode	= OSM_ACTION_DRAW;
	m_aActionPt.x	= WAS_CHANGED;

	SetCapture();
	PathInvalidate(iPath,iIndex-1,3);



return 1;
}

//*****************************************************************************
//*
//*		StartMove
//*
//*****************************************************************************
//	Startet mit dem Verschieben eine Punktes in einem Pfad
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	iIndex		: Ist die Nummer des Punktes
//	iOffCmd		: Ist die Abschalte-Message
//	Ergibt 1 wenn das Verschieben gestartet werden konnte
int COsmCtrl::StartMove(unsigned uPath,int iIndex,int iOffCmd)
{
LPPOINT		pItems;



	if(m_uMouseMode)
		{
		return 0;
		}

	if(m_iXorMode && m_iXorOn)
		{
		DrawXor();
		}

	if(uPath>=OSM_PATHS || !m_aPaths[uPath].Check(iIndex) || !((m_uPathMode>>uPath)&1))
		{
		return 0;
		}


	if(m_iPathSelPt>=0)									// Zuvor gewhlten Punkt neu zeichnen
	if(m_iPathSelPt!=m_iPathMovePt || m_iPathSel!=m_iPathMove)
		{
		PathInvalidate(m_iPathSel,m_iPathSelPt,1);
		}

	pItems			= (LPPOINT)m_aPaths[uPath].Get(iIndex);
	m_iPathSel		= uPath;
	m_iPathSelPt	= iIndex;
	m_iPathMove		= uPath;
	m_iPathMovePt	= iIndex;
	m_uMouseOff		= iOffCmd;
	m_aPathPoint	= *pItems;
	m_uMouseMode	= OSM_ACTION_POINT;
	m_aActionPt.x	= WAS_CHANGED;

	SetCapture();
	PathInvalidate(uPath,iIndex,1);



return 1;
}

//*****************************************************************************
//*
//*		StartPlace
//*
//*****************************************************************************
//	Startet mit dem Verschieben eines beliebigen Punktes in einem Pfad
//	iPosX/Y		: OSM-Position des Punktes
//	iOffCmd		: Ist die Abschalte-Message
//	iPath		: Ist die Numer des Pfades
//	Ergibt 1 wenn das Verschieben gestartet werden konnte
int COsmCtrl::StartPlace(int iPosX,int iPosY,int iOffCmd,int iPath)
{
OSM_NOTIFY	sNotify;
CPoint		sPoint;
CWnd	   *pWnd;



	if(m_uMouseMode)
		{
		return 0;
		}

	if(m_iXorMode && m_iXorOn)
		{
		DrawXor();
		}

	m_iXorPos1X		= iPosX;
	m_iXorPos1Y		= iPosY;
	m_iXorPos2X		= iPosX;
	m_iXorPos2Y		= iPosY;
	m_uMouseMode	= OSM_ACTION_PLACE;

	SetCapture();

	if(m_uWndStyle&OSM_CHANGE_NM)						// XOR-Notify Nachricht
		{
		sNotify.sHdr.code		= OSM_NM_CHANGEXOR;
		sNotify.sHdr.hwndFrom	= m_hWnd;
		sNotify.sHdr.idFrom		= GetWindowLong(m_hWnd,GWL_ID);
		sNotify.iMode			= OSM_ACTION_PLACE;
		sNotify.iIndex			= iPath;
		sNotify.iPos1X			= m_iXorPos1X;
		sNotify.iPos1Y			= m_iXorPos1Y;
		sNotify.iPos2X			= m_iXorPos2X;
		sNotify.iPos2Y			= m_iXorPos2X;

		PosToCoord(m_iXorPos1X,m_iXorPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
		PosToCoord(m_iXorPos2X,m_iXorPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

		   pWnd = GetParent();
		if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);
		}


return 1;
}

//*****************************************************************************
//*
//*		StartLine
//*
//*****************************************************************************
//	Startet mit einer Linie auf einer OSM-Position
//	iPosX/Y		: OSM-Position der ersten Ecke
//	iOffCmd		: Ist die Abschalte-Message
//	Ergibt 1 wenn die Auswahl gestartet werden konnte
int COsmCtrl::StartLine(int iPosX,int iPosY,int iOffCmd)
{
OSM_NOTIFY	sNotify;
CPoint		sPoint;
CWnd	   *pWnd;



	if(m_uMouseMode)
		{
		return 0;
		}

	if(m_iXorMode && m_iXorOn)
		{
		DrawXor();
		}

	m_iXorPos1X = iPosX&OSM_ZOOM_MASK;
	m_iXorPos1Y = iPosY&OSM_ZOOM_MASK;

	GetCursorPos  (&sPoint);
	ScreenToClient(&sPoint);

	PixelToPos(sPoint.x,sPoint.y,m_iXorPos2X,m_iXorPos2Y);

	m_uMouseMode= OSM_ACTION_LINE;
	m_uMouseOff	= iOffCmd;
	m_iXorMode  = 2;
	m_iXorOn    = 0;

	SetCapture();
	DrawXor();


	if(m_uWndStyle&OSM_CHANGE_NM)						// XOR-Notify Nachricht
		{
		sNotify.sHdr.code		= OSM_NM_CHANGEXOR;
		sNotify.sHdr.hwndFrom	= m_hWnd;
		sNotify.sHdr.idFrom		= GetWindowLong(m_hWnd,GWL_ID);
		sNotify.iMode			= OSM_ACTION_LINE;
		sNotify.iIndex			= 0;
		sNotify.iPos1X			= m_iXorPos1X;
		sNotify.iPos1Y			= m_iXorPos1Y;
		sNotify.iPos2X			= m_iXorPos2X;
		sNotify.iPos2Y			= m_iXorPos2Y;

		PosToCoord(m_iXorPos1X,m_iXorPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
		PosToCoord(m_iXorPos2X,m_iXorPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

		   pWnd = GetParent();
		if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);
		}


return 1;
}


//*****************************************************************************
//*
//*		DrawCallback
//*
//*****************************************************************************
void _cdecl COsmCtrl::DrawCallback(void *pParam,CDC *pDc,PAINTSTRUCT *pPaint)
{
COsmCtrl	   *pThis=(COsmCtrl*)pParam;
OSM_WAYPOINT   *pWayPoint;
BLENDFUNCTION	uBlendFn;
LPCTSTR			pName;
LPPOINT			pPath;
HDC				hPicC;
CRect			sRect;
int				iPicX;
int				iPicY;
int				iPos1X;
int				iPos1Y;
int				iPos2X;
int				iPos2Y;
int				iPosX;
int				iPosY;
int				iOldX;
int				iOldY;
int				iTemp;
int				iPath;
int				iFlag;
int				iTurn;
int				iOffX;
int				iPos;
int				iLen;
int				iPic;


	if(pThis->m_pOsmDrawCB)pThis->m_pOsmDrawCB(pThis->m_pOsmDrawParam,pDc,pPaint);

	if(pThis->m_iLineColor>=0)								// Linie zeichnen
		{
		pDc->SelectStockObject(NULL_BRUSH);
		pDc->SelectStockObject(DC_PEN);
		::SetDCPenColor(pDc->m_hDC,pThis->m_iLineColor);

		pThis->PosToPixel(pThis->m_iLinePos1X,pThis->m_iLinePos1Y,iPos1X,iPos1Y);
		pThis->PosToPixel(pThis->m_iLinePos2X,pThis->m_iLinePos2Y,iPos2X,iPos2Y);

		pDc->MoveTo(iPos1X,iPos1Y);
		pDc->LineTo(iPos2X,iPos2Y);
		}

	if(pThis->m_iRectColor>=0)								// Rechteck zeichnen
		{
		pDc->SelectStockObject(NULL_BRUSH);
		pDc->SelectStockObject(DC_PEN);
		::SetDCPenColor(pDc->m_hDC,pThis->m_iRectColor);

		pThis->PosToPixel(pThis->m_iRectPos1X,pThis->m_iRectPos1Y,iPos1X,iPos1Y);
		pThis->PosToPixel(pThis->m_iRectPos2X,pThis->m_iRectPos2Y,iPos2X,iPos2Y);

		if(iPos1X>iPos2X){iTemp=iPos1X;iPos1X=iPos2X;iPos2X=iTemp;}
		if(iPos1Y>iPos2Y){iTemp=iPos1Y;iPos1Y=iPos2Y;iPos2Y=iTemp;}

		pDc->Rectangle(iPos1X,iPos1Y,iPos2X,iPos2Y);
		}

	if(pThis->m_uPathValid)									// Pfade zeichnen
		{
		for(iPath=0;iPath<OSM_PATHS;iPath++)
			{
			if(!pThis->m_aPathShow[iPath])continue;

			   iLen = pThis->m_aPaths[iPath].Count();
			if(iLen<=1)continue;

			pPath = (LPPOINT)pThis->m_aPaths[iPath].Get();
			iFlag = (pThis->m_uPathMode>>iPath)&1;
			iOldX = -100;
			iOldY = -100;

			iLen--;
			pThis->PosToPixel(pPath[iLen].x,pPath[iLen].y,iPosX,iPosY);
			pDc  ->SelectObject(pThis->m_aPathPens+iPath);
			pDc  ->MoveTo(iPosX,iPosY);

			if(iFlag)										// Punkte ausmahlen
				{
				::SetDCBrushColor(pDc->m_hDC,pThis->m_aPathColor[iPath]);
				pDc->SelectStockObject(DC_BRUSH);
				pDc->Rectangle(iPosX-2,iPosY-2,iPosX+2,iPosY+2);
				}

			for(;iLen>=0;iLen--)
				{
				pThis->PosToPixel(pPath[iLen].x,pPath[iLen].y,iPosX,iPosY);

				if(iOldX==iPosX && iOldY==iPosY)continue;

				pDc->LineTo(iPosX,iPosY);

				if(iFlag)
					{
					pDc->Rectangle(iPosX-2,iPosY-2,iPosX+2,iPosY+2);
					}
				}


			if(iFlag && pThis->m_iPathSel==iPath)			// Ausgewhlten Punkt zeichnen
				{
				iPos = pThis->m_iPathSelPt;

				if(pThis->m_aPaths[iPath].Check(iPos))
					{
					pThis->PosToPixel(pPath[iPos].x,pPath[iPos].y,iPosX,iPosY);
					pDc->SelectStockObject(WHITE_PEN);
					pDc->SelectStockObject(NULL_BRUSH);
					pDc->Rectangle(iPosX-2,iPosY-2,iPosX+2,iPosY+2);
					}
				}
			}
		}


	if(pThis->m_iXorMode)									// XOR-Markierungen
		{
		pThis->m_iXorOn = 0;
		pThis->DrawXor(pDc);
		}


	if(pThis->m_sWayPoints.Count())							// Wegpunkte zeichnen
		{
		uBlendFn.BlendOp				= AC_SRC_OVER;
		uBlendFn.BlendFlags				= 0;
		uBlendFn.SourceConstantAlpha	= 255;
		uBlendFn.AlphaFormat			= AC_SRC_ALPHA;

		pDc->SelectStockObject(ANSI_VAR_FONT);
		pDc->SelectStockObject(WHITE_BRUSH);
		pDc->SelectStockObject(BLACK_PEN);
		pDc->SetTextColor(RGB(0,0,0));
		pDc->SetBkMode(TRANSPARENT);

		pThis->m_sWayPoinSem.Lock();

		for(iPos=0;iPos<pThis->m_sWayPoints.Count();iPos++)	// Alle Wegpunkte durchlaufen
			{
			pWayPoint	= (OSM_WAYPOINT*)pThis->m_sWayPoints.Data(iPos);

			if(!(pWayPoint->iMode&1))continue;

			pThis->PosToPixel(pWayPoint->iPosX,pWayPoint->iPosY,iPosX,iPosY);

			iPic = (pWayPoint->iMode>>2)&0x3F;

			if(pThis->m_aWayPointDc[iPic]==0)
				{
				hPicC  = m_hWayPointDc;
				iPosX -= PIN_OFF_X;
				iPosY -= PIN_OFF_Y;
				iPicX  = PIN_SIZE_X;
				iPicY  = PIN_SIZE_Y;
				}
			else{
				hPicC  = pThis->m_aWayPointDc [iPic];
				iPosX -= pThis->m_aWayPointsX [iPic];
				iPosY -= pThis->m_aWayPointsY [iPic];
				iPicX  = pThis->m_aWayPointsSX[iPic];
				iPicY  = pThis->m_aWayPointsSY[iPic];
				}

			if(pThis->m_iWayPointViews>1)					// Ist Wegpunkt mehrmals sichtbar
				{
				for(;;)
					{
					   iOffX = iPosX+iPicX-pThis->m_iOsmZoomWitdh;
					if(iOffX<=pThis->m_iScrollX)break;
					iPosX  -= pThis->m_iOsmZoomWitdh;
					}

				for(;;)
					{
					   iOffX = iPosX+pThis->m_iOsmZoomWitdh;
					if(iOffX>pThis->m_iScrollX)break;
					iPosX += pThis->m_iOsmZoomWitdh;
					}
				}

			for(iTurn=pThis->m_iWayPointViews;iTurn>0;iTurn--,iPosX+=pThis->m_iOsmZoomWitdh)
				{
				if(!AlphaBlend(pDc->m_hDC,iPosX,iPosY,iPicX,iPicY,hPicC,0,0,iPicX,iPicY,uBlendFn))
					{
					TransparentBlt(pDc->m_hDC,iPosX,iPosY,iPicX,iPicY,hPicC,0,0,iPicX,iPicY,RGB(0,0,0));
					}

				if(!(pWayPoint->iMode&2))continue;

				pName = pThis->m_sWayPoints.Get(iPos);
				iLen  = pThis->m_sWayPoints.Len(iPos);

				if(iLen<=0)continue;

				sRect.left	  = 0;
				sRect.top	  = 0;

				pDc->DrawText(pName,iLen,&sRect,DT_CALCRECT|DT_NOPREFIX|DT_SINGLELINE);

				sRect.right	 += 4;
				sRect.bottom += 2;
				sRect.OffsetRect(iPosX+iPicX+2,iPosY-4);

				::SetDCBrushColor(pDc->m_hDC,pWayPoint->uColor);
				pDc->SelectStockObject(DC_BRUSH);
				pDc->Rectangle(&sRect);
				pDc->DrawText(pName,iLen,&sRect,DT_CENTER|DT_VCENTER|DT_NOPREFIX|DT_SINGLELINE);
				}
			}

		pThis->m_sWayPoinSem.Unlock();
		}


	if(pThis->m_uWndStyle&OSM_MENU)							// Men zeichnen
		{
		uBlendFn.BlendOp				= AC_SRC_OVER;
		uBlendFn.BlendFlags				= 0;
		uBlendFn.SourceConstantAlpha	= 255;
		uBlendFn.AlphaFormat			= AC_SRC_ALPHA;

		if(!AlphaBlend(pDc->m_hDC,pThis->m_iScrollX+pThis->m_iWndSizeX-OSM_MENU_WITDH,pThis->m_iScrollY+10,OSM_MENU_WITDH,OSM_MENU_HEIGHT,m_hMenuDc,0,pThis->m_iMenuHighLight*OSM_MENU_HEIGHT,OSM_MENU_WITDH,OSM_MENU_HEIGHT,uBlendFn))
			{
			TransparentBlt(pDc->m_hDC,pThis->m_iScrollX+pThis->m_iWndSizeX-OSM_MENU_WITDH,pThis->m_iScrollY+10,OSM_MENU_WITDH,OSM_MENU_HEIGHT,m_hMenuDc,0,pThis->m_iMenuHighLight*OSM_MENU_HEIGHT,OSM_MENU_WITDH,OSM_MENU_HEIGHT,RGB(0,0,0));
			}
		}

	if(pThis->m_pOsmPostCB)pThis->m_pOsmPostCB(pThis->m_pOsmPostParam,pDc,pPaint);

}

//*****************************************************************************
//*
//*		MouseCallback
//*
//*****************************************************************************
void _cdecl COsmCtrl::MouseCallback(void *pParam,UINT uType,UINT nFlags,CPoint sPoint)
{
COsmCtrl   *pThis=(COsmCtrl*)pParam;
CWnd	   *pWnd;
OSM_NOTIFY	sNotify;
LPPOINT		pItems;
POINT		sLast;
CRect		sRect;
UINT		uBits;
UINT		uShift;
UINT		uStyle;
short 		sDeltaZ;
int			iPosY;
int			iPosX;
int			iMinY;
int			iMinX;
int			iMaxY;
int			iMaxX;
int			iZoom;
int			iPath;
int			iNum;
int			iVal;


	switch(pThis->m_uMouseMode)
		{
	case OSM_ACTION_MOVE:									// Verschieben

			if(uType==pThis->m_uMouseOff)
				{
				ReleaseCapture();
				pThis->m_uMouseMode = 0;
				return;
				}

			if(uType==WM_MOUSEMOVE)
				{
				pThis->SetScrollEx(pThis->m_iMoveStartX,pThis->m_iMoveStartY,sPoint.x,sPoint.y);
				}

			::SetCursor(IDC_HAND);
			return;

	case OSM_ACTION_RECT:									// Auswahlrechteck

			if(uType==pThis->m_uMouseOff)
				{
				ReleaseCapture();
				if(pThis->m_iXorOn)pThis->DrawXor();

				sNotify.sHdr.code		= OSM_NM_RECTDONE;
				sNotify.sHdr.hwndFrom	= pThis->m_hWnd;
				sNotify.sHdr.idFrom		= GetWindowLong(pThis->m_hWnd,GWL_ID);
				sNotify.iMode			= FALSE;
				sNotify.iIndex			= 0;
				sNotify.iPos1X			= pThis->m_iXorPos1X;
				sNotify.iPos1Y			= pThis->m_iXorPos1Y;
				sNotify.iPos2X			= pThis->m_iXorPos2X;
				sNotify.iPos2Y			= pThis->m_iXorPos2Y;

				pThis->PosToCoord(pThis->m_iXorPos1X,pThis->m_iXorPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
				pThis->PosToCoord(pThis->m_iXorPos2X,pThis->m_iXorPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

				   pWnd = pThis->GetParent();
				if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);

				pThis->m_iXorOn		= 0;
				pThis->m_iXorMode	= 0;
				pThis->m_uMouseMode = 0;
				return;
				}

			pThis->PixelToPos(sPoint.x,sPoint.y,pThis->m_iXorPos2X,pThis->m_iXorPos2Y);

			if(pThis->m_iXorOn)pThis->DrawXor();
			pThis->DrawXor();

			if(pThis->m_uWndStyle&OSM_CHANGE_NM)			// XOR-Notify Nachricht
				{
				sNotify.sHdr.code		= OSM_NM_CHANGEXOR;
				sNotify.sHdr.hwndFrom	= pThis->m_hWnd;
				sNotify.sHdr.idFrom		= GetWindowLong(pThis->m_hWnd,GWL_ID);
				sNotify.iMode			= OSM_ACTION_RECT;
				sNotify.iIndex			= 0;
				sNotify.iPos1X			= pThis->m_iXorPos1X;
				sNotify.iPos1Y			= pThis->m_iXorPos1Y;
				sNotify.iPos2X			= pThis->m_iXorPos2X;
				sNotify.iPos2Y			= pThis->m_iXorPos2Y;

				pThis->PosToCoord(pThis->m_iXorPos1X,pThis->m_iXorPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
				pThis->PosToCoord(pThis->m_iXorPos2X,pThis->m_iXorPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

				   pWnd = pThis->GetParent();
				if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);
				}

			return;

	case OSM_ACTION_LINE:									// Linie

			if(uType==pThis->m_uMouseOff)
				{
				ReleaseCapture();
				if(pThis->m_iXorOn)pThis->DrawXor();

				sNotify.sHdr.code		= OSM_NM_LINEDONE;
				sNotify.sHdr.hwndFrom	= pThis->m_hWnd;
				sNotify.sHdr.idFrom		= GetWindowLong(pThis->m_hWnd,GWL_ID);
				sNotify.iMode			= FALSE;
				sNotify.iIndex			= 0;
				sNotify.iPos1X			= pThis->m_iXorPos1X;
				sNotify.iPos1Y			= pThis->m_iXorPos1Y;
				sNotify.iPos2X			= pThis->m_iXorPos2X;
				sNotify.iPos2Y			= pThis->m_iXorPos2Y;

				pThis->PosToCoord(pThis->m_iXorPos1X,pThis->m_iXorPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
				pThis->PosToCoord(pThis->m_iXorPos2X,pThis->m_iXorPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

				   pWnd = pThis->GetParent();
				if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);

				pThis->m_iXorOn		= 0;
				pThis->m_iXorMode	= 0;
				pThis->m_uMouseMode = 0;
				return;
				}


			pThis->PixelToPos(sPoint.x,sPoint.y,pThis->m_iXorPos2X,pThis->m_iXorPos2Y);

			if(pThis->m_iXorOn)pThis->DrawXor();
			pThis->DrawXor();

			if(pThis->m_uWndStyle&OSM_CHANGE_NM)			// XOR-Notify Nachricht
				{
				sNotify.sHdr.code		= OSM_NM_CHANGEXOR;
				sNotify.sHdr.hwndFrom	= pThis->m_hWnd;
				sNotify.sHdr.idFrom		= GetWindowLong(pThis->m_hWnd,GWL_ID);
				sNotify.iMode			= OSM_ACTION_LINE;
				sNotify.iIndex			= 0;
				sNotify.iPos1X			= pThis->m_iXorPos1X;
				sNotify.iPos1Y			= pThis->m_iXorPos1Y;
				sNotify.iPos2X			= pThis->m_iXorPos2X;
				sNotify.iPos2Y			= pThis->m_iXorPos2Y;

				pThis->PosToCoord(pThis->m_iXorPos1X,pThis->m_iXorPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
				pThis->PosToCoord(pThis->m_iXorPos2X,pThis->m_iXorPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

				   pWnd = pThis->GetParent();
				if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);
				}

			return;

	case OSM_ACTION_POINT:									// Punkt verschieben
	case OSM_ACTION_DRAW:

			iNum   = pThis->m_iPathSelPt;
			iPath  = pThis->m_iPathSel;

			if(uType==pThis->m_uMouseOff)
				{
				if(pThis->m_aActionPt.x!=WAS_CHANGED)		// Gab es keine nderungen
					{
					pThis->CancelAction();
					return;
					}

				ReleaseCapture();

				pThis->m_aPathChg[iPath] = 1;

				if(pThis->m_uMouseMode==OSM_ACTION_DRAW)
					{
					sNotify.sHdr.code	= OSM_NM_DRAWDONE;
					sNotify.iMode		= iNum;
					}
				else{
					sNotify.sHdr.code	= OSM_NM_POINTDONE;
					sNotify.iMode		= FALSE;
					}

				pItems					= (LPPOINT)pThis->m_aPaths[iPath].Get(iNum);
				sNotify.sHdr.hwndFrom	= pThis->m_hWnd;
				sNotify.sHdr.idFrom		= GetWindowLong(pThis->m_hWnd,GWL_ID);
				sNotify.iIndex			= iPath;
				sNotify.iPos1X			= pItems->x;
				sNotify.iPos1Y			= pItems->y;
				sNotify.iPos2X			= pThis->m_aPathPoint.x;
				sNotify.iPos2Y			= pThis->m_aPathPoint.y;

				pThis->PosToCoord(sNotify.iPos1X,sNotify.iPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
				pThis->PosToCoord(sNotify.iPos2X,sNotify.iPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

				   pWnd = pThis->GetParent();
				if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);

				pThis->m_iXorOn			= 0;
				pThis->m_iXorMode		= 0;
				pThis->m_uMouseMode		= 0;
				pThis->PathInvalidate(iPath,iNum,1);

				return;
				}

			if(abs(pThis->m_aActionPt.x-sPoint.x)<2)		// Noch nicht weit genug verschoben
			if(abs(pThis->m_aActionPt.y-sPoint.y)<2)
				{
				::SetCursor(IDC_CROSS);
				return;
				}

			pItems = (LPPOINT)pThis->m_aPaths[iPath].Get(pThis->m_iPathMovePt);
			sLast  = *pItems;
			pThis->PixelToPos(sPoint.x,sPoint.y,pItems);
			pThis->PathInvalidate(iPath,iNum-1,3,&sLast);
			pThis->m_aActionPt.x = WAS_CHANGED;
			::SetCursor(IDC_CROSS);

			return;

	case OSM_ACTION_PLACE:									// Einen Punkt verschieben

			if(uType==pThis->m_uMouseOff)
				{
				ReleaseCapture();
				if(pThis->m_iXorOn)pThis->DrawXor();

				sNotify.sHdr.code		= OSM_NM_PLACEDONE;
				sNotify.sHdr.hwndFrom	= pThis->m_hWnd;
				sNotify.sHdr.idFrom		= GetWindowLong(pThis->m_hWnd,GWL_ID);
				sNotify.iMode			= FALSE;
				sNotify.iIndex			= pThis->m_iPathSel;
				sNotify.iPos1X			= pThis->m_iXorPos1X;
				sNotify.iPos1Y			= pThis->m_iXorPos1Y;
				sNotify.iPos2X			= pThis->m_iXorPos2X;
				sNotify.iPos2Y			= pThis->m_iXorPos2Y;

				pThis->PosToCoord(pThis->m_iXorPos1X,pThis->m_iXorPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
				pThis->PosToCoord(pThis->m_iXorPos2X,pThis->m_iXorPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

				   pWnd = pThis->GetParent();
				if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);

				pThis->m_iXorOn		= 0;
				pThis->m_iXorMode	= 0;
				pThis->m_uMouseMode = 0;
				return;
				}

			pThis->PixelToPos(sPoint.x,sPoint.y,pThis->m_iXorPos1X,pThis->m_iXorPos1Y);

			if(pThis->m_uWndStyle&OSM_CHANGE_NM)			// XOR-Notify Nachricht
				{
				sNotify.sHdr.code		= OSM_NM_CHANGEXOR;
				sNotify.sHdr.hwndFrom	= pThis->m_hWnd;
				sNotify.sHdr.idFrom		= GetWindowLong(pThis->m_hWnd,GWL_ID);
				sNotify.iMode			= OSM_ACTION_PLACE;
				sNotify.iIndex			= pThis->m_iPathSel;
				sNotify.iPos1X			= pThis->m_iXorPos1X;
				sNotify.iPos1Y			= pThis->m_iXorPos1Y;
				sNotify.iPos2X			= pThis->m_iXorPos2X;
				sNotify.iPos2Y			= pThis->m_iXorPos2Y;

				pThis->PosToCoord(pThis->m_iXorPos1X,pThis->m_iXorPos1Y,sNotify.dPos1X,sNotify.dPos1Y);
				pThis->PosToCoord(pThis->m_iXorPos2X,pThis->m_iXorPos2Y,sNotify.dPos2X,sNotify.dPos2Y);

				   pWnd = pThis->GetParent();
				if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);
				}

			return;

		}



	uStyle = pThis->m_uWndStyle;
	iVal   = 0;

	if(uStyle&OSM_MENU)
		{
		if(sPoint.x>pThis->m_iScrollX+pThis->m_iWndSizeX-OSM_MENU_WITDH)
		if(sPoint.y<pThis->m_iScrollY+10+(int)OSM_MENU_HEIGHT)
		if(sPoint.y>pThis->m_iScrollY+10)
			{
			iVal = (sPoint.y-pThis->m_iScrollY-10)/(OSM_MENU_HEIGHT/3)+1;
			}
		}



	if(pThis->m_pOsmMouseCB && !pThis->m_uMouseMode)		// Gibt es einen User-Callback
		{
		nFlags |= iVal<<16;

		if(pThis->m_pOsmMouseCB(pThis->m_pOsmMouseParam,uType,nFlags,sPoint))
			{
			return;
			}
		}

	switch(uType)
		{
	case WM_LBUTTONDOWN:

			pThis->SetFocus();

			if(pThis->m_iMenuHighLight)						// Ist der Cursor ber dem Men
				{
				switch(pThis->m_iMenuHighLight)
					{
				case 1:	pThis->SetZoom(pThis->m_iOsmZoom+1);	break;
				case 2:	pThis->SetZoom(pThis->m_iOsmZoom-1);	break;
				case 3:
						sNotify.sHdr.code		= OSM_NM_CHANGEMAP;
						sNotify.sHdr.hwndFrom	= pThis->m_hWnd;
						sNotify.sHdr.idFrom		= GetWindowLong(pThis->m_hWnd,GWL_ID);
						sNotify.iMode			= FALSE;
						sNotify.iIndex			= 0;
						sNotify.iPos1X			= sPoint.x;
						sNotify.iPos1Y			= sPoint.y;
						sNotify.iPos2X			= sPoint.x-pThis->m_iScrollX;
						sNotify.iPos2Y			= sPoint.y-pThis->m_iScrollY;

						   pWnd = pThis->GetParent();
						if(pWnd)pWnd->SendMessage(WM_NOTIFY,sNotify.sHdr.idFrom,(LPARAM)&sNotify);

						pThis->m_iMenuHighLight = -1;
						break;
					}

				return;
				}

															// Einen Punkt auswhlen
			if(pThis->m_iPathMove>=0 && pThis->m_iPathMovePt>=0)
				{
				if(pThis->m_aMouseAction[0]==OSM_ACTION_DRAW && (pThis->m_iPathMovePt<0 || (nFlags&MK_CONTROL)))
					{
					if((pThis->m_iPathSel  !=pThis->m_iPathMove)||
					   (pThis->m_iPathSelPt!=pThis->m_iPathMovePt))
						{
						iPath				= pThis->m_iPathSel;
						iNum				= pThis->m_iPathSelPt;
						pThis->m_iPathSel	= pThis->m_iPathMove;
						pThis->m_iPathSelPt	= pThis->m_iPathMovePt;
						pThis->PathInvalidate(iPath,iNum,1);
						}

					uType = OSM_ACTION_DRAW;
					}
				else{
					uType = OSM_ACTION_POINT;
					}

				pThis->m_uMouseOff  = WM_LBUTTONUP;
				break;
				}

															// Ist die Maus auf einem Punkt
			if(pThis->m_iPathMovePt>=0 && pThis->m_uPathMode && (pThis->m_uWndStyle&OSM_POINT_CHANGE))
				{
				break;
				}

			if(pThis->m_aMouseAction[0]){uType=pThis->m_aMouseAction[0];pThis->m_uMouseOff=WM_LBUTTONUP;break;}
			if(pThis->m_uWndStyle & OSM_MOVE_L){uType = OSM_ACTION_MOVE;pThis->m_uMouseOff=WM_LBUTTONUP;break;}

			break;

	case WM_RBUTTONDOWN:

			pThis->SetFocus();

			if(pThis->m_aMouseAction[1]){uType=pThis->m_aMouseAction[1];pThis->m_uMouseOff=WM_RBUTTONUP;break;}
			if(pThis->m_uWndStyle & OSM_MOVE_R){uType = OSM_ACTION_MOVE;pThis->m_uMouseOff=WM_RBUTTONUP;break;}
			break;

	case WM_MBUTTONDOWN:

			if(pThis->m_aMouseAction[2]){uType=pThis->m_aMouseAction[2];pThis->m_uMouseOff=WM_MBUTTONUP;break;}
			if(pThis->m_uWndStyle & OSM_MOVE_M){uType = OSM_ACTION_MOVE;pThis->m_uMouseOff=WM_MBUTTONUP;break;}
			break;

	case WM_MOUSEWHEEL:

			if(pThis->m_uWndStyle&OSM_MOUSE_SCROLL)
				{
				sDeltaZ	= HIWORD(nFlags);
				iVal	= 1<<(OSM_ZOOM_BASE-pThis->m_iOsmAreaBit-pThis->m_iOsmZoom-1);
				iVal   *= sDeltaZ;
				pThis->SetScrollPos(pThis->m_iOsmScrollX,pThis->m_iOsmScrollY-iVal);
				break;
				}

			if(!(pThis->m_uWndStyle&OSM_MOUSE_ZOOM))break;

			sDeltaZ	= HIWORD(nFlags);

			     if(sDeltaZ>0)iZoom=pThis->m_iOsmZoom+1;
			else if(sDeltaZ<0)iZoom=pThis->m_iOsmZoom-1;
			else break;

			pThis->PixelToPos(sPoint.x,sPoint.y,iPosX,iPosY);
			sPoint.x -= pThis->m_iScrollX;
			sPoint.y -= pThis->m_iScrollY;
			pThis->SetZoom(iZoom);

			if((UINT)sPoint.x<(UINT)pThis->m_iWndSizeX)
			if((UINT)sPoint.y<(UINT)pThis->m_iWndSizeY)		// Bei Cursor scrollen
				{
				sPoint.x += pThis->m_iScrollX;
				sPoint.y += pThis->m_iScrollY;
				pThis->SetScrollEx(iPosX,iPosY,sPoint.x,sPoint.y);
				}

			break;

	case WM_MOUSEMOVE:

			uStyle = pThis->m_uWndStyle;

			if(uStyle&OSM_MENU)
				{
				if(pThis->m_iMenuHighLight!=iVal)
					{
					pThis->m_iMenuHighLight = iVal;
					pThis->InvalidateRect(pThis->m_iScrollX+pThis->m_iWndSizeX-OSM_MENU_WITDH,pThis->m_iScrollY+10,OSM_MENU_WITDH,OSM_MENU_HEIGHT);
					}

				if(iVal)break;
				}

			pThis->m_iPathMove		= -1;
			pThis->m_iPathMovePt	= -1;

															// Ist der Maus-Cursor auf einem Punkt
			if((uStyle&OSM_POINT_CHANGE) && (pThis->m_uPathMode&pThis->m_uPathValid))
				{
				uBits = pThis->m_uPathMode&pThis->m_uPathValid;

				for(iPath=0;iPath<OSM_PATHS && uBits;iPath++,uBits>>=1)
					{
					if(!(uBits&1))continue;

					if(!pThis->m_aPathShow[iPath])continue;

					pItems = (LPPOINT)pThis->m_aPaths[iPath].Get();
					iNum   =		  pThis->m_aPaths[iPath].Count();
					uShift = (OSM_ZOOM_BASE-pThis->m_iOsmAreaBit-pThis->m_iOsmZoom);
					iVal   = (1<<uShift)*3;

					pThis->PixelToPos(sPoint.x,sPoint.y,iPosX,iPosY);
					iMinX  = iPosX-iVal;
					iMaxX  = iPosX+iVal;
					iMinY  = iPosY-iVal;
					iMaxY  = iPosY+iVal;

					for(iNum--;iNum>=0;iNum--)
						{
						if(pItems[iNum].x<iMinX)continue;
						if(pItems[iNum].x>iMaxX)continue;
						if(pItems[iNum].y<iMinY)continue;
						if(pItems[iNum].y>iMaxY)continue;
															// Mit STRG keine Punke whlen
						if(GetAsyncKeyState(VK_CONTROL)&0x8000)
							{
							break;
							}

						::SetCursor(IDC_CROSS);
						pThis->m_iPathMove	 = iPath;
						pThis->m_iPathMovePt = iNum;
						uBits				 = 0;
						break;
						}
					}
				}

			break;
		}


	switch(uType)
		{
	case OSM_ACTION_MOVE:									// Mit Verschieben beginnen

			::SetCursor(IDC_HAND);
			pThis->SetCapture();
			pThis->m_uMouseMode = uType;
			pThis->PixelToPos(sPoint.x,sPoint.y,pThis->m_iMoveStartX,pThis->m_iMoveStartY);
			break;

	case OSM_ACTION_RECT:									// Mit dem rechteck beginnen

			pThis->PixelToPos(sPoint.x,sPoint.y,iPosX,iPosY);
			pThis->StartSelect(iPosX,iPosY,pThis->m_uMouseOff);
			break;

	case OSM_ACTION_LINE:									// Mit einer Linie beginnen

			pThis->PixelToPos(sPoint.x,sPoint.y,iPosX,iPosY);
			pThis->StartLine(iPosX,iPosY,pThis->m_uMouseOff);
			break;

	case OSM_ACTION_DRAW:									// Einen Pfad zeichnen

			::SetCursor(IDC_CROSS);
			iVal = (pThis->m_iPathMovePt<0 || pThis->m_iPathMove<0);
			if(!pThis->StartDraw(pThis->m_iPathSel,pThis->m_iPathSelPt,&sPoint,pThis->m_uMouseOff))return;
			if(iVal){pThis->m_aActionPt.x=WAS_CHANGED;return;}
			break;

	case OSM_ACTION_POINT:									// Einen Punkt verschieben

			::SetCursor(IDC_CROSS);
			if(!pThis->StartMove(pThis->m_iPathMove,pThis->m_iPathMovePt,pThis->m_uMouseOff))return;
			break;

	case OSM_ACTION_PLACE:									// Einen beliebigen Punkt verschieben

			::SetCursor(IDC_CROSS);
			pThis->PixelToPos(sPoint.x,sPoint.y,iPosX,iPosY);
			if(!pThis->StartPlace(iPosX,iPosY,pThis->m_uMouseOff,pThis->m_iPathMove))return;
			break;

	default:return;
		}


		pThis->m_aActionPt = sPoint;


}

//*****************************************************************************
//*
//*		SetAction
//*
//*****************************************************************************
//	Die Aktion fr eine Maustaste festlegen
//	uButton		: Ist die Taste
//					0 = LBUTTON_DOWN
//					1 = RBUTTON_DOWN
//					2 = MBUTTON_DOWN
//	uAction		: Ist die neue Aktion
//	Ergibt die vorherige Aktion der Taste
UINT COsmCtrl::SetAction(unsigned uButton,UINT uAction)
{
UINT	uOld;


	if(uButton>=3)return OSM_ACTION_NONE;

	uOld = m_aMouseAction[uButton];
	m_aMouseAction[uButton] = uAction;


return uOld;
}

//*****************************************************************************
//*
//*		GetAction
//*
//*****************************************************************************
//	Die aktuelle Aktion fr eine Maustaste abfragen
//	uButton		: Ist die Taste
//					0 = LBUTTON_DOWN
//					1 = RBUTTON_DOWN
//					2 = MBUTTON_DOWN
//	Ergibt die vorherige Aktion der Taste
UINT COsmCtrl::GetAction(unsigned uButton)
{

	if(uButton>=3)return OSM_ACTION_NONE;

return m_aMouseAction[uButton];
}

//*****************************************************************************
//*
//*		SetWheelMode
//*
//*****************************************************************************
//	Stellt den Modus fr das Mausrad ein
//	uMode		: Neuer Modus (0=nichts 1=zoomen 2=scrollen 3=alten Modus holen)
//	Ergibt den alten Modus
UINT COsmCtrl::SetWheelMode(UINT uMode)
{
UINT	uOld;


	if(m_uWndStyle&OSM_MOUSE_SCROLL)
		{
		uOld = 2;
		}
	else if(m_uWndStyle&OSM_MOUSE_ZOOM)
		{
		uOld = 1;
		}
	else{
		uOld = 0;
		}

	if(uMode>2)return uOld;

	if(uOld!=uMode)
		{
		            m_uWndStyle &= ~(OSM_MOUSE_ZOOM|OSM_MOUSE_SCROLL);
		if(uMode==1)m_uWndStyle|=OSM_MOUSE_ZOOM;
		if(uMode==2)m_uWndStyle|=OSM_MOUSE_SCROLL;
		}



return uOld;
}

//*****************************************************************************
//*
//*		GetImage
//*
//*****************************************************************************
BitmapImage	*COsmCtrl::GetImage()
{
return CBitmapWnd::GetImage();
}

//*****************************************************************************
//*
//*		InvalidateWayPoint
//*
//*****************************************************************************
//	Einen Wegpunkt neu zeichnen
//	iIndex		: Ist die Nummer des Wegpunkts
//	iRemove		: Soll die Anzeige des Wegpunkt gelscht werden
void COsmCtrl::InvalidateWayPoint(int iIndex,int iRemove)
{
OSM_WAYPOINT   *pWayPoint;
LPRECT			pCmdRect;
int				iSizeX;
int				iSizeY;
int				iOffX;
int				iPosX;
int				iPosY;
int				iPos;
int				iPic;



	if(!m_bWindowOk)return;

		pWayPoint = (OSM_WAYPOINT*)m_sWayPoints.Data(iIndex);
	if(!pWayPoint || !(pWayPoint->iMode&1))
		{
		return;
		}

	iPic = (pWayPoint->iMode>>2)&0x3F;

	PosToPixel(pWayPoint->iPosX,pWayPoint->iPosY,iPosX,iPosY);

	if(m_aWayPointDc[iPic]==0)
		{
		iPosX  -= PIN_OFF_X;
		iPosY  -= PIN_OFF_Y;
		iSizeX	= PIN_SIZE_X;
		iSizeY  = PIN_SIZE_Y;
		}
	else{
		iPosX  -= m_aWayPointsX [iPic];
		iPosY  -= m_aWayPointsY [iPic];
		iSizeX	= m_aWayPointsSX[iPic];
		iSizeY	= m_aWayPointsSY[iPic];
		}

	if(pWayPoint->iMode&2)
		{
		iPosY  -= 4;
		iSizeY += 4;
		iSizeX += m_sWayPoints.Len(iIndex)*16+20;
		}


	if(iRemove)											// Wegpunkt ausblenden
		{
		pWayPoint->iMode &= ~1;
		}


	if(m_iWayPointViews>1)								// Sind mehrere Wegpunkte sichtbar
		{
		for(;;)
			{
			   iOffX = iPosX+iSizeX-m_iOsmZoomWitdh;
			if(iOffX<=m_iScrollX)break;
			iPosX  -= m_iOsmZoomWitdh;
			}

		for(;;)
			{
			   iOffX = iPosX+m_iOsmZoomWitdh;
			if(iOffX>m_iScrollX)break;
			iPosX += m_iOsmZoomWitdh;
			}
		}


	if(GetCurrentThreadId()==m_uWindowThread)			// Neu zeichnen
		{
		for(iPos=m_iWayPointViews;iPos>0;iPos--,iPosX+=m_iOsmZoomWitdh)
			{
			InvalidateRect(iPosX,iPosY,iSizeX,iSizeY);
			}
		}
	else{
		for(iPos=m_iWayPointViews;iPos>0;iPos--,iPosX+=m_iOsmZoomWitdh)
			{
			pCmdRect		 = new RECT;
			pCmdRect->left	 = iPosX;
			pCmdRect->top	 = iPosY;
			pCmdRect->right	 = iSizeX;
			pCmdRect->bottom = iSizeY;

			if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
				{
				Sleep(20);

				if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
					{
					delete pCmdRect;
					}
				}
			}
		}


}

//*****************************************************************************
//*
//*		OnInvalidate
//*
//*****************************************************************************
LRESULT COsmCtrl::OnInvalidate(WPARAM wParam,LPARAM lParam)
{
LPRECT	pRect=(LPRECT)lParam;


	InvalidateRect(pRect->left,pRect->top,pRect->right,pRect->bottom);
	delete pRect;


return 0;
}

//*****************************************************************************
//*
//*		SetWayPointImg
//*
//*****************************************************************************
//	Stellt ein Bild fr einen Wegpunkt ein
//	uIndex		: Ist der Bildindex (0..63)
//	hBitmap		: Ist das Bitmap-Handle
//	bOffX/Y		: Offset des Nullpunktes im Bild (0..255)
//	bRedraw		: Soll der Wegpunkt neu gezeichnet werden
//	Ergibt TRUE wenn das Bild eingefgt werden konnte
BOOL COsmCtrl::SetWayPointImg(UINT uIndex,HBITMAP hBitmap,BYTE bOffX,BYTE bOffY,BOOL bRedraw)
{
OSM_WAYPOINT   *pWayPoint;
BITMAPINFO		sInfo;
int				iOldX;
int				iOldY;
int				iPosX;
int				iPosY;
int				iPos;


	if(uIndex>63 || hBitmap==0)
		{
		return FALSE;
		}

	if(m_aWayPointDc[uIndex]==0)
		{
		m_aWayPointDc[uIndex] = CreateCompatibleDC(NULL);
		iOldX = 0;
		iOldY = 0;
		}
	else{
		iOldX = m_aWayPointsSX[uIndex];
		iOldY = m_aWayPointsSY[uIndex];
		}

	sInfo.bmiHeader.biSize		  	= sizeof(BITMAPINFOHEADER);
	sInfo.bmiHeader.biWidth		  	= 0;
	sInfo.bmiHeader.biHeight	  	= 0;
	sInfo.bmiHeader.biPlanes	  	= 1;
	sInfo.bmiHeader.biBitCount	  	= 0;
	sInfo.bmiHeader.biCompression 	= BI_RGB;
	sInfo.bmiHeader.biSizeImage		= 0;
	sInfo.bmiHeader.biXPelsPerMeter	= 0;
	sInfo.bmiHeader.biYPelsPerMeter	= 0;

	m_aWayPointDc[uIndex]=CreateCompatibleDC(NULL);
	SelectObject(m_aWayPointDc[uIndex],hBitmap);
	GetDIBits   (m_aWayPointDc[uIndex],hBitmap,0,0,NULL,&sInfo,DIB_RGB_COLORS);

	m_aWayPointsX [uIndex] = bOffX;
	m_aWayPointsY [uIndex] = bOffY;
	m_aWayPointsSX[uIndex] = (BYTE)sInfo.bmiHeader.biWidth;
	m_aWayPointsSY[uIndex] = (BYTE)sInfo.bmiHeader.biHeight;


	if(bRedraw && m_bWindowOk)							// Betroffene Wegunkte neuzeuchnen
		{
		m_sWayPoinSem.Lock();

		for(iPos=0;iPos<m_sWayPoints.Count();iPos++)
			{
			pWayPoint	= (OSM_WAYPOINT*)m_sWayPoints.Data(iPos);

			if(!(pWayPoint->iMode&1))continue;

			PosToPixel(pWayPoint->iPosX,pWayPoint->iPosY,iPosX,iPosY);

			iPosX -= bOffX;
			iPosY -= bOffY;

			if(sInfo.bmiHeader.biWidth <iOldX)sInfo.bmiHeader.biWidth =iOldX;
			if(sInfo.bmiHeader.biHeight<iOldY)sInfo.bmiHeader.biHeight=iOldY;

			InvalidateRect(iPosX,iPosY,sInfo.bmiHeader.biWidth,sInfo.bmiHeader.biHeight);
			}

		m_sWayPoinSem.Unlock();
		}




return TRUE;
}

//*****************************************************************************
//*
//*		DelWayPoint
//*
//*****************************************************************************
//	Lscht einen Wegpunkt aus der internen Liste
//	iPos		: Ist die Nummer des Wegpunktes.
//	bRedraw		: Soll der Wegpunkt neu gezeichnet werden
//	Ergibt die TRUE wenn erfolgreich
BOOL COsmCtrl::DelWayPoint(int iPos,BOOL bRedraw)
{
int		iRet;


	m_sWayPoinSem.Lock();

	if(bRedraw)										// Neu zeichnen
		{
		InvalidateWayPoint(iPos,1);
		}

	iRet = m_sWayPoints.Del(iPos);

	m_sWayPoinSem.Unlock();



return iRet;
}

//*****************************************************************************
//*
//*		DelWayPoints
//*
//*****************************************************************************
//	Lscht alle Wegpunkte
//	bRedraw		: Soll der Wegpunkt neu gezeichnet werden
void COsmCtrl::DelWayPoints(BOOL bRedraw)
{
LPRECT	pCmdRect;
int		iCount;


	m_sWayPoinSem.Lock();

	   iCount = m_sWayPoints.Count();
	if(iCount==0)
		{
		m_sWayPoinSem.Unlock();
		return;
		}

	m_sWayPoints.Del();
	m_sWayPoinSem.Unlock();

	if(bRedraw && iCount && m_bWindowOk)			// Neu zeichnen
		{
		if(GetCurrentThreadId()==m_uWindowThread)
			{
			RedrawWindow();
			}
		else{
			pCmdRect		 = new RECT;
			pCmdRect->left	 = m_iScrollX;
			pCmdRect->top	 = m_iScrollY;
			pCmdRect->right	 = m_iScrollX+m_iWndSizeX;
			pCmdRect->bottom = m_iScrollY+m_iWndSizeY;

			if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
				{
				Sleep(20);

				if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
					{
					delete pCmdRect;
					}
				}
			}
		}


}

//*****************************************************************************
//*
//*		GetWayPoint
//*
//*****************************************************************************
//	Holt die Daten zu einem Wegpunkt
//	iIndex		: Ist die Nummer des Wegpunktes.
//	dPosX/Y		: Hier werden die Koordinaten gespeichert
//	Ergibt einen Zeiger auf den Namen des Wegpunktes, oder 0 bei einem Fehler.
LPCTSTR	COsmCtrl::GetWayPoint(int iIndex,double &dPosX,double &dPosY)
{
OSM_WAYPOINT   *pWayPoint;


	if(iIndex<0 || iIndex>=m_sWayPoints.Count())	// Prfe den Index
		{
		return NULL;
		}

	pWayPoint	= (OSM_WAYPOINT*)m_sWayPoints.Data(iIndex);
	dPosX		= pWayPoint->dPosX;
	dPosY		= pWayPoint->dPosY;


return m_sWayPoints[iIndex];
}

LPCTSTR	COsmCtrl::GetWayPoint(int iIndex,int &iPosX,int &iPosY)
{
OSM_WAYPOINT   *pWayPoint;



	if(iIndex<0 || iIndex>=m_sWayPoints.Count())	// Prfe den Index
		{
		return NULL;
		}

	pWayPoint	= (OSM_WAYPOINT*)m_sWayPoints.Data(iIndex);
	iPosX		= pWayPoint->iPosX;
	iPosY		= pWayPoint->iPosY;


return m_sWayPoints[iIndex];
}

//*****************************************************************************
//*
//*		SetWayPoint
//*
//*****************************************************************************
//	ndert einen Wegpunkt in der internen Liste
//	iIndex		: Ist die Nummer des Wegpunktes.
//	pName		: Ist der Name des Wegpunktes (wenn nicht 0)
//	dPosX/Y		: Ist die Koordinate des Wegpunktes (wenn beide grer -1000)
//	iMode		: Anzeige Modus des Wegpunktes	(wenn ungleich -1)
//					Bit0    = anzeigen
//					Bit1    = Text anzeigen
//					Bit2..7 = Bildnummer
//	uColor		: Ist die Farbe des Textfeldes (0xFFFFFFFF=Keine nderung oder RGB(...))
//	bRedraw		: Soll der Wegpunkt neu gezeichnet werden
//	Ergibt die TRUE wenn erfolgreich
BOOL COsmCtrl::SetWayPoint(int iIndex,LPCTSTR pName,double dPosX,double dPosY,int iMode,COLORREF uColor,BOOL bRedraw)
{
OSM_WAYPOINT   *pWayPoint;
int				iPosX;
int				iPosY;
int				iPic;
int				iNum;
int				iOk;


	if(iIndex<0 || iIndex>=m_sWayPoints.Count())	// Prfe den Index
		{
		return FALSE;
		}


	pWayPoint	= (OSM_WAYPOINT*)m_sWayPoints.Data(iIndex);
	iPic		= (pWayPoint->iMode>>2)&0x3F;
	iOk			= IsWindow(m_hWnd);

	if(iMode==-1)iMode=pWayPoint->iMode;
	if(pName && !_tcscmp(pName,m_sWayPoints[iIndex]))
		{
		pName = 0;
		}

	if(uColor==0xFFFFFFFF)							// Keine Farbnderung
		{
		uColor = pWayPoint->uColor;
		}
													// Auf nderungen prfen
	if(iMode==pWayPoint->iMode && pName==0 && uColor==pWayPoint->uColor)
		{
		CoordToPos(dPosX,dPosY,iPosX,iPosY);

		iNum = OSM_ZOOM_BASE-m_iOsmAreaBit-m_iOsmZoom;

		if(((iPosX^pWayPoint->iPosX)>>iNum)==0)		// Auf der gleichen Position
		if(((iPosY^pWayPoint->iPosY)>>iNum)==0)
			{
			pWayPoint->dPosX = dPosX;
			pWayPoint->dPosY = dPosY;
			pWayPoint->iPosX = iPosX;
			pWayPoint->iPosY = iPosY;

			return TRUE;
			}
		}


	m_sWayPoinSem.Lock();

	if(bRedraw)										// Neu zeichnen
		{
		InvalidateWayPoint(iIndex,1);
		}

	if(pName)										// Neuer Name
		{
		m_sWayPoints.Set(pName,iIndex);
		pWayPoint	= (OSM_WAYPOINT*)m_sWayPoints.Data(iIndex);
		}

	pWayPoint->iMode  = iMode;
	pWayPoint->uColor = uColor;

	if(dPosX>-1000.0 && dPosY>-1000.0)				// Neue Koordinaten
		{
		pWayPoint->dPosX = dPosX;
		pWayPoint->dPosY = dPosY;

		CoordToPos(dPosX,dPosY,pWayPoint->iPosX,pWayPoint->iPosY);
		}

	if(bRedraw)										// Neu zeichnen
		{
		InvalidateWayPoint(iIndex);
		}

	m_sWayPoinSem.Unlock();



return TRUE;
}

//*****************************************************************************
//*
//*		AddWayPoint
//*
//*****************************************************************************
//	Fgt einen Wegpunkt ein in die interne Liste ein
//	pName		: Ist der Name des Wegpunktes
//	dPosX/Y		: Ist die Koordinate des Wegpunktes
//	iMode		: Anzeige Modus des Wegpunktes
//					Bit0    = anzeigen
//					Bit1    = Text anzeigen
//					Bit2..7 = Bildnummer
//	uColor		: Farbe frs Textfeld RGB(...)
//	Ergibt die Indexnummer des Wegpunktes
int	COsmCtrl::AddWayPoint(LPCTSTR pName,double dPosX,double dPosY,int iMode,COLORREF uColor,BOOL bRedraw)
{
OSM_WAYPOINT	sPoint;
int				iIndex;


	sPoint.iMode	= iMode;
	sPoint.dPosX	= dPosX;
	sPoint.dPosY	= dPosY;
	sPoint.uColor	= uColor;

	CoordToPos(sPoint.dPosX,sPoint.dPosY,sPoint.iPosX,sPoint.iPosY);

	m_sWayPoinSem.Lock();
	iIndex = m_sWayPoints.Add(pName,CSC_END,&sPoint);
	m_sWayPoinSem.Unlock();

	if(bRedraw)InvalidateWayPoint(iIndex);


return iIndex;
}

//*****************************************************************************
//*
//*		GetXorRect
//*
//*****************************************************************************
//	Die aktuellen XOR-Koordinaten holen
//	Ergibt die aktuelle Mausaktion
UINT COsmCtrl::GetXorRect(int &iPosX1,int &iPosY1,int &iPosX2,int &iPosY2)
{

	iPosX1	= m_iXorPos1X;
	iPosY1	= m_iXorPos1Y;
	iPosX2	= m_iXorPos2X;
	iPosY2	= m_iXorPos2Y;

return m_uMouseMode;
}

UINT COsmCtrl::GetXorRect(double &dPosX1,double &dPosY1,double &dPosX2,double &dPosY2)
{

	PosToCoord(m_iXorPos1X,m_iXorPos1Y,dPosX1,dPosY1);
	PosToCoord(m_iXorPos2X,m_iXorPos2Y,dPosX2,dPosY2);


return m_uMouseMode;
}



//*****************************************************************************
//*
//*		GetViewRect
//*
//*****************************************************************************
//	Holt die globalen OSM-Koordinaten des anzezeigten Rechtecks
void COsmCtrl::GetViewRect(int &iPosX1,int &iPosY1,int &iPosX2,int &iPosY2)
{

	PixelToPos(m_iScrollX            ,m_iScrollY            ,iPosX1,iPosY1);
	PixelToPos(m_iScrollX+m_iWndSizeX,m_iScrollY+m_iWndSizeY,iPosX2,iPosY2);

}

//*****************************************************************************
//*
//*		SetRect
//*
//*****************************************************************************
//	Zeichnet ein Rechteck in das OSM-Fenster
//	iNewX1/iNewY1	: Erste Ecke (OSM-Koordinate bezogen auf OSM_ZOOM_BASE)
//	iNewX2/iNewY2	: Gegenberligente Ecke (OSM-Koordinate bezogen auf OSM_ZOOM_BASE)
//	uColor			: Farbe des Rechtecks
void COsmCtrl::SetRect(int iNewX1,int iNewY1,int iNewX2,int iNewY2,COLORREF uColor)
{
int			iPos1X;
int			iPos1Y;
int			iPos2X;
int			iPos2Y;
int			iTemp;


	if(m_iRectColor>=0)										// Altes Rechteck lschen
		{
		m_iRectColor = -1;

		PosToPixel(m_iRectPos1X,m_iRectPos1Y,iPos1X,iPos1Y);
		PosToPixel(m_iRectPos2X,m_iRectPos2Y,iPos2X,iPos2Y);

		InvalidateRect(iPos1X,iPos1Y,iPos2X-iPos1X+1,iPos2Y-iPos1Y+1);
		}

 	if(iNewX1>iNewX2){iTemp=iNewX1;iNewX1=iNewX2;iNewX2=iTemp;}
	if(iNewY1>iNewY2){iTemp=iNewY1;iNewY1=iNewY2;iNewY2=iTemp;}


	m_iRectPos1X	= iNewX1;								// Rechteck-Position
	m_iRectPos1Y	= iNewY1;								//		*
	m_iRectPos2X	= iNewX2;								//		*
	m_iRectPos2Y	= iNewY2;								//		*
	m_iRectColor	= uColor&0x00FFFFFF;

	PosToPixel(m_iRectPos1X,m_iRectPos1Y,iPos1X,iPos1Y);
	PosToPixel(m_iRectPos2X,m_iRectPos2Y,iPos2X,iPos2Y);

	InvalidateRect(iPos1X,iPos1Y,iPos2X-iPos1X+1,iPos2Y-iPos1Y+1);
}

//*****************************************************************************
//*
//*		SetLine
//*
//*****************************************************************************
//	Zeichnet eine Linie in das OSM-Fenster
//	iNewX1/iNewY1	: Erster  Punkt der Linie (OSM-Koordinate bezogen auf OSM_ZOOM_BASE)
//	iNewX2/iNewY2	: Zweiter Punkt der Linie (OSM-Koordinate bezogen auf OSM_ZOOM_BASE)
//	uColor			: Farbe der Linie
void COsmCtrl::SetLine(int iNewX1,int iNewY1,int iNewX2,int iNewY2,COLORREF uColor)
{
int			iPos1X;
int			iPos1Y;
int			iPos2X;
int			iPos2Y;
int			iTemp;


	if(m_iLineColor>=0)										// Alte Linie lschen
		{
		m_iLineColor = -1;

		PosToPixel(m_iLinePos1X,m_iLinePos1Y,iPos1X,iPos1Y);
		PosToPixel(m_iLinePos2X,m_iLinePos2Y,iPos2X,iPos2Y);

		InvalidateRect(iPos1X,iPos1Y,iPos2X-iPos1X+1,iPos2Y-iPos1Y+1);
		}

 	if(iNewX1>iNewX2){iTemp=iNewX1;iNewX1=iNewX2;iNewX2=iTemp;}
	if(iNewY1>iNewY2){iTemp=iNewY1;iNewY1=iNewY2;iNewY2=iTemp;}


	m_iLinePos1X	= iNewX1;								// Rechteck-Position
	m_iLinePos1Y	= iNewY1;								//		*
	m_iLinePos2X	= iNewX2;								//		*
	m_iLinePos2Y	= iNewY2;								//		*
	m_iLineColor	= uColor&0x00FFFFFF;


	PosToPixel(m_iRectPos1X,m_iRectPos1Y,iPos1X,iPos1Y);
	PosToPixel(m_iRectPos2X,m_iRectPos2Y,iPos2X,iPos2Y);

	InvalidateRect(iPos1X,iPos1Y,iPos2X-iPos1X+1,iPos2Y-iPos1Y+1);
}

//*****************************************************************************
//*
//*		SetRectOff
//*
//*****************************************************************************
//	Enfernt ein Rechteck aus dem OSM-Fenster
void COsmCtrl::SetRectOff()
{
int			iPos1X;
int			iPos1Y;
int			iPos2X;
int			iPos2Y;


	if(m_iRectColor>=0)										// Altes Rechteck lschen
		{
		m_iRectColor = -1;

		PosToPixel(m_iRectPos1X,m_iRectPos1Y,iPos1X,iPos1Y);
		PosToPixel(m_iRectPos2X,m_iRectPos2Y,iPos2X,iPos2Y);

		InvalidateRect(iPos1X,iPos1Y,iPos2X-iPos1X+1,iPos2Y-iPos1Y+1);
		}

}

//*****************************************************************************
//*
//*		SetLineOff
//*
//*****************************************************************************
//	Entfernt eine Linie aus dem OSM-Fenster
void COsmCtrl::SetLineOff()
{
int			iPos1X;
int			iPos1Y;
int			iPos2X;
int			iPos2Y;


	if(m_iLineColor>=0)										// Alte Linie lschen
		{
		m_iLineColor = -1;

		PosToPixel(m_iLinePos1X,m_iLinePos1Y,iPos1X,iPos1Y);
		PosToPixel(m_iLinePos2X,m_iLinePos2Y,iPos2X,iPos2Y);

		InvalidateRect(iPos1X,iPos1Y,iPos2X-iPos1X+1,iPos2Y-iPos1Y+1);
		}

}


//*****************************************************************************
//*
//*		DoCopy
//*
//*****************************************************************************
//	Kopiert das aktuelle Bild oder eine Auschnitt
//	iMode		: Was soll kopiert werden
//					0 = sichtbares Bild
//					1 = XOR-Markierung
//					2 = Rect-Markierung
//	pImage		: Ist das Ziel in dem das Bild abgelegt wird.
//				  Bei NULL wird in die Zwischenablage kopiert.
//	Ergibt TRUE wenn das Bild kopiert wurde
BOOL COsmCtrl::DoCopy(int iMode,BitmapImage *pImage)
{
BitmapImage	sImage;
int			iPos1X;
int			iPos1Y;
int			iPos2X;
int			iPos2Y;



	if(iMode==1)
		{
		PosToPixel(m_iXorPos1X,m_iXorPos1Y,iPos1X,iPos1Y);
		PosToPixel(m_iXorPos2X,m_iXorPos2Y,iPos2X,iPos2Y);
		}
	else if(iMode==2)
		{
		PosToPixel(m_iRectPos1X,m_iRectPos1Y,iPos1X,iPos1Y);
		PosToPixel(m_iRectPos2X,m_iRectPos2Y,iPos2X,iPos2Y);
		}
	else{
		iPos1X	= m_iScrollX;
		iPos1Y	= m_iScrollY;
		iPos2X	= m_iScrollX+m_iWndSizeX-1;
		iPos2Y	= m_iScrollY+m_iWndSizeY-1;
		}


	if(pImage)
		{
		return pImage->Get(*m_pImage,iPos1X,iPos1Y,iPos2X-iPos1X+1,iPos2Y-iPos1Y+1);
		}


	sImage.Get(*m_pImage,iPos1X,iPos1Y,iPos2X-iPos1X+1,iPos2Y-iPos1Y+1);



return sImage.Copy();
}

//*****************************************************************************
//*
//*		SetRect
//*
//*****************************************************************************
//	Speichert einen Kartenabschnitt ein Rechteck in das OSM-Fenster
//	iPosX/iPosY		: Linke obere Ecke (OSM-Koordinate bezogen auf OSM_ZOOM_BASE)
//	iZoom			: Zoom-Faktor
//	sOutput			: Hier wird der Kartenabschnitt gespeichert. Das Bild
//					  muss auf die richtige Gre ggesetzt werden.
//	pProgress		: Hier wird der Verlauf in 1/1024 gespeichert
//	pBreak			: Hier wird auf Abbruch abgefragt
//	iSizeX/Y		: Ist die Gre des Bildes (bei 0 von sOutput holen)
//	Ergibt 0 wenn ok, oder eine Fehlernummer
//						1 = Falscher Zoomfaktor
//						2 = Das Bild ist nicht in Ordung
//						3 = User-Abbruch
int COsmCtrl::GetMapFrame(int iPosX,int iPosY,int iZoom,BitmapImage &sOutput,int *pProgress,int *pBreak,int iSizeX,int iSizeY)
{
char			cPath[256];
char			cType[256];
char 		   *pBuffer;
ServerEntry	   *pServer;
const char     *pFixed;
const char     *pPath;
BitmapImage		sImage;
unsigned		uErrors;
unsigned		uShift;
unsigned		uLayer;
unsigned		uArea;
unsigned		uBit;
int				iByte;
int				iTemp;
int				iSubX;
int				iSubY;
int				iOffX;
int				iOffY;
int				iNumX;
int				iNumY;
int				iStep;
int				iSize;
int				iMax;
int				iAll;
int				iSet;
int				iNum;
int				iPos;




	if(iZoom<OSM_ZOOM_MIN || iZoom>OSM_ZOOM_MAX)return 1;
	if(sOutput.IsOk()==0)return 2;

	if(iSizeX<=0)iSizeX=sOutput.Width ();
	if(iSizeY<=0)iSizeY=sOutput.Height();

	uShift	= OSM_ZOOM_BASE-OSM_AREA_BITS-iZoom;
	iPosX  &= OSM_ZOOM_MASK;
	iPosY  &= OSM_ZOOM_MASK;
	iPosX >>= uShift;
	iPosY >>= uShift;
	iSubX	= -(iPosX&OSM_AREA_MASK);
	iSubY	= -(iPosY&OSM_AREA_MASK);
	iMax	= 1<<iZoom;
	uArea	= OSM_AREA<<uShift;
	iNumX	= (iSizeX-iSubX+OSM_AREA_MASK)>>OSM_AREA_BITS;
	iNumY	= (iSizeY-iSubY+OSM_AREA_MASK)>>OSM_AREA_BITS;
	iAll	= iNumX*iNumY;
	iStep	= 0;
	iByte	= 0;
	uErrors = 0;
	pBuffer	= 0;

	if(!pProgress)pProgress=&iTemp;

	*pProgress	= 0;


	strlcpy(cPath,m_cCachePath,sizeof(cPath),m_iCacheLen);


	for(iOffY=iSubY;iOffY<iSizeY;iOffY+=OSM_AREA)
		{
		   iNumY = (iPosY+iOffY)/OSM_AREA;
		if(iNumY>=iMax)break;

		for(iOffX=iSubX;iOffX<iSizeX;iOffX+=OSM_AREA)
			{
			iNumX  = (iPosX+iOffX)/OSM_AREA;
			iNumX %=  iMax;

			if(!m_iRunning)return 3;
			if(pBreak && *pBreak)return 3;

															// Alle Layer prfen
			for(uLayer=0,uBit=1;m_uLayerMask&uBit;uLayer++,uBit<<=1)
				{
				if(m_iRunning==0)break;

				pServer	= m_aLayers+uLayer;

				    pPath = pServer->cPath;
				if(*pPath=='/')pPath++;

				if(pServer->ucFixed)
					{
					pFixed = "../OsmFixed/";
					}
				else{
					pFixed = "";
					}

				iPos = strlchar(pServer->cSuffix,'.');

				if(pServer->cSuffix[iPos])					// Gibt es ein Suffix
					{
					iNum = strwlen(pServer->cSuffix+iPos,0x07);
					strlcpy(cType ,pServer->cSuffix+iPos,sizeof(cType),iNum);
					}
				else{
					memcpy(cType,"png",4);
					}

				sprintf(cPath+m_iCacheLen,"%s%s%s%s/%u/%u/%u.%s",pFixed,pServer->cServer,(*pPath)? "/":"",pPath,iZoom,iNumX,iNumY,cType);
				sprintf(m_cDownload,"%s%s/%u/%u/%u%s",(*pPath)? "/":"",pPath,iZoom,iNumX,iNumY,pServer->cSuffix);


				while(m_iRunning)
					{
					if(pBreak && *pBreak)return 3;

					iSize = LengthFile(cPath);

					if(iSize>1)								// Ist die Datei bereits im Cache
						{
						uErrors=0;
						break;
						}

					if(m_sOsmCall.Lock(250))
						{
						iSize = LengthFile(cPath);

						if(iSize>=1)						// Ist die Datei jetzt im Cache
							{
							uErrors = 0;
							break;
							}

						uErrors++;

						if(uErrors>2)
							{
							break;
							}
						}

					m_sOsmSem.Lock();

					for(iPos=0;iPos<OSM_JOBS;iPos++)		// Suche anderem Thread fr Kachel
						{
						if(m_aJops[iPos].uReady)
							{
							iSet = iPos;
							continue;
							}

						if(m_aJops[iPos].uIdxX==(UINT)iNumX)
						if(m_aJops[iPos].uIdxY==(UINT)iNumY)
							{
							break;
							}
						}

					if(iPos<OSM_JOBS || iSet<0)				// Wird die Kachel gerade geladen
						{
						m_sOsmSem.Unlock();
						continue;
						}

					m_uJopTurn++;

					m_aJops[iSet].uIdxX		= iNumX;
					m_aJops[iSet].uIdxY		= iNumY;
					m_aJops[iSet].uZoom		= iZoom;
					m_aJops[iSet].uTurnId	= m_uTurnId;
					m_aJops[iSet].uTurn		= m_uJopTurn;
					m_aJops[iSet].uExtern	= 1;
					m_aJops[iSet].uReady	= 0;
					m_aJops[iSet].sNext.Unlock();			// Den Job starten

					m_sOsmSem.Unlock();
					}


				if(uErrors)									// Gab es einen Fehler
					{
					uErrors=0;
					continue;
					}

				if(iSize>iByte)
					{
					if(pBuffer)delete pBuffer;
					iByte	=  (iSize+256+0x3FFF)&~0x3FFF;
					pBuffer	=  new char[iByte];
					iByte  -= 256;
					}

				if(LoadMemory(cPath,pBuffer,iByte+256,1)!=iSize)
					{
					continue;
					}

				if(ImageLoader(pBuffer,iSize,&sImage))
					{
					continue;
					}

				if(sImage.Width ()!=OSM_BLOCK_AREA)continue;
				if(sImage.Height()!=OSM_BLOCK_AREA)continue;

				sOutput.SetUserData(0);
				sOutput.Put(sImage,iOffX,iOffY);
				}

			iStep += 1;

			*pProgress = (8192*iStep)/iAll;
			}
		}

	*pProgress = 8192;

	if(pBuffer)delete pBuffer;


return 0;
}

//*****************************************************************************
//*
//*		AddPathPoint
//*
//*****************************************************************************
//	Fgt einen Punkt in eine Pfad ein
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	iPosX/Y		: OSM-Koordinate vom Punkt bezogen auf OSM_ZOOM_BASE
//	iIndex		: Index wo der Punkt eingefgt werden soll
//	bRedraw		: Soll das Fenser neugezeichnet werden.
//	Ergibt die Einfge Position bzw. -1 bei einem Fehler
int	COsmCtrl::AddPathPoint(unsigned uPath,int iPosX,int iPosY,int iIndex,BOOL bRedraw)
{
LPRECT	pCmdRect;
LPPOINT	pItems;
POINT	sPoint;
int		iMinX;
int		iMinY;
int		iMaxX;
int		iMaxY;
int		iBit;



	if(uPath>=OSM_PATHS)return -1;							// Prfe die Pfadnummer


	m_sPathSem.Lock();

	sPoint.x	= iPosX;
	sPoint.y	= iPosY;
	iIndex		= m_aPaths[uPath].Add(&sPoint,iIndex);


	if(iIndex==0)											// Wurde der Pfad neu erzeugt
		{
		iBit	= 1<<uPath;

		if(!(m_uPathValid&iBit))
			{
			m_uPathValid |= iBit;
			m_aPathPens[uPath].CreatePen(PS_SOLID|PS_ENDCAP_ROUND,m_aPathWidth[uPath],m_aPathColor[uPath]);
			m_aPathChg [uPath] = 0;
			}
		}


	if(bRedraw && m_bWindowOk)								// Genderte Zone neu zeichnen
		{
		iMinX	= iPosX;
		iMaxX	= iPosX;
		iMinY	= iPosY;
		iMaxY	= iPosY;

		pItems	= (LPPOINT)m_aPaths[uPath].Get(iIndex);

		if(iIndex>0)
			{
			if(iMinX>pItems[-1].x)
					iMinX=pItems[-1].x;
			else	iMaxX=pItems[-1].x;

			if(iMinY>pItems[-1].y)
					iMinY=pItems[-1].y;
			else	iMaxY=pItems[-1].y;
			}

		if(iIndex+1<m_aPaths[uPath].Count())
			{
			if(iMinX>pItems[1].x)iMinX=pItems[1].x;
			if(iMaxX<pItems[1].x)iMaxX=pItems[1].x;
			if(iMinY>pItems[1].y)iMinY=pItems[1].y;
			if(iMaxY<pItems[1].y)iMaxY=pItems[1].y;
			}

		PosToPixel(iMinX,iMinY,iMinX,iMinY);
		PosToPixel(iMaxX,iMaxY,iMaxX,iMaxY);


		if(GetCurrentThreadId()==m_uWindowThread)			// Sind wir im Fenster Thread
			{
			InvalidateRect(iMinX-PATH_FRAME,iMinY-PATH_FRAME,iMaxX-iMinX+PATH_FRAME*2,iMaxY-iMinY+PATH_FRAME*2);
			}
		else{
			pCmdRect		 = new RECT;
			pCmdRect->left	 = iMinX-PATH_FRAME;
			pCmdRect->top	 = iMinY-PATH_FRAME;
			pCmdRect->right	 = iMaxX-iMinX+PATH_FRAME*2;
			pCmdRect->bottom = iMaxY-iMinY+PATH_FRAME*2;

			if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
				{
				Sleep(20);

				if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
					{
					delete pCmdRect;
					}
				}
			}
		}

	m_sPathSem.Unlock();


return iIndex;
}

//*****************************************************************************
//*
//*		GetPathPoint
//*
//*****************************************************************************
//	Holt einen Punkt aus eine Pfad
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	iPosX/Y		: Hier wird die OSM-Koordinate vom Punkt gespeichert
//	iIndex		: Index vom Punkt im Pfad
//	Ergibt die Position bzw. -1 bei einem Fehler
int	COsmCtrl::GetPathPoint(unsigned uPath,int &iPosX,int &iPosY,int iIndex)
{
LPPOINT	pItems;


	if(uPath>=OSM_PATHS)return -1;							// Prfe die Pfadnummer
	if(!m_aPaths[uPath].Check(iIndex))return -1;

	pItems	= (LPPOINT)m_aPaths[uPath].Get(iIndex);
	iPosX	= pItems->x;
	iPosY	= pItems->y;


return iIndex;
}

//*****************************************************************************
//*
//*		DelPathPoint
//*
//*****************************************************************************
//	Lscht einen Punkt in einem Pfad
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	iIndex		: Index wo des Punkts der gelscht werden soll
//	bRedraw		: Soll das Fenser neugezeichnet werden.
//	Ergibt TRUE wenn der Punkt gelscht werden konnte
BOOL COsmCtrl::DelPathPoint(unsigned uPath,int iIndex,BOOL bRedraw)
{
LPRECT	pCmdRect;
LPPOINT	pItems;
int		iMinX;
int		iMinY;
int		iMaxX;
int		iMaxY;
int		iBit;



	if(uPath>=OSM_PATHS)return FALSE;						// Prfe die Pfadnummer
	if(!m_aPaths[uPath].Check(iIndex))return FALSE;


	m_sPathSem.Lock();

	if(bRedraw)												// Genderte Zone berechnen
		{
		pItems	= (LPPOINT)m_aPaths[uPath].Get(iIndex);
		iMinX	= pItems->x;
		iMaxX	= pItems->x;
		iMinY	= pItems->y;
		iMaxY	= pItems->y;

		if(iIndex>0)
			{
			if(iMinX>pItems[-1].x)
					iMinX=pItems[-1].x;
			else	iMaxX=pItems[-1].x;

			if(iMinY>pItems[-1].y)
					iMinY=pItems[-1].y;
			else	iMaxY=pItems[-1].y;
			}

		if(iIndex+1<m_aPaths[uPath].Count())
			{
			if(iMinX>pItems[1].x)iMinX=pItems[1].x;
			if(iMaxX<pItems[1].x)iMaxX=pItems[1].x;
			if(iMinY>pItems[1].y)iMinY=pItems[1].y;
			if(iMaxY<pItems[1].y)iMaxY=pItems[1].y;
			}
		}

	if(m_iPathSel==(int)uPath)								// Gewhlten Punkt lschen
		{
		if(m_iPathSelPt==iIndex)
			{
			m_iPathSel		= -1;
			m_iPathSelPt	= -1;
			}
		else if(m_iPathSelPt>iIndex)
			{
			m_iPathSelPt   -=  1;
			}
		}

	m_aPaths[uPath].Del(iIndex);

	if(m_aPaths[uPath].Count()==0)							// Wurde der Pfad gelscht
		{
		iBit	      = 1<<uPath;
		m_uPathValid &= ~iBit;
		m_aPathPens[uPath].DeleteObject();
		}

	if(bRedraw && m_bWindowOk)								// Genderte Zone neu zeichnen
		{
		PosToPixel(iMinX,iMinY,iMinX,iMinY);
		PosToPixel(iMaxX,iMaxY,iMaxX,iMaxY);

		if(GetCurrentThreadId()==m_uWindowThread)			// Sind wir im Fenster Thread
			{
			InvalidateRect(iMinX-PATH_FRAME,iMinY-PATH_FRAME,iMaxX-iMinX+PATH_FRAME*2,iMaxY-iMinY+PATH_FRAME*2);
			}
		else{
			pCmdRect		 = new RECT;
			pCmdRect->left	 = iMinX-PATH_FRAME;
			pCmdRect->top	 = iMinY-PATH_FRAME;
			pCmdRect->right	 = iMaxX-iMinX+PATH_FRAME*2;
			pCmdRect->bottom = iMaxY-iMinY+PATH_FRAME*2;

			if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
				{
				Sleep(20);

				if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
					{
					delete pCmdRect;
					}
				}
			}
		}

	m_sPathSem.Unlock();


return TRUE;
}

//*****************************************************************************
//*
//*		DelPath
//*
//*****************************************************************************
//	Lscht einen ganzen Pfad
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	bRedraw		: Soll das Fenser neugezeichnet werden.
//	Ergibt TRUE wenn der Pfad gelscht werden konnte
BOOL COsmCtrl::DelPath(unsigned uPath,BOOL bRedraw)
{
LPRECT	pCmdRect;
LPPOINT	pItems;
int		iMinX;
int		iMinY;
int		iMaxX;
int		iMaxY;
int		iBit;
int		iLen;
int		iPos;



	if(uPath>=OSM_PATHS)return FALSE;						// Prfe die Pfadnummer


	m_sPathSem.Lock();

	iLen = m_aPaths[uPath].Count();

	if(iLen==0)
		{
		m_aPathChg[uPath] = 0;
		m_sPathSem.Unlock();
		return FALSE;
		}

 	if(bRedraw)												// Genderte Zone berechnen
		{
		pItems	= (LPPOINT)m_aPaths[uPath].Get();
		iMinX	= pItems->x;
		iMaxX	= pItems->x;
		iMinY	= pItems->y;
		iMaxY	= pItems->y;

		for(iPos=1;iPos<iLen;iPos++)
			{
			if(iMinX>pItems[iPos].x)iMinX=pItems[iPos].x;
			if(iMaxX<pItems[iPos].x)iMaxX=pItems[iPos].x;
			if(iMinY>pItems[iPos].y)iMinY=pItems[iPos].y;
			if(iMaxY<pItems[iPos].y)iMaxY=pItems[iPos].y;
			}
		}


	if(m_iPathSel==(int)uPath)								// Gewhlten Punkt lschen
		{
		m_iPathSel			=-1;
		m_iPathSelPt		=-1;
		}


	m_aPaths[uPath].Del();

 	iBit	      = 1<<uPath;
	m_uPathValid &= ~iBit;
	m_aPathChg [uPath] = 0;
	m_aPathPens[uPath].DeleteObject();


 	if(bRedraw && m_bWindowOk)								// Genderte Zone neu zeichnen
		{
		PosToPixel(iMinX,iMinY,iMinX,iMinY);
		PosToPixel(iMaxX,iMaxY,iMaxX,iMaxY);


		if(GetCurrentThreadId()==m_uWindowThread)			// Sind wir im Fenster Thread
			{
			InvalidateRect(iMinX-PATH_FRAME,iMinY-PATH_FRAME,iMaxX-iMinX+PATH_FRAME*2,iMaxY-iMinY+PATH_FRAME*2);
			}
		else{
			pCmdRect		 = new RECT;
			pCmdRect->left	 = iMinX-PATH_FRAME;
			pCmdRect->top	 = iMinY-PATH_FRAME;
			pCmdRect->right	 = iMaxX-iMinX+PATH_FRAME*2;
			pCmdRect->bottom = iMaxY-iMinY+PATH_FRAME*2;

			if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
				{
				Sleep(20);

				if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
					{
					delete pCmdRect;
					}
				}
			}
		}

	m_sPathSem.Unlock();


return TRUE;
}

//*****************************************************************************
//*
//*		SetPathPen
//*
//*****************************************************************************
//	Holt die Farbe und Breite eine Pfades
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	uColor		: Ist die neue Farbe fr den Pfad
//	uWidth		: Ist die neu Breite
//	bRedraw		: Soll das Fenser neugezeichnet werden.
//	Ergibt TRUE wenn die Farbe gendert werden konnte
BOOL COsmCtrl::GetPathPen(unsigned uPath,COLORREF &uColor,unsigned &uWidth)
{

	if(uPath>=OSM_PATHS)return FALSE;						// Prfe die Pfadnummer

	uColor = m_aPathColor[uPath];							// Keine nderungen
	uWidth = m_aPathWidth[uPath];


return TRUE;
}

//*****************************************************************************
//*
//*		SetPathPen
//*
//*****************************************************************************
//	Stellt die Farbe eine Pfades ein
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	uColor		: Ist die neue Farbe fr den Pfad
//	uWidth		: Ist die neu Breite
//	bRedraw		: Soll das Fenser neugezeichnet werden.
//	Ergibt TRUE wenn die Farbe gendert werden konnte
BOOL COsmCtrl::SetPathPen(unsigned uPath,COLORREF uColor,unsigned uWidth,BOOL bRedraw)
{
LPRECT	pCmdRect;
LPPOINT	pItems;
int		iMinX;
int		iMinY;
int		iMaxX;
int		iMaxY;
int		iLen;
int		iPos;



	if(uPath>=OSM_PATHS)return FALSE;						// Prfe die Pfadnummer

	if(uWidth>8)uWidth=8;
	uColor &= 0x00FFFFFF;

	m_sPathSem.Lock();

	if(uColor==m_aPathColor[uPath])							// Keine nderungen
	if(uWidth==m_aPathWidth[uPath])
		{
		m_sPathSem.Unlock();
		return TRUE;
		}

	m_aPathColor[uPath]	= uColor;
	m_aPathWidth[uPath] = uWidth;

	iLen = m_aPaths[uPath].Count();

	if(iLen==0)
		{
		m_sPathSem.Unlock();
		return TRUE;
		}


 	if(bRedraw)												// Genderte Zone berechnen
		{
		pItems	= (LPPOINT)m_aPaths[uPath].Get();
		iMinX	= pItems->x;
		iMaxX	= pItems->x;
		iMinY	= pItems->y;
		iMaxY	= pItems->y;

		for(iPos=1;iPos<iLen;iPos++)
			{
			if(iMinX>pItems[iPos].x)iMinX=pItems[iPos].x;
			if(iMaxX<pItems[iPos].x)iMaxX=pItems[iPos].x;
			if(iMinY>pItems[iPos].y)iMinY=pItems[iPos].y;
			if(iMaxY<pItems[iPos].y)iMaxY=pItems[iPos].y;
			}
		}


	m_aPathPens[uPath].DeleteObject();
	m_aPathPens[uPath].CreatePen(PS_SOLID|PS_ENDCAP_ROUND,m_aPathWidth[uPath],m_aPathColor[uPath]|0x80000);


 	if(bRedraw && m_bWindowOk)								// Genderte Zone neu zeichnen
		{
		PosToPixel(iMinX,iMinY,iMinX,iMinY);
		PosToPixel(iMaxX,iMaxY,iMaxX,iMaxY);

		if(GetCurrentThreadId()==m_uWindowThread)			// Sind wir im Fenster Thread
			{
			InvalidateRect(iMinX-PATH_FRAME,iMinY-PATH_FRAME,iMaxX-iMinX+PATH_FRAME*2,iMaxY-iMinY+PATH_FRAME*2);
			}
		else{
			pCmdRect		 = new RECT;
			pCmdRect->left	 = iMinX-PATH_FRAME;
			pCmdRect->top	 = iMinY-PATH_FRAME;
			pCmdRect->right	 = iMaxX-iMinX+PATH_FRAME*2;
			pCmdRect->bottom = iMaxY-iMinY+PATH_FRAME*2;

			if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
				{
				Sleep(20);

				if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
					{
					delete pCmdRect;
					}
				}
			}
		}

	m_sPathSem.Unlock();


return TRUE;
}


//*****************************************************************************
//*
//*		SetPathMode
//*
//*****************************************************************************
//	Stellt den Punkt-Modus fr einen Pfad ein
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	uMode		: Neue Modus (0=nur Pfad  1=nderbar)
//	bRedraw		: Soll das Fenser neugezeichnet werden.
//	Ergibt TRUE wenn der Modus gendert werden konnte
BOOL COsmCtrl::SetPathMode(unsigned uPath,unsigned uMode,BOOL bRedraw)
{
LPRECT	pCmdRect;
LPPOINT	pItems;
UINT	uValue;
int		iMinX;
int		iMinY;
int		iMaxX;
int		iMaxY;
int		iLen;
int		iPos;



	if(uPath>=OSM_PATHS)return FALSE;						// Prfe die Pfadnummer

	uMode  &= 1;
	uValue	= (m_uPathMode>>uPath)&1;

	if(uMode==uValue)
		{
		return TRUE;
		}

	m_sPathSem.Lock();

	iLen = m_aPaths[uPath].Count();

 	if(bRedraw && iLen)										// Genderte Zone berechnen
		{
		pItems	= (LPPOINT)m_aPaths[uPath].Get();
		iMinX	= pItems->x;
		iMaxX	= pItems->x;
		iMinY	= pItems->y;
		iMaxY	= pItems->y;

		for(iPos=1;iPos<iLen;iPos++)
			{
			if(iMinX>pItems[iPos].x)iMinX=pItems[iPos].x;
			if(iMaxX<pItems[iPos].x)iMaxX=pItems[iPos].x;
			if(iMinY>pItems[iPos].y)iMinY=pItems[iPos].y;
			if(iMaxY<pItems[iPos].y)iMaxY=pItems[iPos].y;
			}
		}


	m_uPathMode ^= 1<<uPath;


	if(bRedraw && m_bWindowOk)								// Genderte Zone neu zeichnen
		{
		PosToPixel(iMinX,iMinY,iMinX,iMinY);
		PosToPixel(iMaxX,iMaxY,iMaxX,iMaxY);

		if(GetCurrentThreadId()==m_uWindowThread)			// Sind wir im Fenster Thread
			{
			InvalidateRect(iMinX-PATH_FRAME,iMinY-PATH_FRAME,iMaxX-iMinX+PATH_FRAME*2,iMaxY-iMinY+PATH_FRAME*2);
			}
		else{
			pCmdRect		 = new RECT;
			pCmdRect->left	 = iMinX-PATH_FRAME;
			pCmdRect->top	 = iMinY-PATH_FRAME;
			pCmdRect->right	 = iMaxX-iMinX+PATH_FRAME*2;
			pCmdRect->bottom = iMaxY-iMinY+PATH_FRAME*2;

			if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
				{
				Sleep(20);

				if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
					{
					delete pCmdRect;
					}
				}
			}
		}

	m_sPathSem.Unlock();


return TRUE;
}

//*****************************************************************************
//*
//*		PathInvalidate
//*
//*****************************************************************************
//	Zeichnet den Bereich einiger Punkte eines Pfads neu
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	iStart		: Erster Punkt
//	iCount		: Anzahl der Punkte
//	Ergibt die Einfge Position bzw. -1 bei einem Fehler
void COsmCtrl::PathInvalidate(unsigned uPath,int iStart,int iCount,LPPOINT pLast)
{
LPRECT	pCmdRect;
LPPOINT	pItems;
int		iMinX;
int		iMinY;
int		iMaxX;
int		iMaxY;
int		iEnd;



	if(uPath>=OSM_PATHS)return;								// Prfe die Pfadnummer

	if(iStart<0)
		{
		iCount += iStart;
		iStart	= 0;
		}

	   iEnd = iStart+iCount;
	if(iEnd>m_aPaths[uPath].Count())
		{
		iEnd = m_aPaths[uPath].Count();
		}


	if(iStart>=iEnd)return;


	pItems = (LPPOINT)m_aPaths[uPath].Get();

	if(pLast)
		{
		iMinX  = pLast->x;
		iMaxX  = pLast->x;
		iMinY  = pLast->y;
		iMaxY  = pLast->y;
		}
	else{
		iMinX  = pItems[iStart].x;
		iMaxX  = pItems[iStart].x;
		iMinY  = pItems[iStart].y;
		iMaxY  = pItems[iStart].y;
		iStart++;
		}

	for(;iStart<iEnd;iStart++)
		{
		if(iMinX>pItems[iStart].x)iMinX=pItems[iStart].x;
		if(iMaxX<pItems[iStart].x)iMaxX=pItems[iStart].x;
		if(iMinY>pItems[iStart].y)iMinY=pItems[iStart].y;
		if(iMaxY<pItems[iStart].y)iMaxY=pItems[iStart].y;
		}


	if(m_bWindowOk)											// Genderte Zone neu zeichnen
		{
		PosToPixel(iMinX,iMinY,iMinX,iMinY);
		PosToPixel(iMaxX,iMaxY,iMaxX,iMaxY);

		if(GetCurrentThreadId()==m_uWindowThread)			// Sind wir im Fenster Thread
			{
			InvalidateRect(iMinX-PATH_FRAME,iMinY-PATH_FRAME,iMaxX-iMinX+PATH_FRAME*2,iMaxY-iMinY+PATH_FRAME*2);
			}
		else{
			pCmdRect		 = new RECT;
			pCmdRect->left	 = iMinX-PATH_FRAME;
			pCmdRect->top	 = iMinY-PATH_FRAME;
			pCmdRect->right	 = iMaxX-iMinX+PATH_FRAME*2;
			pCmdRect->bottom = iMaxY-iMinY+PATH_FRAME*2;

			if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
				{
				Sleep(20);

				if(!PostMessage(WM_INVALIDATE,0,(LPARAM)pCmdRect))
					{
					delete pCmdRect;
					}
				}
			}
		}

}


//*****************************************************************************
//*
//*		SetPathVisible
//*
//*****************************************************************************
//	Macht einen Pfad sichtbar oder unsichtbar
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	bShow		: Soll der Pfad sichbar werden
//	Ergibt den alten Zustand
BOOL COsmCtrl::SetPathVisible(unsigned uPath,BOOL bShow)
{



	if(uPath>=OSM_PATHS)return FALSE;
	if(m_aPathShow[uPath]==bShow)return m_aPathShow[uPath];


	m_aPathShow[uPath] = bShow;
	PathInvalidate(uPath,0,m_aPaths[uPath].Count());



return !bShow;
}

//*****************************************************************************
//*
//*		GetPathLength
//*
//*****************************************************************************
//	Berechnet die Lnge eines Pfades
//	uPath		: Ist die Pfadnummer (0..OSM_PATHS-1)
//	Ergibt die Lnge des Pfades in Metern.
double COsmCtrl::GetPathLength(unsigned uPath)
{
LPPOINT	pItems;
double	dLastY;
double	dLastX;
double	dPosY;
double	dPosX;
double	dLen;
int		iNum;
int		iPos;



	if(uPath>=OSM_PATHS)return 0;


	pItems	= (LPPOINT)m_aPaths[uPath].Get();
	iNum	= m_aPaths[uPath].Count();
	dLen	= 0.0;

	if(iNum==0)return 0;

	PosToCoord(pItems[0].x,pItems[0].y,dLastX,dLastY);

	for(iPos=1;iPos<iNum;iPos++)
		{
		PosToCoord(pItems[iPos].x,pItems[iPos].y,dPosX,dPosY);

		dLen   += DistanceOfPointsEx(dLastX*0.017453292519943295769236907684886,dLastY*0.017453292519943295769236907684886,dPosX*0.017453292519943295769236907684886,dPosY*0.017453292519943295769236907684886);
		dLastX	= dPosX;
		dLastY	= dPosY;
		}


return dLen;
}


//*****************************************************************************
//*
//*		DistanceToPixel
//*
//*****************************************************************************
//	Berechenet einen Abstand in Pixel
//	dCoordX/Y	: Punkt auf derKarte von wo aus berechnet werden soll
//	dDistance	: Abstand der Berechnet werden soll
//	iMode		: Bit0=X-Richtung verwenden  Bit1=Y-Richtung verwenden
int COsmCtrl::DistanceToPixel(double dCoordX,double dCoordY,double dDistance,int iMode)
{
int		iPixelX;
int		iPixelY;
double	dDeltaX;
double	dDeltaY;
double	dDiff;



	CoordToPixel(dCoordX,dCoordY,iPixelX,iPixelY);

	switch(iMode&3)
		{
	case 3:	PixelToCoord(iPixelX+1,iPixelY+1,dDeltaX,dDeltaY);	dDistance*=0.7071;	break;
	case 2:	PixelToCoord(iPixelX  ,iPixelY+1,dDeltaX,dDeltaY);	break;
	default:PixelToCoord(iPixelX+1,iPixelY  ,dDeltaX,dDeltaY);	break;
		}

	dDiff = DistanceOfPointsEx(dCoordX*0.017453292519943295769236907684886,dCoordY*0.017453292519943295769236907684886,dDeltaX*0.017453292519943295769236907684886,dDeltaY*0.017453292519943295769236907684886);


return (int)fabs(dDistance/dDiff);
}


//*****************************************************************************
//*
//*		DistanceToPixel
//*
//*****************************************************************************
//	Berechenet einen Abstand zweier Punkte in Pixel
//	dCoordX/Y	: Erster Punkt auf der Karte von wo aus berechnet werden soll
//	dDeldaX/Y	: Zweiter auf der Karte von wo aus berechnet werden soll
//	dDistance	: Abstand der Berechnet werden soll
//	iMode		: Bit0=X-Richtung verwenden  Bit1=Y-Richtung verwenden
int COsmCtrl::DistanceToPixel(double dCoordX,double dCoordY,double dDeltaX,double dDeltaY)
{
int		iPixelX1;
int		iPixelY1;
int		iPixelX2;
int		iPixelY2;
double	dDiffX;
double	dDiffY;



	CoordToPixel(dCoordX,dCoordY,iPixelX1,iPixelY1);
	CoordToPixel(dDeltaX,dDeltaY,iPixelX2,iPixelY2);

	dDiffX = iPixelX1-iPixelX2;
	dDiffY = iPixelY1-iPixelY2;


return (int)sqrt(dDiffX*dDiffX+dDiffY*dDiffY);;
}

//*****************************************************************************
//*
//*		SetDual
//*
//*****************************************************************************
//	Alles doppelt so gro zeichnen
//	iMode		: Doppelte Gre aktivieren (0=aus 1=ein -1=wechseln
//	Ergibt 1 wenn sich etwas verndert hat
int	COsmCtrl::SetDual(int iMode)
{
int			iPosX;
int			iPosY;
UINT		uNew;


	if(iMode<0)
		{
		   uNew = (m_uWindowDual)? 0:1;
		}
	else{
		   uNew	= (iMode)? 1:0;
		if(uNew==m_uWindowDual)return 0;
		}

	m_sOsmSem.Lock();

	iPosX			= m_iOsmScrollX;
	iPosY			= m_iOsmScrollY;	

	m_uWindowDual	= uNew;									// Die Kacheln im Fenster doppelt so gro zeichnen
	m_iOsmAreaBit	= OSM_AREA_BITS+uNew;	
	m_iOsmArea		= 1<<m_iOsmAreaBit;	
	m_iOsmAreaMask	= m_iOsmArea-1;	

	m_iOsmZoomWitdh	= m_iOsmZoomSize<<m_iOsmAreaBit;		// Ist die Weltgre in Pixel
	m_iTempStepX	= m_iTempSizeX>>m_iOsmAreaBit;
	m_iTempStepY	= m_iTempSizeY>>m_iOsmAreaBit;


	#ifdef _DEBUG
	if(uNew)m_pTempStates[m_iTempStepX*m_iTempStepY]=0xFDFDFDFD;
	#endif 	

	m_sOsmSem.Unlock();

	SetScrollPos(iPosX,iPosY,TRUE);
	SetZoom(m_iOsmZoom+1-uNew*2);
	
	m_bServerChanged = TRUE;
	
	Reinit();


return 1;
}



